@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2013-2023 NVidia Corporation
+    Copyright (c) 2013-2024 NVidia Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -494,7 +494,7 @@ typedef struct
     NvU64                   base                            NV_ALIGN_BYTES(8); // IN
     NvU64                   length                          NV_ALIGN_BYTES(8); // IN
     NvU64                   offset                          NV_ALIGN_BYTES(8); // IN
-    UvmGpuMappingAttributes perGpuAttributes[UVM_MAX_GPUS_V2];                 // IN
+    UvmGpuMappingAttributes perGpuAttributes[UVM_MAX_GPUS];                    // IN
     NvU64                   gpuAttributesCount              NV_ALIGN_BYTES(8); // IN
     NvS32                   rmCtrlFd;                                          // IN
     NvU32                   hClient;                                           // IN
@@ -837,12 +837,6 @@ typedef struct
 // Initialize any tracker object such as a queue or counter
 // UvmToolsCreateEventQueue, UvmToolsCreateProcessAggregateCounters,
 // UvmToolsCreateProcessorCounters.
-// Note that the order of structure elements has the version as the last field.
-// This is used to tell whether the kernel supports V2 events or not because
-// the V1 UVM_TOOLS_INIT_EVENT_TRACKER ioctl would not read or update that
-// field but V2 will. This is needed because it is possible to create an event
-// queue before CUDA is initialized which means UvmSetDriverVersion() hasn't
-// been called yet and the kernel version is unknown.
 //
 #define UVM_TOOLS_INIT_EVENT_TRACKER                                  UVM_IOCTL_BASE(56)
 typedef struct
@@ -853,9 +847,8 @@ typedef struct
     NvProcessorUuid processor;                            // IN
     NvU32           allProcessors;                        // IN
     NvU32           uvmFd;                                // IN
+    NvU32           version;                              // IN (UvmToolsEventQueueVersion)
     NV_STATUS       rmStatus;                             // OUT
-    NvU32           requestedVersion;                     // IN
-    NvU32           grantedVersion;                       // OUT
 } UVM_TOOLS_INIT_EVENT_TRACKER_PARAMS;
 
 //
@@ -936,23 +929,15 @@ typedef struct
 
 //
 // UvmToolsGetProcessorUuidTable
-// Note that tablePtr != 0 and count == 0 means that tablePtr is assumed to be
-// an array of size UVM_MAX_PROCESSORS_V1 and that only UvmEventEntry_V1
-// processor IDs (physical GPU UUIDs) will be reported.
-// tablePtr == 0 and count == 0 can be used to query how many processors are
-// present in order to dynamically allocate the correct size array since the
-// total number of processors is returned in 'count'.
 //
 #define UVM_TOOLS_GET_PROCESSOR_UUID_TABLE                            UVM_IOCTL_BASE(64)
 typedef struct
 {
     NvU64     tablePtr                 NV_ALIGN_BYTES(8); // IN
-    NvU32     count;                                      // IN/OUT
+    NvU32     version;                                    // IN (UvmToolsEventQueueVersion)
     NV_STATUS rmStatus;                                   // OUT
-    NvU32     version;                                    // OUT
 } UVM_TOOLS_GET_PROCESSOR_UUID_TABLE_PARAMS;
 
-
 //
 // UvmMapDynamicParallelismRegion
 //
@@ -995,7 +980,7 @@ typedef struct
 {
     NvU64                   base                            NV_ALIGN_BYTES(8); // IN
     NvU64                   length                          NV_ALIGN_BYTES(8); // IN
-    UvmGpuMappingAttributes perGpuAttributes[UVM_MAX_GPUS_V2];                 // IN
+    UvmGpuMappingAttributes perGpuAttributes[UVM_MAX_GPUS];                    // IN
     NvU64                   gpuAttributesCount              NV_ALIGN_BYTES(8); // IN
     NV_STATUS               rmStatus;                                          // OUT
 } UVM_ALLOC_SEMAPHORE_POOL_PARAMS;
