@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -48,7 +48,10 @@
 namespace DisplayPort
 {
     class EvoInterface;
- 
+
+#define SET_DP_IMP_ERROR(pErrorCode, errorCode) \
+        if (pErrorCode && *pErrorCode == DP_IMP_ERROR_NONE) *pErrorCode = errorCode;
+
     typedef enum
     {
         DP_IMP_ERROR_NONE,
@@ -62,6 +65,7 @@ namespace DisplayPort
         DP_IMP_ERROR_INSUFFICIENT_BANDWIDTH,
         DP_IMP_ERROR_INSUFFICIENT_BANDWIDTH_DSC,
         DP_IMP_ERROR_INSUFFICIENT_BANDWIDTH_NO_DSC,
+        DP_IMP_ERROR_INSUFFICIENT_DP_TUNNELING_BANDWIDTH,
         DP_IMP_ERROR_WATERMARK_BLANKING,
         DP_IMP_ERROR_PPS_COLOR_FORMAT_NOT_SUPPORTED,
         DP_IMP_ERROR_PPS_INVALID_HBLANK,
@@ -271,6 +275,10 @@ namespace DisplayPort
 
         virtual DscCaps getDscCaps() = 0;
 
+        virtual NvBool isDynamicPPSSupported() = 0;
+
+        virtual NvBool isDynamicDscToggleSupported() = 0;
+
         //
         // This function returns the device itself or its parent device that is doing
         // DSC decompression for it.
@@ -318,8 +326,14 @@ namespace DisplayPort
         virtual bool isMSAOverMSTCapable() = 0;
         virtual bool isFakedMuxDevice() = 0;
         virtual bool setPanelReplayConfig(panelReplayConfig prcfg) = 0;
+        virtual bool getPanelReplayConfig(panelReplayConfig *pPrcfg) = 0;
         virtual bool isPanelReplaySupported() = 0;
         virtual bool getPanelReplayStatus(PanelReplayStatus *pPrStatus) = 0;
+        virtual bool getDeviceSpecificData(NvU8 *oui, NvU8 *deviceIdString,
+                                           NvU8 *hwRevision, NvU8 *swMajorRevision,
+                                           NvU8 *swMinorRevision) = 0;
+
+        virtual bool setModeList(DisplayPort::DpModesetParams *pModeList, unsigned numModes) = 0;
 
     protected:
             virtual ~Device() {}
@@ -402,7 +416,7 @@ namespace DisplayPort
         virtual LinkConfiguration getActiveLinkConfig() = 0;
 
         // Get Current link configuration
-        virtual void getCurrentLinkConfig(unsigned & laneCount, NvU64 & linkRate) = 0;
+        virtual void getCurrentLinkConfig(unsigned &laneCount, NvU64 &linkRate) = 0;
 
         // Get the clock calculation supported by the panel
         virtual unsigned getPanelDataClockMultiplier() = 0;
@@ -497,8 +511,8 @@ namespace DisplayPort
 
         virtual bool compoundQueryAttach(Group * target,
                                          const DpModesetParams &modesetParams,      // Modeset info
-                                         DscParams *pDscParams,				     // DSC parameters
-                                         DP_IMP_ERROR *errorStatus = NULL) = 0;     // Error Status code     
+                                         DscParams *pDscParams,                     // DSC parameters
+                                         DP_IMP_ERROR *errorStatus = NULL) = 0;     // Error Status code
 
         virtual bool endCompoundQuery() = 0;
 
@@ -591,6 +605,8 @@ namespace DisplayPort
         virtual void notifyGPUCapabilityChange() = 0;
         virtual void notifyHBR2WAREngage() = 0;
 
+        virtual bool dpUpdateDscStream(Group *target, NvU32 dscBpp) = 0;
+
         // Create a new Group.  Note that if you wish to do a modeset but send the
         // stream nowhere, you may do a modeset with an EMPTY group.  This is expected
         // to be the mechanism by which monitor faking is implemented.
@@ -653,9 +669,10 @@ namespace DisplayPort
         // Compound queries and notify attaches(link train) would use the preferred link config unless it is reset again.
         // (not advisable to leave a preferred link config always ON).
         //
-        virtual bool setPreferredLinkConfig(LinkConfiguration & lc, bool commit,
+        virtual bool setPreferredLinkConfig(LinkConfiguration &lc, bool commit,
                                             bool force = false,
-                                            LinkTrainingType forceTrainType = NORMAL_LINK_TRAINING) = 0;
+                                            LinkTrainingType forceTrainType = NORMAL_LINK_TRAINING,
+                                            bool forcePreferredLinkConfig = false) = 0;
 
         //
         // Resets the preferred link config and lets the library go back to default LT policy.
@@ -685,8 +702,8 @@ namespace DisplayPort
 
         virtual bool getHDCPAbortCodesDP12(NvU32 &hdcpAbortCodesDP12) = 0;
 
-        virtual bool getOuiSink(unsigned &ouiId, char * modelName,
-                                size_t modelNameBufferSize, NvU8 & chipRevision) = 0;
+        virtual bool getOuiSink(unsigned &ouiId, unsigned char * modelName,
+                                size_t modelNameBufferSize, NvU8 &chipRevision) = 0;
 
         virtual bool getIgnoreSourceOuiHandshake() = 0;
         virtual void setIgnoreSourceOuiHandshake(bool bIgnore) = 0;
@@ -707,6 +724,7 @@ namespace DisplayPort
         virtual bool setTestPattern(NV0073_CTRL_DP_TESTPATTERN testPattern,
                                     NvU8 laneMask, NV0073_CTRL_DP_CSTM cstm,
                                     NvBool bIsHBR2, NvBool bSkipLaneDataOverride) = 0;
+
         // "data" is an array of NV0073_CTRL_MAX_LANES unsigned ints
         virtual bool getLaneConfig(NvU32 *numLanes, NvU32 *data) = 0;
         // "data" is an array of NV0073_CTRL_MAX_LANES unsigned ints
@@ -729,9 +747,11 @@ namespace DisplayPort
     virtual bool readPsrErrorStatus(vesaPsrErrorStatus *psrErr) = 0;
     virtual bool writePsrEvtIndicator(vesaPsrEventIndicator psrErr) = 0;
     virtual bool readPsrEvtIndicator(vesaPsrEventIndicator *psrErr) = 0;
-    virtual bool updatePsrLinkState(bool bTrainLink) = 0;
+    virtual bool updatePsrLinkState(bool bTurnOnLink) = 0;
 
     virtual bool readPrSinkDebugInfo(panelReplaySinkDebugInfo *prDbgInfo) = 0;
+    virtual void enableDpTunnelingBwAllocationSupport() = 0;
+    virtual bool willLinkSupportModeSST(const LinkConfiguration &linkConfig, const ModesetInfo &modesetInfo) = 0;
 
     protected:
            virtual ~Connector() {}
