@@ -49,6 +49,9 @@
 #define    HDCP_FLAGS_ABORT_DEVICE_INVALID     0x00080000 // Abort due to an invalid device in DP1.2 topology
 #define    HDCP_FLAGS_ABORT_HOP_LIMIT_EXCEEDED 0x80000000 // Abort, number of devices in DP1.2 topology exceeds supported limit
 
+#define    DP_TUNNEL_REQUEST_BW_MAX_TIME_MS          (1000U)
+#define    DP_TUNNEL_REQUEST_BW_POLLING_INTERVAL_MS    (10U)
+
 static inline unsigned getDataClockMultiplier(NvU64 linkRate, NvU64 laneCount)
 {
     //
@@ -102,9 +105,9 @@ namespace DisplayPort
         NvU8     cachedSourceChipRevision;
         bool     bOuiCached;
 
-        unsigned ouiId;                                             // Sink ouiId
-        char modelName[NV_DPCD_SOURCE_DEV_ID_STRING__SIZE + 1];     // Device Model-name
-        bool    bIgnoreSrcOuiHandshake;                             // Skip writing source OUI
+        unsigned ouiId;                                                   // Sink ouiId
+        unsigned char modelName[NV_DPCD_SOURCE_DEV_ID_STRING__SIZE + 1];  // Device Model-name
+        bool    bIgnoreSrcOuiHandshake;                                   // Skip writing source OUI
 
         LinkPolicy    linkPolicy;
 
@@ -192,6 +195,7 @@ namespace DisplayPort
         bool compoundQueryResult;
         unsigned compoundQueryCount;
         unsigned compoundQueryLocalLinkPBN;
+        NvU64 compoundQueryUsedTunnelingBw;
         bool compoundQueryForceEnableFEC;
 
         unsigned freeSlots;
@@ -210,6 +214,7 @@ namespace DisplayPort
         // this is the link config requested by a client.
         // can be set and reset by the client for a given operation.
         LinkConfiguration preferredLinkConfig;
+        bool forcePreferredLinkConfig;
 
         //
         // Desired link configuration of single head multiple sst secondary connector.
@@ -309,10 +314,10 @@ namespace DisplayPort
         bool        bNoFallbackInPostLQA;
 
         bool        bReportDeviceLostBeforeNew;
-        bool        bEnableAudioBeyond48K;
         bool        bDisableSSC;
         bool        bEnableFastLT;
         NvU32       maxLinkRateFromRegkey;
+        bool        bFlushTimeslotWhenDirty;
 
         //
         // Latency(ms) to apply between link-train and FEC enable for bug
@@ -348,9 +353,6 @@ namespace DisplayPort
         //
         bool        bPowerDownPhyBeforeD3;
 
-        // Force DSC on sink irrespective of LT status
-        bool        bForceDscOnSink;
-
         //
         // Reset the MSTM_CTRL registers on branch device irrespective of
         // IRQ VECTOR register having stale message. Certain branch devices
@@ -360,8 +362,9 @@ namespace DisplayPort
         // the stale messages from previous discovery.
         //
         bool        bForceClearPendingMsg;
-        bool        bSkipFakeDeviceDpcdAccess;
-
+        NvU64       allocatedDpTunnelBw;
+        NvU64       allocatedDpTunnelBwShadow;
+        bool        bForceDisableTunnelBwAllocation;
 
         Group *perHeadAttachedGroup[NV_MAX_HEADS];
         NvU32 inTransitionHeadMask;
@@ -373,16 +376,16 @@ namespace DisplayPort
         void setPolicyAssessLinkSafely(bool enabled);
 
         void discoveryDetectComplete();
-        void discoveryNewDevice(const DiscoveryManager::Device & device);
-        void discoveryLostDevice(const Address & address);
-        void processNewDevice(const DiscoveryManager::Device & device,
-            const Edid & edid,
+        void discoveryNewDevice(const DiscoveryManager::Device &device);
+        void discoveryLostDevice(const Address &address);
+        void processNewDevice(const DiscoveryManager::Device &device,
+            const Edid &edid,
             bool isMultistream,
             DwnStreamPortType portType,
             DwnStreamPortAttribute portAttribute,
             bool isCompliance = false);
 
-        void applyEdidWARs(Edid & edid, DiscoveryManager::Device device);
+        void applyEdidWARs(Edid &edid, DiscoveryManager::Device device);
         void applyRegkeyOverrides(const DP_REGKEY_DATABASE& dpRegkeyDatabase);
 
         ResStatusNotifyMessage ResStatus;
@@ -444,6 +447,9 @@ namespace DisplayPort
                                          const DpModesetParams &modesetParams,      // Modeset info
                                          DscParams *pDscParams = NULL,              // DSC parameters
                                          DP_IMP_ERROR *pErrorCode = NULL);          // Error Status code
+        virtual bool compoundQueryAttachTunneling(const DpModesetParams &modesetParams,
+                                                  DscParams *pDscParams = NULL,
+                                                  DP_IMP_ERROR *pErrorCode = NULL);
 
         virtual bool endCompoundQuery();
 
@@ -495,6 +501,7 @@ namespace DisplayPort
         char tagHDCPReauthentication;
         char tagDelayedHdcpCapRead;
         char tagDelayedHDCPCPIrqHandling;
+        char tagDpBwAllocationChanged;
 
         //
         //  Enable disable TMDS mode
@@ -556,13 +563,20 @@ namespace DisplayPort
         virtual void notifyAttachEnd(bool modesetCancelled);
         virtual void notifyDetachBegin(Group * target);
         virtual void notifyDetachEnd(bool bKeepOdAlive = false);
+        virtual bool willLinkSupportModeSST(const LinkConfiguration &linkConfig, const ModesetInfo &modesetInfo);
 
         bool performIeeeOuiHandshake();
         void setIgnoreSourceOuiHandshake(bool bIgnore);
         bool getIgnoreSourceOuiHandshake();
-        bool willLinkSupportModeSST(const LinkConfiguration & linkConfig, const ModesetInfo & modesetInfo);
         void forceLinkTraining();
 
+        bool     updateDpTunnelBwAllocation();
+        TriState requestDpTunnelBw(NvU8 requestedBw);
+        bool     allocateDpTunnelBw(NvU64 bandwidth);
+        bool     allocateMaxDpTunnelBw();
+        NvU64    getMaxTunnelBw();
+        void     enableDpTunnelingBwAllocationSupport();
+
         void assessLink(LinkTrainingType trainType = NORMAL_LINK_TRAINING);
 
         bool isLinkInD3();
@@ -584,7 +598,7 @@ namespace DisplayPort
         }
         bool trainLinkOptimized(LinkConfiguration lConfig);
         bool trainLinkOptimizedSingleHeadMultipleSST(GroupImpl * group);
-        bool getValidLowestLinkConfig(LinkConfiguration & lConfig, LinkConfiguration & lowestSelected, ModesetInfo queryModesetInfo);
+        bool getValidLowestLinkConfig(LinkConfiguration &lConfig, LinkConfiguration &lowestSelected, ModesetInfo queryModesetInfo);
         bool postLTAdjustment(const LinkConfiguration &, bool force);
         void populateUpdatedLaneSettings(NvU8* voltageSwingLane, NvU8* preemphasisLane, NvU32 *data);
         void populateDscCaps(DSC_INFO* dscInfo, DeviceImpl * dev, DSC_INFO::FORCED_DSC_PARAMS* forcedParams);
@@ -594,8 +608,8 @@ namespace DisplayPort
         void populateDscBranchCaps(DSC_INFO* dscInfo, DeviceImpl * dev);
         void populateDscModesetInfo(MODESET_INFO * pModesetInfo, const DpModesetParams * pModesetParams);
 
-        bool train(const LinkConfiguration & lConfig, bool force, LinkTrainingType trainType = NORMAL_LINK_TRAINING);
-        bool validateLinkConfiguration(const LinkConfiguration & lConfig);
+        virtual bool train(const LinkConfiguration &lConfig, bool force, LinkTrainingType trainType = NORMAL_LINK_TRAINING);
+        virtual bool validateLinkConfiguration(const LinkConfiguration &lConfig);
 
         virtual bool assessPCONLinkCapability(PCONLinkControl *params);
         bool trainPCONFrlLink(PCONLinkControl *pConControl);
@@ -604,14 +618,14 @@ namespace DisplayPort
         bool setDeviceDscState(Device * dev, bool bEnableDsc);
 
         // the lowest level function(nearest to the hal) for the connector.
-        bool rawTrain(const LinkConfiguration & lConfig, bool force, LinkTrainingType linkTrainingType);
+        bool rawTrain(const LinkConfiguration &lConfig, bool force, LinkTrainingType linkTrainingType);
 
-        bool enableFlush();
-        bool beforeAddStream(GroupImpl * group, bool force=false, bool forFlushMode = false);
-        void afterAddStream(GroupImpl * group);
-        void beforeDeleteStream(GroupImpl * group, bool forFlushMode = false);
-        void afterDeleteStream(GroupImpl * group);
-        void disableFlush(bool test=false);
+        virtual bool enableFlush();
+        virtual bool beforeAddStream(GroupImpl * group, bool force=false, bool forFlushMode = false);
+        virtual void afterAddStream(GroupImpl * group);
+        virtual void beforeDeleteStream(GroupImpl * group, bool forFlushMode = false);
+        virtual void afterDeleteStream(GroupImpl * group);
+        virtual void disableFlush(bool test=false);
 
         bool beforeAddStreamMST(GroupImpl * group, bool force = false, bool forFlushMode = false);
 
@@ -619,27 +633,28 @@ namespace DisplayPort
 
         bool deleteAllVirtualChannels();
         void clearTimeslices();
-        bool allocateTimeslice(GroupImpl * targetGroup);
+        virtual bool allocateTimeslice(GroupImpl * targetGroup);
         void freeTimeslice(GroupImpl * targetGroup);
         void flushTimeslotsToHardware();
         bool getHDCPAbortCodesDP12(NvU32 &hdcpAbortCodesDP12);
-        bool getOuiSink(unsigned &ouiId, char * modelName, size_t modelNameBufferSize, NvU8 & chipRevision);
+        bool getOuiSink(unsigned &ouiId, unsigned char * modelName, size_t modelNameBufferSize, NvU8 &chipRevision);
         bool hdcpValidateKsv(const NvU8 *ksv, NvU32 Size);
         void cancelHdcpCallbacks();
         bool handleCPIRQ();
         void handleSSC();
         void handleMCCSIRQ();
+        void handleDpTunnelingIrq();
         void handleHdmiLinkStatusChanged();
         void sortActiveGroups(bool ascending);
         void configInit();
         void handlePanelReplayError();
 
-        virtual DeviceImpl* findDeviceInList(const Address & address);
+        virtual DeviceImpl* findDeviceInList(const Address &address);
         virtual void disconnectDeviceList();
         void notifyLongPulseInternal(bool statusConnected);
         virtual void notifyLongPulse(bool status);
         virtual void notifyShortPulse();
-        virtual Group * newGroup() ;
+        virtual Group * newGroup();
         virtual void destroy();
         virtual void createFakeMuxDevice(const NvU8 *buffer, NvU32 bufferSize);
         virtual void deleteFakeMuxDevice();
@@ -650,7 +665,10 @@ namespace DisplayPort
         virtual bool isFECSupported();
         virtual bool isFECCapable();
         virtual NvU32 maxLinkRateSupported();
-        virtual bool setPreferredLinkConfig(LinkConfiguration & lc, bool commit, bool force = false, LinkTrainingType trainType = NORMAL_LINK_TRAINING);
+        bool setPreferredLinkConfig(LinkConfiguration &lc, bool commit, 
+                                    bool force = false,
+                                    LinkTrainingType trainType = NORMAL_LINK_TRAINING,
+                                    bool forcePreferredLinkConfig = false);
         virtual bool resetPreferredLinkConfig(bool force = false);
         virtual void setAllowMultiStreaming(bool bAllowMST);
         virtual bool getAllowMultiStreaming(void);
@@ -664,12 +682,13 @@ namespace DisplayPort
         Group * createFirmwareGroup();
         virtual void notifyGPUCapabilityChange();
         virtual void notifyHBR2WAREngage();
+        bool dpUpdateDscStream(Group *target, NvU32 dscBpp);
 
         bool getTestPattern(NV0073_CTRL_DP_TESTPATTERN *testPattern);
         bool setTestPattern(NV0073_CTRL_DP_TESTPATTERN testPattern, NvU8 laneMask, NV0073_CTRL_DP_CSTM cstm, NvBool bIsHBR2, NvBool bSkipLaneDataOverride = false);
         bool getLaneConfig(NvU32 *numLanes, NvU32 *data);    // "data" is an array of NV0073_CTRL_MAX_LANES unsigned ints
         bool setLaneConfig(NvU32 numLanes, NvU32 *data);    // "data" is an array of NV0073_CTRL_MAX_LANES unsigned ints
-        void getCurrentLinkConfig(unsigned & laneCount, NvU64 & linkRate);  // CurrentLink Configuration
+        void getCurrentLinkConfig(unsigned &laneCount, NvU64 &linkRate);  // CurrentLink Configuration
         unsigned getPanelDataClockMultiplier();
         unsigned getGpuDataClockMultiplier();
         void configurePowerState(bool bPowerUp);
@@ -707,16 +726,16 @@ namespace DisplayPort
     //
     struct DevicePendingEDIDRead : protected EdidReadMultistream::EdidReadMultistreamEventSink, public ListElement
     {
-        EdidReadMultistream      reader;
-        DiscoveryManager::Device device;
         ConnectorImpl *          parent;
+        DiscoveryManager::Device device;
+        EdidReadMultistream      reader;
 
         void mstEdidCompleted(EdidReadMultistream * from);
         void mstEdidReadFailed(EdidReadMultistream * from);
 
     public:
         DevicePendingEDIDRead(ConnectorImpl *  _parent, MessageManager * manager, DiscoveryManager::Device dev)
-            : reader(_parent->timer, manager, this, dev.address), device(dev), parent(_parent)
+            : parent(_parent), device(dev), reader(_parent->timer, manager, this, dev.address)
         {
         }
     };
