@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2010-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2010-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -36,6 +36,7 @@
 #include "ctrl/ctrl0073/ctrl0073specific.h" // NV0073_CTRL_HDCP_VPRIME_SIZE
 #include "displayport.h"
 
+#define NV_SUPPORTED_DP_LINK_RATES__SIZE NV_SUPPORTED_DP1X_LINK_RATES__SIZE
 namespace DisplayPort
 {
     typedef NvU64 LinkRate;
@@ -44,58 +45,36 @@ namespace DisplayPort
     {
     public:
         NvU8 entries;
+        // Store link rate in multipler of 10MBPS to save space
+        NvU16 element[NV_SUPPORTED_DP_LINK_RATES__SIZE];
 
-        virtual void clear() = 0;
-        virtual bool import(NvU8 linkBw)
-        {
-            DP_ASSERT(0);
-            return false;
-        }
-
-        virtual LinkRate getLowerRate(LinkRate rate) = 0;
-        virtual LinkRate getMaxRate() = 0;
-        virtual NvU8 getNumElements() = 0;
-
-        NvU8 getNumLinkRates()
-        {
-            return entries;
-        }
-
-    };
-
-    class LinkRates1x : virtual public LinkRates
-    {
-    public:
-        // Store link rate in multipler of 270MBPS to save space
-        NvU8 element[NV_SUPPORTED_DP1X_LINK_RATES__SIZE];
-
-        LinkRates1x()
+        LinkRates()
         {
             entries = 0;
-            for (int i = 0; i < NV_SUPPORTED_DP1X_LINK_RATES__SIZE; i++)
+            for (int i = 0; i < NV_SUPPORTED_DP_LINK_RATES__SIZE; i++)
             {
                 element[i] = 0;
             }
         }
 
-        virtual void clear()
+        void clear()
         {
             entries = 0;
-            for (int i = 0; i < NV_SUPPORTED_DP1X_LINK_RATES__SIZE; i++)
+            for (int i = 0; i < NV_SUPPORTED_DP_LINK_RATES__SIZE; i++)
             {
                 element[i] = 0;
             }
         }
 
-        virtual bool import(NvU8 linkBw)
+        bool import(NvU16 linkBw)
         {
-            if (!IS_VALID_LINKBW(linkBw))
+            if (!IS_VALID_LINKBW_10M(linkBw))
             {
                 DP_ASSERT(0 && "Unsupported Link Bandwidth");
                 return false;
             }
 
-            if (entries < NV_SUPPORTED_DP1X_LINK_RATES__SIZE)
+            if (entries < NV_SUPPORTED_DP_LINK_RATES__SIZE)
             {
                 element[entries] = linkBw;
                 entries++;
@@ -105,47 +84,49 @@ namespace DisplayPort
                 return false;
         }
 
-        virtual LinkRate getLowerRate(LinkRate rate)
+        LinkRate getLowerRate(LinkRate rate)
         {
             int i;
-            NvU8 linkBw = (NvU8)(rate / DP_LINK_BW_FREQ_MULTI_MBPS);
 
-            if ((entries == 0) || (linkBw <= element[0]))
+            if ((entries == 0) || (rate <= element[0]))
                 return 0;
 
             for (i = entries - 1; i > 0; i--)
             {
-                if (linkBw > element[i])
+                if (rate > element[i])
                     break;
             }
 
-            rate = (LinkRate)element[i] * DP_LINK_BW_FREQ_MULTI_MBPS;
-            return rate;
+            return ((LinkRate)element[i]);
         }
 
-        virtual LinkRate getMaxRate()
+        LinkRate getMaxRate()
         {
             LinkRate rate = 0;
             if ((entries > 0) &&
-                (entries <= NV_SUPPORTED_DP1X_LINK_RATES__SIZE))
+                (entries <= NV_SUPPORTED_DP_LINK_RATES__SIZE))
             {
-                rate = (LinkRate)element[entries - 1] * DP_LINK_BW_FREQ_MULTI_MBPS;
+                rate = (LinkRate)element[entries - 1];
             }
-
             return rate;
         }
-        virtual NvU8 getNumElements()
+
+        NvU8 getNumElements()
         {
-            return NV_SUPPORTED_DP1X_LINK_RATES__SIZE;
+            return NV_SUPPORTED_DP_LINK_RATES__SIZE;
         }
 
+        NvU8 getNumLinkRates()
+        {
+            return entries;
+        }
     };
-
     class LinkPolicy : virtual public Object
     {
     protected:
         bool            bNoFallback;                // No fallback when LT fails
-        LinkRates1x     linkRates;
+        LinkRates       linkRates;
+
     public:
         LinkPolicy() : bNoFallback(false)
         {
@@ -242,7 +223,6 @@ namespace DisplayPort
         bool     disablePostLTRequest;
         bool     bEnableFEC;
         bool     bDisableLTTPR;
-
         //
         // The counter to record how many times link training happens.
         // Client can reset the counter by calling setLTCounter(0)
@@ -252,7 +232,8 @@ namespace DisplayPort
         LinkConfiguration() :
             lanes(0), peakRatePossible(0), peakRate(0), minRate(0),
             enhancedFraming(false), multistream(false), disablePostLTRequest(false),
-            bEnableFEC(false), bDisableLTTPR(false), linkTrainCounter(0) {};
+            bEnableFEC(false), bDisableLTTPR(false),
+            linkTrainCounter(0) {};
 
         LinkConfiguration(LinkPolicy * p, unsigned lanes, LinkRate peakRate,
             bool enhancedFraming, bool MST, bool disablePostLTRequest = false,
@@ -281,8 +262,40 @@ namespace DisplayPort
             return linkTrainCounter;
         }
 
+        // Returns data rate in Bytes per second
+        NvU64 convertLinkRateToDataRate(LinkRate linkRate) const
+        {
+            NvU64 dataRate;
+            dataRate = LINK_RATE_TO_DATA_RATE_8B_10B(linkRate);
+            return dataRate;
+        }
+
+        // Returns minRate in data rate in Bytes per second
+        NvU64 convertMinRateToDataRate() const
+        {
+            NvU64 dataRate;
+            dataRate = DP_LINK_RATE_BITSPS_TO_BYTESPS(OVERHEAD_8B_10B(minRate));
+            return dataRate;
+        }
+
+        NvU64 getTotalDataRate() const
+        {
+            return (convertLinkRateToDataRate(peakRate) * lanes);
+        }
+
         NvU64 linkOverhead(NvU64 rate)
         {
+            if(IS_VALID_LINKBW_10M(rate))
+            {
+                // Converting here so that minRate from 10M is converted to bps
+                rate = DP_LINK_RATE_10M_TO_BPS(rate);
+            }
+            else
+            {
+                // Convert from data rate to bps
+                rate = DATA_RATE_8B_10B_TO_LINK_RATE_BPS(rate);
+            }
+
             if(bEnableFEC)
             {
 
@@ -294,8 +307,8 @@ namespace DisplayPort
             else
             {
                 // if FEC is not enabled, link overhead comprises only of
-                // 0.05% downspread.
-                return rate - 5 * rate/ 1000;
+                // 0.6% downspread.
+                return rate - 6 * rate/ 1000;
 
             }
         }
@@ -303,7 +316,6 @@ namespace DisplayPort
         void enableFEC(bool setFEC)
         {
             bEnableFEC = setFEC;
-
             // If FEC is enabled, update minRate with FEC+downspread overhead.
             minRate = linkOverhead(peakRate);
         }
@@ -316,44 +328,130 @@ namespace DisplayPort
               bDisableLTTPR(false),
               linkTrainCounter(0)
         {
+            //
             // Reverse engineer a link configuration from Total TotalLinkPBN
             // Note that HBR2 twice HBR. The table below treats HBR2x1 and HBRx2, etc.
-
             //
-            //    BW     Effective Lanes    Total TotalLinkPBN
-            //    165    1                  195.5555556
-            //    165    2                  391.1111111
-            //    165    4                  782.2222222
-            //    270    1                  320
-            //    270    2                  640
-            //    270    4                  1280
-            //    270    8                  2560
+            // PBN Calculation
+            // Definition of PBN is "54/64 MBps".
+            // Note this is the "data" actually transmitted in the main link.
+            // So we need to take channel coding into consideration.
+            // Formula: PBN = Lane Count * Link Rate (Gbps) * 1000 * (1/8) * ChannelCoding Efficiency * (64 / 54)
+            // Example:
+            // 1. 4 * HBR2:    4 * 5.4 * 1000 * (1/8) * (8/10) * (64/54) = 2560
+            // 2. 2 * UHBR10:  2 * 10 * 1000 * (1/8) * (128/132) * (64/54) = 2873
+            //
+            // Full list:
+            //
+            //   BW (Gbps)        Lanes              TotalLinkPBN
+            //     1.62             1                     192
+            //     1.62             2                     384
+            //     1.62             4                     768
+            //     2.70             1                     320
+            //     2.70             2                     640
+            //     2.70             4                    1280
+            //     5.40             1                     640
+            //     5.40             2                    1280
+            //     5.40             4                    2560
+            //     8.10             1                     960
+            //     8.10             2                    1920
+            //     8.10             4                    3840
+            //    10.00             1                    1436
+            //    10.00             2                    2873
+            //    10.00             4                    5746
+            //    13.50             1                    1939
+            //    13.50             2                    3878
+            //    13.50             4                    7757
+            //    20.00             1                    2873
+            //    20.00             2                    5746
+            //    20.00             4                   11492
             //
 
             if (TotalLinkPBN <= 90)
-                peakRatePossible = peakRate = RBR, minRate = linkOverhead(RBR), lanes=0; // FAIL
-            if (TotalLinkPBN <= 195)
-                peakRatePossible = peakRate = RBR, minRate = linkOverhead(RBR), lanes=1;
+            {
+                peakRatePossible = dp2LinkRate_1_62Gbps;
+                peakRate = peakRatePossible;
+                minRate = linkOverhead(dp2LinkRate_1_62Gbps);
+                lanes = 0; // FAIL
+            }
+            if (TotalLinkPBN <= 192)
+            {
+                peakRatePossible = dp2LinkRate_1_62Gbps;
+                peakRate = peakRatePossible;
+                minRate = linkOverhead(dp2LinkRate_1_62Gbps);
+                lanes = 1;
+            }
             else if (TotalLinkPBN <= 320)
-                peakRatePossible = peakRate = HBR, minRate=linkOverhead(HBR), lanes = 1;
-            else if (TotalLinkPBN <= 391)
-                peakRatePossible = peakRate = RBR, minRate=linkOverhead(RBR), lanes = 2;
+            {
+                peakRatePossible = dp2LinkRate_2_70Gbps;
+                peakRate = peakRatePossible;
+                minRate = linkOverhead(dp2LinkRate_2_70Gbps);
+                lanes = 1;
+            }
+            else if (TotalLinkPBN <= 384)
+            {
+                peakRatePossible = dp2LinkRate_1_62Gbps;
+                peakRate = peakRatePossible;
+                minRate = linkOverhead(dp2LinkRate_1_62Gbps);
+                lanes = 2;
+            }
             else if (TotalLinkPBN <= 640)
-                peakRatePossible = peakRate = HBR, minRate=linkOverhead(HBR), lanes = 2;   // could be HBR2x1, but TotalLinkPBN works out same
-            else if (TotalLinkPBN <= 782)
-                peakRatePossible = peakRate = RBR, minRate=linkOverhead(RBR), lanes = 4;
+            {
+                // could be HBR2 x 1, but TotalLinkPBN works out same
+                peakRatePossible = dp2LinkRate_2_70Gbps;
+                peakRate = peakRatePossible;
+                minRate = linkOverhead(dp2LinkRate_2_70Gbps);
+                lanes = 2;
+            }
+            else if (TotalLinkPBN <= 768)
+            {
+                peakRatePossible = dp2LinkRate_1_62Gbps;
+                peakRate = peakRatePossible;
+                minRate = linkOverhead(dp2LinkRate_1_62Gbps);
+                lanes = 4;
+            }
             else if (TotalLinkPBN <= 960)
-                peakRatePossible = peakRate = HBR3, minRate=linkOverhead(HBR3), lanes = 1;
+            {
+                peakRatePossible = dp2LinkRate_8_10Gbps;
+                peakRate = peakRatePossible;
+                minRate = linkOverhead(dp2LinkRate_8_10Gbps);
+                lanes = 1;
+            }
             else if (TotalLinkPBN <= 1280)
-                peakRatePossible = peakRate = HBR, minRate=linkOverhead(HBR), lanes = 4;   // could be HBR2x2
+            {
+                // could be HBR2 x 2
+                peakRatePossible = dp2LinkRate_2_70Gbps;
+                peakRate = peakRatePossible;
+                minRate = linkOverhead(dp2LinkRate_2_70Gbps);
+                lanes = 4;
+            }
             else if (TotalLinkPBN <= 1920)
-                peakRatePossible = peakRate = HBR3, minRate=linkOverhead(HBR3), lanes = 2;   // could be HBR2x
+            {
+                peakRatePossible = dp2LinkRate_8_10Gbps;
+                peakRate = peakRatePossible;
+                minRate = linkOverhead(dp2LinkRate_8_10Gbps);
+                lanes = 2;
+            }
             else if (TotalLinkPBN <= 2560)
-                peakRatePossible = peakRate = HBR2, minRate=linkOverhead(HBR2), lanes = 4;
+            {
+                peakRatePossible = dp2LinkRate_5_40Gbps;
+                peakRate = peakRatePossible;
+                minRate = linkOverhead(dp2LinkRate_5_40Gbps);
+                lanes = 4;
+            }
             else if (TotalLinkPBN <= 3840)
-                peakRatePossible = peakRate = HBR3, minRate=linkOverhead(HBR3), lanes = 4;
-            else {
-                peakRatePossible = peakRate = RBR, minRate = linkOverhead(RBR), lanes = 0; // FAIL
+            {
+                peakRatePossible = dp2LinkRate_8_10Gbps;
+                peakRate = peakRatePossible;
+                minRate = linkOverhead(dp2LinkRate_8_10Gbps);
+                lanes = 4;
+            }
+            else
+            {
+                peakRatePossible = dp2LinkRate_1_62Gbps;
+                peakRate = peakRatePossible;
+                minRate = linkOverhead(dp2LinkRate_1_62Gbps);
+                lanes = 0; // FAIL
                 DP_ASSERT(0 && "Unknown configuration");
             }
         }
@@ -412,7 +510,6 @@ namespace DisplayPort
             }
 
             minRate = linkOverhead(peakRate);
-
             return lanes != laneCount_0;
         }
 
@@ -443,7 +540,7 @@ namespace DisplayPort
         NvU32 slotsForPBN(NvU32 allocatedPBN, bool usable = false)
         {
             NvU64 bytes_per_pbn      = 54 * 1000000 / 64;     // this comes out exact
-            NvU64 bytes_per_timeslot = peakRate * lanes / 64;
+            NvU64 bytes_per_timeslot = getTotalDataRate() / 64;
 
             if (bytes_per_timeslot == 0)
                 return (NvU32)-1;
@@ -463,7 +560,7 @@ namespace DisplayPort
         NvU32 PBNForSlots(NvU32 slots)                      // Rounded down
         {
             NvU64 bytes_per_pbn      = 54 * 1000000 / 64;     // this comes out exact
-            NvU64 bytes_per_timeslot = peakRate * lanes / 64;
+            NvU64 bytes_per_timeslot = getTotalDataRate() / 64;
 
             return (NvU32)(bytes_per_timeslot * slots/ bytes_per_pbn);
         }
@@ -484,8 +581,8 @@ namespace DisplayPort
 
         bool operator< (const LinkConfiguration & right) const
         {
-            NvU64 leftMKBps = peakRate * lanes;
-            NvU64 rightMKBps = right.peakRate * right.lanes;
+            NvU64 leftMKBps = getTotalDataRate();
+            NvU64 rightMKBps = right.getTotalDataRate();
 
             if (leftMKBps == rightMKBps)
             {
@@ -497,5 +594,6 @@ namespace DisplayPort
             }
         }
     };
+
 }
 #endif //INCLUDED_DP_LINKCONFIG_H
