@@ -46,6 +46,7 @@ typedef struct
     NV_DECLARE_ALIGNED(NvU32 lineNumber, 8);
     NvU8 argumentCount; //! Count of arguments not including format string.
     NvU8 printLevel;
+    NvU8 specialType;
 } libosLogMetadata;
 
 /**
@@ -272,8 +273,12 @@ void LibosLogTokens(const libosLogMetadata * metadata, const LibosPrintfArgument
  * The only exception is renaming headers with prints inside static inline functions; since these don't overlap
  * with basenames, and we can't reliably get a nonvolatile version of a header file name here, such name changes
  * alone won't be able to trigger ucode releases.
+ *
+ * Section .logging_const contains only strings. We set SHF_MERGE and SHF_STRINGS ELF section
+ * flags with entsize=1 so that strings get merged by LD. Comment character # works around GCC
+ * appending its own default flags.
  */
-#    define LIBOS_SECTION_LOGGING_CONST    __attribute__((section(".logging_const")))
+#    define LIBOS_SECTION_LOGGING_CONST    __attribute__((section(".logging_const, \"MSa\", @progbits, 1 #")))
 #    define LIBOS_SECTION_LOGGING_METADATA __attribute__((section(".logging_metadata")))
 #    define LIBOS_LOGGING_AUX_METADATA_DUMP \
     static const LIBOS_SECTION_LOGGING_CONST int  libos_dummy_line[] LIBOS_ATTR_USED = {__LINE__};
@@ -291,24 +296,31 @@ void LibosLogTokens(const libosLogMetadata * metadata, const LibosPrintfArgument
 /*!
  *  Cast remaining log arguments to integers for storage
  */
-#define LIBOS_LOG_INTERNAL(dispatcher, level, ...)                                                             \
+#define LIBOS_LOG_INTERNAL_SPECIAL(dispatcher, level, special, ...)                                            \
     do                                                                                                         \
     {                                                                                                          \
         static const LIBOS_SECTION_LOGGING_CONST char libos_pvt_format[]   = {LIBOS_MACRO_FIRST(__VA_ARGS__)}; \
         static const LIBOS_SECTION_LOGGING_CONST char libos_pvt_file[]     = {__FILE__};                       \
-        if (0)                                                                                                 \
-            LIBOS_CHECK_PRINTF_FMT(__VA_ARGS__);                                                               \
         LIBOS_LOGGING_AUX_METADATA_DUMP;                                                                       \
         static const LIBOS_SECTION_LOGGING_METADATA libosLogMetadata libos_pvt_meta = {                        \
             .filename      = &libos_pvt_file[0],                                                               \
             .format        = &libos_pvt_format[0],                                                             \
             .lineNumber    = __LINE__,                                                                         \
             .argumentCount = LIBOS_MACRO_GET_COUNT(__VA_ARGS__) - 1,                                           \
-            .printLevel    = level};                                                                           \
+            .printLevel    = level,                                                                            \
+            .specialType   = special};                                                                         \
         const NvU64 tokens[] = {APPLY_REMAINDER(__VA_ARGS__)(NvU64) & libos_pvt_meta};                         \
         dispatcher(sizeof(tokens) / sizeof(*tokens), &tokens[0]);                                              \
     } while (0)
 
+#define LIBOS_LOG_INTERNAL(dispatcher, level, ...)                                                             \
+    do                                                                                                         \
+    {                                                                                                          \
+        if (0)                                                                                                 \
+            LIBOS_CHECK_PRINTF_FMT(__VA_ARGS__);                                                               \
+        LIBOS_LOG_INTERNAL_SPECIAL(dispatcher, level, 0 /* specialType */, __VA_ARGS__);                       \
+    } while (0)
+
 /*!
  *  Cast remaining log arguments to integers for storage
  *  This macro is used when logging for other task's address
