@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2019-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2019-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -81,13 +81,14 @@
 
 #include <gpu/mem_mgr/mem_mgr.h>
 #include <gpu/kern_gpu_power.h>
+#include <gpu_mgr/gpu_mgr.h>
 #include <core/locks.h>
 #include "kernel/gpu/intr/intr.h"
 
 #include <gpu/mem_sys/kern_mem_sys.h>
 #include <gpu/subdevice/subdevice.h>
 #include <ctrl/ctrl2080/ctrl2080unix.h>
-#include <objtmr.h>
+#include <gpu/timer/objtmr.h>
 
 //
 // Schedule timer based callback, to check for the complete GPU Idleness.
@@ -563,135 +564,43 @@ subdeviceCtrlCmdOsUnixAudioDynamicPower_IMPL
     return NV_OK;
 }
 
-/*!
- * @brief: Function to indicate if Video Memory is powered off or not by
- * checking if GPU is in GCOFF state.
- *
- * @param[in]   sp     nvidia_stack_t pointer.
- * @param[in]   pNv    nv_state_t pointer.
- *
- * @return      String indicating Video Memory power status.
- */
-
-const char* NV_API_CALL rm_get_vidmem_power_status(
-    nvidia_stack_t *sp,
-    nv_state_t     *pNv
-)
-{
-    THREAD_STATE_NODE threadState;
-    void              *fp;
-    GPU_MASK          gpuMask;
-    const char        *pVidmemPowerStatus = "?";
-
-    NV_ENTER_RM_RUNTIME(sp,fp);
-    threadStateInit(&threadState, THREAD_STATE_FLAGS_NONE);
-
-    // LOCK: acquire API lock
-    if ((rmapiLockAcquire(API_LOCK_FLAGS_NONE, RM_LOCK_MODULES_DYN_POWER)) == NV_OK)
-    {
-        OBJGPU *pGpu = NV_GET_NV_PRIV_PGPU(pNv);
-
-        // LOCK: acquire per device lock
-        if ((pGpu != NULL) &&
-           ((rmGpuGroupLockAcquire(pGpu->gpuInstance, GPU_LOCK_GRP_SUBDEVICE,
-                                   GPUS_LOCK_FLAGS_NONE, RM_LOCK_MODULES_DYN_POWER,
-                                   &gpuMask)) == NV_OK))
-        {
-            if (pGpu->getProperty(pGpu, PDB_PROP_GPU_GCOFF_STATE_ENTERED))
-            {
-                pVidmemPowerStatus = "Off";
-            }
-            else
-            {
-                pVidmemPowerStatus = "Active";
-            }
-
-            // UNLOCK: release per device lock
-            rmGpuGroupLockRelease(gpuMask, GPUS_LOCK_FLAGS_NONE);
-        }
-
-        // UNLOCK: release API lock
-        rmapiLockRelease();
-    }
-
-    threadStateFree(&threadState, THREAD_STATE_FLAGS_NONE);
-    NV_EXIT_RM_RUNTIME(sp,fp);
-
-    return pVidmemPowerStatus;
-}
-
 /*!
  * @brief: Function to indicate if GC6/GC-OFF is supported
  * or not on the SKU.
  *
- * @param[in]   sp             nvidia_stack_t pointer.
- * @param[in]   pNv            nv_state_t pointer.
+ * @param[in]   pGpu           OBJGPU pointer.
  * @param[in]   bGcxTypeGc6    If true, returns string indicating GC6 support
  *                             otherwise returns GC-OFF support.
  *
  * @return      String indicating GC6/GC-OFF support status.
  */
-const char* NV_API_CALL rm_get_gpu_gcx_support(
-    nvidia_stack_t *sp,
-    nv_state_t     *pNv,
-    NvBool         bGcxTypeGC6
+static const char* RmGetGpuGcxSupport(
+    OBJGPU *pGpu,
+    NvBool  bGcxTypeGC6
 )
 {
-    THREAD_STATE_NODE threadState;
-    void              *fp;
-    GPU_MASK          gpuMask;
-    const char        *pSupported = "?";
-
-    NV_ENTER_RM_RUNTIME(sp,fp);
-    threadStateInit(&threadState, THREAD_STATE_FLAGS_NONE);
-
-    // LOCK: acquire API lock
-    if ((rmapiLockAcquire(API_LOCK_FLAGS_NONE, RM_LOCK_MODULES_DYN_POWER)) == NV_OK)
+    if (!pGpu->getProperty(pGpu, PDB_PROP_GPU_UNIX_DYNAMIC_POWER_SUPPORTED))
     {
-        OBJGPU *pGpu = NV_GET_NV_PRIV_PGPU(pNv);
-
-        // LOCK: acquire per device lock
-        if ((pGpu != NULL) &&
-            ((rmGpuGroupLockAcquire(pGpu->gpuInstance, GPU_LOCK_GRP_SUBDEVICE,
-                                    GPUS_LOCK_FLAGS_NONE, RM_LOCK_MODULES_DYN_POWER,
-                                    &gpuMask)) == NV_OK))
-        {
-            pSupported = "Not Supported";
-
-            if (!pGpu->getProperty(pGpu, PDB_PROP_GPU_UNIX_DYNAMIC_POWER_SUPPORTED))
-            {
-                goto done;
-            }
-
-            if (bGcxTypeGC6)
-            {
-                if (pGpu->getProperty(pGpu, PDB_PROP_GPU_RTD3_GC6_SUPPORTED))
-                {
-                    pSupported = "Supported";
-                }
-            }
-            else
-            {
-                if ((pGpu->getProperty(pGpu, PDB_PROP_GPU_RTD3_GCOFF_SUPPORTED)) ||
-                    (pGpu->getProperty(pGpu, PDB_PROP_GPU_LEGACY_GCOFF_SUPPORTED)))
-                {
-                    pSupported = "Supported";
-                }
-            }
-
-done:
-            // UNLOCK: release per device lock
-            rmGpuGroupLockRelease(gpuMask, GPUS_LOCK_FLAGS_NONE);
-        }
-
-        //UNLOCK: release API lock
-        rmapiLockRelease();
+        return "Not Supported";
     }
 
-    threadStateFree(&threadState, THREAD_STATE_FLAGS_NONE);
-    NV_EXIT_RM_RUNTIME(sp,fp);
+    if (bGcxTypeGC6)
+    {
+        if (pGpu->getProperty(pGpu, PDB_PROP_GPU_RTD3_GC6_SUPPORTED))
+        {
+            return "Supported";
+        }
+    }
+    else
+    {
+        if ((pGpu->getProperty(pGpu, PDB_PROP_GPU_RTD3_GCOFF_SUPPORTED)) ||
+            (pGpu->getProperty(pGpu, PDB_PROP_GPU_LEGACY_GCOFF_SUPPORTED)))
+        {
+            return "Supported";
+        }
+    }
 
-    return pSupported;
+    return "Not Supported";
 }
 
 /*!
@@ -823,7 +732,7 @@ rmReadAndParseDynamicPowerRegkey
         return NV_OK;
     }
 
-    chipId = DRF_VAL(_PMC, _BOOT_42, _CHIP_ID, pNvp->pmc_boot_42);
+    chipId = gpuGetChipIdFromPmcBoot42(pNvp->pmc_boot_42);
 
     // From GA102+, we enable RTD3 only if system is found to be Notebook
     if ((chipId >= NV_PMC_BOOT_42_CHIP_ID_GA102) &&
@@ -1129,8 +1038,8 @@ os_ref_dynamic_power(
         {
         default:
         case NV_DYNAMIC_POWER_STATE_IN_USE:
-            NV_PRINTF(LEVEL_ERROR, "NVRM: %s: Unexpected dynamic power state 0x%x\n",
-                      __FUNCTION__, old_state);
+            NV_PRINTF(LEVEL_ERROR, "Unexpected dynamic power state 0x%x\n",
+                      old_state);
             /* fallthrough */
         case NV_DYNAMIC_POWER_STATE_IDLE_INDICATED:
             status = nv_indicate_not_idle(nv);
@@ -1349,7 +1258,7 @@ static NvBool RmCheckForGcxSupportOnCurrentState(
  * finished.
  *
  * Queue with lock flags:
- *     OS_QUEUE_WORKITEM_FLAGS_LOCK_GPU_GROUP_SUBDEVICE_RW
+ *     OS_QUEUE_WORKITEM_FLAGS_LOCK_GPU_GROUP_SUBDEVICE
  *
  * @param[in]   gpuInstance     GPU instance ID.
  * @param[in]   pArgs           Unused callback closure.
@@ -1388,12 +1297,20 @@ static void timerCallbackToRemoveIdleHoldoff(
     void *pCallbackData
 )
 {
-    OBJGPU *pGpu   = reinterpretCast(pCallbackData, OBJGPU *);
+    NV_STATUS  status = NV_OK;
+    OBJGPU    *pGpu   = reinterpretCast(pCallbackData, OBJGPU *);
 
-    osQueueWorkItemWithFlags(pGpu,
-                             RmRemoveIdleHoldoff,
-                             NULL,
-                             OS_QUEUE_WORKITEM_FLAGS_LOCK_GPU_GROUP_SUBDEVICE_RW);
+    status = osQueueWorkItemWithFlags(pGpu,
+                                      RmRemoveIdleHoldoff,
+                                      NULL,
+                                      OS_QUEUE_WORKITEM_FLAGS_LOCK_GPU_GROUP_SUBDEVICE);
+
+    if (status != NV_OK)
+    {
+        NV_PRINTF(LEVEL_ERROR,
+                  "Queuing of remove idle holdoff work item failed with status : 0x%x\n",
+                  status);
+    }
 }
 
 /*!
@@ -1482,8 +1399,8 @@ static void timerCallbackForIdlePreConditions(
                 NV_ASSERT(0);
                 /* fallthrough */
             case NV_DYNAMIC_POWER_STATE_IDLE_INDICATED:
-                NV_PRINTF(LEVEL_ERROR, "NVRM: %s: unexpected dynamic power state 0x%x\n",
-                          __FUNCTION__, nvp->dynamic_power.state);
+                NV_PRINTF(LEVEL_ERROR, "unexpected dynamic power state 0x%x\n",
+                          nvp->dynamic_power.state);
                 /* fallthrough */
             case NV_DYNAMIC_POWER_STATE_IN_USE:
                 break;
@@ -1740,7 +1657,7 @@ static NV_STATUS CreateDynamicPowerCallbacks(
     if (status != NV_OK)
     {
         NV_PRINTF(LEVEL_ERROR,
-                  "NVRM: Error creating dynamic power precondition check callback\n");
+                  "Error creating dynamic power precondition check callback\n");
         nvp->dynamic_power.idle_precondition_check_event = NvP64_NULL;
         nvp->dynamic_power.indicate_idle_event = NvP64_NULL;
         nvp->dynamic_power.remove_idle_holdoff = NvP64_NULL;
@@ -1760,7 +1677,7 @@ static NV_STATUS CreateDynamicPowerCallbacks(
     if (status != NV_OK)
     {
         NV_PRINTF(LEVEL_ERROR,
-                  "NVRM: Error creating callback to indicate GPU idle\n");
+                  "Error creating callback to indicate GPU idle\n");
         nvp->dynamic_power.idle_precondition_check_event = NvP64_NULL;
         nvp->dynamic_power.indicate_idle_event = NvP64_NULL;
         nvp->dynamic_power.remove_idle_holdoff = NvP64_NULL;
@@ -1780,7 +1697,7 @@ static NV_STATUS CreateDynamicPowerCallbacks(
     if (status != NV_OK)
     {
         NV_PRINTF(LEVEL_ERROR,
-                  "NVRM: Error creating callback to decrease kernel refcount\n");
+                  "Error creating callback to decrease kernel refcount\n");
         nvp->dynamic_power.idle_precondition_check_event = NvP64_NULL;
         nvp->dynamic_power.indicate_idle_event = NvP64_NULL;
         nvp->dynamic_power.remove_idle_holdoff = NvP64_NULL;
@@ -1831,7 +1748,7 @@ static void RmScheduleCallbackForIdlePreConditionsUnderGpuLock(
         }
         else
         {
-            NV_PRINTF(LEVEL_ERROR, "NVRM: Error scheduling precondition callback\n");
+            NV_PRINTF(LEVEL_ERROR, "Error scheduling precondition callback\n");
         }
     }
 }
@@ -1893,7 +1810,7 @@ static void RmScheduleCallbackToIndicateIdle(
         status = tmrCtrlCmdEventSchedule(pGpu, &scheduleEventParams);
 
         if (status != NV_OK)
-            NV_PRINTF(LEVEL_ERROR, "NVRM: Error scheduling indicate idle callback\n");
+            NV_PRINTF(LEVEL_ERROR, "Error scheduling indicate idle callback\n");
     }
 }
 
@@ -1928,7 +1845,7 @@ static void RmScheduleCallbackToRemoveIdleHoldoff(
         if (status != NV_OK)
         {
             NV_PRINTF(LEVEL_ERROR,
-                      "NVRM: Error scheduling kernel refcount decrement callback\n");
+                      "Error scheduling kernel refcount decrement callback\n");
         }
         else
         {
@@ -1943,7 +1860,9 @@ static void RmScheduleCallbackToRemoveIdleHoldoff(
  * @param[in]   pGpu    OBJGPU pointer.
  */
 static NvBool RmCheckRtd3GcxSupport(
-    nv_state_t *pNv
+    nv_state_t *pNv,
+    NvBool     *bGC6Support,
+    NvBool     *bGCOFFSupport
 )
 {
     nv_priv_t *nvp = NV_GET_NV_PRIV(pNv);
@@ -1951,31 +1870,21 @@ static NvBool RmCheckRtd3GcxSupport(
     RM_API    *pRmApi = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
     NV_STATUS  status;
     NV0080_CTRL_GPU_GET_VIRTUALIZATION_MODE_PARAMS virtModeParams = { 0 };
-    NvBool     bGC6Support   = NV_FALSE;
-    NvBool     bGCOFFSupport = NV_FALSE;
-
     if (!pGpu->getProperty(pGpu, PDB_PROP_GPU_UNIX_DYNAMIC_POWER_SUPPORTED))
     {
-        NV_PRINTF(LEVEL_NOTICE, "NVRM: RTD3/GC6 is not supported for this arch\n");
+        NV_PRINTF(LEVEL_NOTICE, "RTD3 is not supported for this arch\n");
         return NV_FALSE;
     }
 
-    if (nvp->b_mobile_config_enabled)
-    {
-        bGC6Support = pGpu->getProperty(pGpu, PDB_PROP_GPU_RTD3_GC6_SUPPORTED);
-        bGCOFFSupport = bGC6Support;
-    }
-    else
-    {
-        bGC6Support = pGpu->getProperty(pGpu, PDB_PROP_GPU_RTD3_GC6_SUPPORTED);
-        bGCOFFSupport = pGpu->getProperty(pGpu, PDB_PROP_GPU_RTD3_GCOFF_SUPPORTED);
-    }
+    *bGC6Support = pGpu->getProperty(pGpu, PDB_PROP_GPU_RTD3_GC6_SUPPORTED);
+    *bGCOFFSupport = nvp->b_mobile_config_enabled ? *bGC6Support :
+                     pGpu->getProperty(pGpu, PDB_PROP_GPU_RTD3_GCOFF_SUPPORTED);
 
-    if (!bGC6Support && !bGCOFFSupport)
+    if (!(*bGC6Support) && !(*bGCOFFSupport))
     {
         NV_PRINTF(LEVEL_NOTICE,
-                  "NVRM: Disabling RTD3. [GC6 support=%d GCOFF support=%d]\n",
-                  bGC6Support, bGCOFFSupport);
+                  "Disabling RTD3. [GC6 support=%d GCOFF support=%d]\n",
+                  *bGC6Support, *bGCOFFSupport);
         return NV_FALSE;
     }
 
@@ -1986,7 +1895,7 @@ static NvBool RmCheckRtd3GcxSupport(
     if (status != NV_OK)
     {
         NV_PRINTF(LEVEL_ERROR,
-                  "NVRM: Failed to get Virtualization mode, status=0x%x\n",
+                  "Failed to get Virtualization mode, status=0x%x\n",
                   status);
         return NV_FALSE;
     }
@@ -1994,7 +1903,7 @@ static NvBool RmCheckRtd3GcxSupport(
     if ((virtModeParams.virtualizationMode != NV0080_CTRL_GPU_VIRTUALIZATION_MODE_NONE) &&
         (virtModeParams.virtualizationMode != NV0080_CTRL_GPU_VIRTUALIZATION_MODE_NMOS))
     {
-        NV_PRINTF(LEVEL_NOTICE, "NVRM: RTD3/GC6 is not supported on VM\n");
+        NV_PRINTF(LEVEL_NOTICE, "RTD3 is not supported on VM\n");
         return NV_FALSE;
     }
 
@@ -2007,47 +1916,42 @@ static NvBool RmCheckRtd3GcxSupport(
  *
  * @param[in]   nv      nv_state_t pointer.
  */
-void RmInitDeferredDynamicPowerManagement(
-    nv_state_t *nv
+static void RmInitDeferredDynamicPowerManagement(
+    nv_state_t *nv,
+    NvBool      bRtd3Support
 )
 {
     NV_STATUS  status;
     nv_priv_t *nvp  = NV_GET_NV_PRIV(nv);
 
-    // LOCK: acquire GPUs lock
-    if ((status = rmGpuLocksAcquire(GPUS_LOCK_FLAGS_NONE, RM_LOCK_MODULES_DYN_POWER)) == NV_OK)
+    if (nvp->dynamic_power.mode == NV_DYNAMIC_PM_FINE)
     {
-        if (nvp->dynamic_power.mode == NV_DYNAMIC_PM_FINE)
+        OBJGPU *pGpu = NV_GET_NV_PRIV_PGPU(nv);
+
+        if (!bRtd3Support)
         {
-            OBJGPU *pGpu = NV_GET_NV_PRIV_PGPU(nv);
-
-            if (!RmCheckRtd3GcxSupport(nv))
-            {
-                 nvp->dynamic_power.mode = NV_DYNAMIC_PM_NEVER;
-                 nvp->dynamic_power.b_fine_not_supported = NV_TRUE;
-                 NV_PRINTF(LEVEL_NOTICE, "NVRM: RTD3/GC6 is not supported\n");
-                 goto unlock;
-            }
-            osAddGpuDynPwrSupported(gpuGetInstance(pGpu));
-            nvp->dynamic_power.b_fine_not_supported = NV_FALSE;
-            status = CreateDynamicPowerCallbacks(pGpu);
-
-            if (status == NV_OK)
-            {
-                RmScheduleCallbackForIdlePreConditionsUnderGpuLock(pGpu);
-
-                nvp->dynamic_power.deferred_idle_enabled = NV_TRUE;
-                // RM's default is GCOFF allow
-                nvp->dynamic_power.clients_gcoff_disallow_refcount = 0;
-            }
+             nvp->dynamic_power.mode = NV_DYNAMIC_PM_NEVER;
+             nvp->dynamic_power.b_fine_not_supported = NV_TRUE;
+             NV_PRINTF(LEVEL_NOTICE, "RTD3 is not supported.\n");
+             return;
         }
-unlock:
-        // UNLOCK: release GPUs lock
-        rmGpuLocksRelease(GPUS_LOCK_FLAGS_NONE, NULL);
-    }
+        osAddGpuDynPwrSupported(gpuGetInstance(pGpu));
+        nvp->dynamic_power.b_fine_not_supported = NV_FALSE;
+        status = CreateDynamicPowerCallbacks(pGpu);
 
-    if (status != NV_OK)
-       NV_PRINTF(LEVEL_ERROR, "NVRM: Failed to register for dynamic power callbacks\n");
+        if (status == NV_OK)
+        {
+            RmScheduleCallbackForIdlePreConditionsUnderGpuLock(pGpu);
+
+            nvp->dynamic_power.deferred_idle_enabled = NV_TRUE;
+            // RM's default is GCOFF allow
+            nvp->dynamic_power.clients_gcoff_disallow_refcount = 0;
+        }
+        else
+        {
+            NV_PRINTF(LEVEL_ERROR, "Failed to register for dynamic power callbacks\n");
+        }
+    }
 }
 
 /*!
@@ -2262,7 +2166,7 @@ NV_STATUS RmGcxPowerManagement(
         // GC6 3.0 is possible without AML override but due to changes required
         // for GCOFF-1.0 in SBIOS & HW, AML override is needed for GC6-3.0 also.
         //
-        NV_PRINTF(LEVEL_INFO,"NVRM: AML overrides present in Desktop");
+        NV_PRINTF(LEVEL_INFO,"AML overrides present in Desktop");
     }
 
     nv->d0_state_in_suspend = NV_FALSE;
@@ -2382,9 +2286,8 @@ NV_STATUS RmGcxPowerManagement(
             {
                 status = NV_ERR_NOT_SUPPORTED;
                 NV_PRINTF(LEVEL_ERROR,
-                          "NVRM: %s: System suspend failed with current system suspend configuration. "
-                          "Please change the system suspend configuration to s2idle in /sys/power/mem_sleep.\n",
-                          __FUNCTION__);
+                          "System suspend failed with current system suspend configuration. "
+                          "Please change the system suspend configuration to s2idle in /sys/power/mem_sleep.\n");
             }
         }
     }
@@ -2555,8 +2458,12 @@ NV_STATUS NV_API_CALL rm_transition_dynamic_power(
     status = rmGpuLocksAcquire(GPUS_LOCK_FLAGS_NONE, RM_LOCK_MODULES_DYN_POWER);
     if (status == NV_OK)
     {
+        rmapiEnterRtd3PmPath();
+
         status = RmTransitionDynamicPower(pGpu, bEnter, bTryAgain);
 
+        rmapiLeaveRtd3PmPath();
+
         // UNLOCK: release GPUs lock
         rmGpuLocksRelease(GPUS_LOCK_FLAGS_NONE, NULL);
     }
@@ -2625,72 +2532,47 @@ void RmHandleDisplayChange(
         }
         else
             NV_PRINTF(LEVEL_ERROR,
-                      "NVRM: %s: Failed to increment dynamic power refcount\n",
-                      __FUNCTION__);
+                      "Failed to increment dynamic power refcount\n");
     }
 }
 
 /*!
  * @brief: Function to query Dynamic Power Management
  *
- * @param[in]   sp     nvidia_stack_t pointer.
- * @param[in]   pNv    nv_state_t pointer.
+ * @param[in]   nvp     nv_priv_t pointer.
  *
  * @return      String indicating Dynamic Power Management status.
  */
-const char* NV_API_CALL rm_get_dynamic_power_management_status(
-    nvidia_stack_t *sp,
-    nv_state_t     *pNv
+static const char* RmGetDynamicPowerManagementStatus(
+        nv_priv_t *nvp
 )
 {
-    void              *fp;
-    nv_priv_t         *nvp = NV_GET_NV_PRIV(pNv);
-    const char        *returnString = "?";
-
-    NV_ENTER_RM_RUNTIME(sp,fp);
-
-    // LOCK: acquire API lock
-    if ((rmapiLockAcquire(API_LOCK_FLAGS_NONE, RM_LOCK_MODULES_DYN_POWER)) == NV_OK)
+   /*
+    * If the platform/driver does not support Dynamic Power Management,
+    * we set mode as NV_DYNAMIC_PM_NEVER. Hence, after RmInit if the
+    * mode is still NV_DYNAMIC_PM_FINE, we are sure that it is
+    * supported and enabled. Also see NOTE.
+    */
+    if (nvp->dynamic_power.mode == NV_DYNAMIC_PM_FINE)
+    {
+        return "Enabled (fine-grained)";
+    }
+    else if (nvp->dynamic_power.mode == NV_DYNAMIC_PM_COARSE)
+    {
+        return "Enabled (coarse-grained)";
+    }
+    else if (nvp->dynamic_power.mode == NV_DYNAMIC_PM_NEVER)
     {
-        OBJGPU *pGpu = NV_GET_NV_PRIV_PGPU(pNv);
 
-        // Check if RM is inited
-        if (pGpu != NULL)
-        {
-           /*
-            * If the platform/driver does not support Dynamic Power Management,
-            * we set mode as NV_DYNAMIC_PM_NEVER. Hence, after RmInit if the
-            * mode is still NV_DYNAMIC_PM_FINE, we are sure that it is
-            * supported and enabled. Also see NOTE.
-            */
-            if (nvp->dynamic_power.mode == NV_DYNAMIC_PM_FINE)
-            {
-                returnString = "Enabled (fine-grained)";
-            }
-            else if (nvp->dynamic_power.mode == NV_DYNAMIC_PM_COARSE)
-            {
-                returnString = "Enabled (coarse-grained)";
-            }
-            else if (nvp->dynamic_power.mode == NV_DYNAMIC_PM_NEVER)
-            {
-
-                if (nvp->dynamic_power.dynamic_power_regkey ==
-                    NV_REG_DYNAMIC_POWER_MANAGEMENT_NEVER)
-                    returnString = "Disabled";
-                else if (nvp->dynamic_power.dynamic_power_regkey ==
-                         NV_REG_DYNAMIC_POWER_MANAGEMENT_DEFAULT)
-                    returnString = "Disabled by default";
-                else
-                    returnString = "Not supported";
-            }
-        }
-        //UNLOCK: release API lock
-        rmapiLockRelease();
+        if (nvp->dynamic_power.dynamic_power_regkey ==
+            NV_REG_DYNAMIC_POWER_MANAGEMENT_NEVER)
+            return "Disabled";
+        else if (nvp->dynamic_power.dynamic_power_regkey ==
+                 NV_REG_DYNAMIC_POWER_MANAGEMENT_DEFAULT)
+            return "Disabled by default";
     }
 
-    NV_EXIT_RM_RUNTIME(sp,fp);
-
-    return returnString;
+    return "Not supported";
 }
 
 static void RmHandleIdleSustained(
@@ -2723,12 +2605,11 @@ static void RmQueueIdleSustainedWorkitem(
         status = osQueueWorkItemWithFlags(pGpu,
                                           RmHandleIdleSustained,
                                           NULL,
-                                          OS_QUEUE_WORKITEM_FLAGS_LOCK_GPU_GROUP_SUBDEVICE_RW);
+                                          OS_QUEUE_WORKITEM_FLAGS_LOCK_GPU_GROUP_SUBDEVICE);
         if (status != NV_OK)
         {
             NV_PRINTF(LEVEL_WARNING,
-                      "NVRM: %s: Failed to queue RmHandleIdleSustained() workitem.\n",
-                      __FUNCTION__);
+                      "Failed to queue RmHandleIdleSustained() workitem.\n");
             RmScheduleCallbackForIdlePreConditionsUnderGpuLock(pGpu);
             return;
         }
@@ -2739,14 +2620,15 @@ static void RmQueueIdleSustainedWorkitem(
 /*
  * Allocate resources needed for S0ix-based system power management.
  */
-void
+static void
 RmInitS0ixPowerManagement(
-    nv_state_t *nv
+    nv_state_t *nv,
+    NvBool bRtd3Support,
+    NvBool bGC6Support
 )
 {
     nv_priv_t  *nvp = NV_GET_NV_PRIV(nv);
     NvU32       data;
-    NvBool      bRtd3Gc6Support = NV_FALSE;
 
     // S0ix-based S2Idle, on desktops, is not supported yet. Return early for desktop SKUs
     if (!nvp->b_mobile_config_enabled)
@@ -2754,15 +2636,6 @@ RmInitS0ixPowerManagement(
         return;
     }
 
-    // LOCK: acquire GPUs lock
-    if (rmGpuLocksAcquire(GPUS_LOCK_FLAGS_NONE, RM_LOCK_MODULES_INIT) == NV_OK)
-    {
-        bRtd3Gc6Support = RmCheckRtd3GcxSupport(nv);
-
-        // UNLOCK: release GPUs lock
-        rmGpuLocksRelease(GPUS_LOCK_FLAGS_NONE, NULL);
-    }
-
     /*
      * The GPU S0ix-based system power management will be enabled
      * only if all the following necessary requirements are met:
@@ -2771,7 +2644,7 @@ RmInitS0ixPowerManagement(
      * 2. The platform has support for s0ix.
      * 3. Feature regkey EnableS0ixPowerManagement is enabled.
      */
-    if (bRtd3Gc6Support &&
+    if (bRtd3Support && bGC6Support &&
         nv_platform_supports_s0ix() &&
         (osReadRegistryDword(NULL, NV_REG_ENABLE_S0IX_POWER_MANAGEMENT,
                              &data) == NV_OK) && (data == 1))
@@ -2791,3 +2664,86 @@ RmInitS0ixPowerManagement(
         }
     }
 }
+
+void RmInitPowerManagement(
+    nv_state_t *nv
+)
+{
+    // LOCK: acquire GPUs lock
+    if (rmGpuLocksAcquire(GPUS_LOCK_FLAGS_NONE, RM_LOCK_MODULES_INIT) == NV_OK)
+    {
+        NvBool bGC6Support   = NV_FALSE;
+        NvBool bGCOFFSupport = NV_FALSE;
+        NvBool bRtd3Support  = RmCheckRtd3GcxSupport(nv, &bGC6Support, &bGCOFFSupport);
+
+        RmInitDeferredDynamicPowerManagement(nv, bRtd3Support);
+        RmInitS0ixPowerManagement(nv, bRtd3Support, bGC6Support);
+
+        // UNLOCK: release GPUs lock
+        rmGpuLocksRelease(GPUS_LOCK_FLAGS_NONE, NULL);
+    }
+}
+
+void RmDestroyPowerManagement(
+    nv_state_t *nv
+)
+{
+    RmDestroyDeferredDynamicPowerManagement(nv);
+}
+
+void NV_API_CALL rm_get_power_info(
+    nvidia_stack_t  *sp,
+    nv_state_t      *pNv,
+    nv_power_info_t *powerInfo
+)
+{
+    THREAD_STATE_NODE threadState;
+    void              *fp;
+    GPU_MASK          gpuMask;
+    const char       *pVidmemPowerStatus = "?";
+    const char       *pGc6Supported = "?";
+    const char       *pGcoffSupported = "?";
+    const char       *pDynamicPowerStatus = "?";
+    const char       *pS0ixStatus = "?";
+
+    NV_ENTER_RM_RUNTIME(sp,fp);
+    threadStateInit(&threadState, THREAD_STATE_FLAGS_NONE);
+
+    // LOCK: acquire API lock
+    if ((rmapiLockAcquire(API_LOCK_FLAGS_NONE, RM_LOCK_MODULES_DYN_POWER)) == NV_OK)
+    {
+        OBJGPU *pGpu = NV_GET_NV_PRIV_PGPU(pNv);
+
+        // LOCK: acquire per device lock
+        if ((pGpu != NULL) &&
+           ((rmGpuGroupLockAcquire(pGpu->gpuInstance, GPU_LOCK_GRP_SUBDEVICE,
+                                   GPUS_LOCK_FLAGS_NONE, RM_LOCK_MODULES_DYN_POWER,
+                                   &gpuMask)) == NV_OK))
+        {
+            nv_priv_t *pNvp = NV_GET_NV_PRIV(pNv);
+
+            pVidmemPowerStatus = pGpu->getProperty(pGpu, PDB_PROP_GPU_GCOFF_STATE_ENTERED) ?
+                                    "Off" : "Active";
+            pDynamicPowerStatus = RmGetDynamicPowerManagementStatus(pNvp);
+            pGc6Supported = RmGetGpuGcxSupport(pGpu, NV_TRUE);
+            pGcoffSupported = RmGetGpuGcxSupport(pGpu, NV_FALSE);
+            pS0ixStatus = pNvp->s0ix_pm_enabled ? "Enabled" : "Disabled";
+
+            // UNLOCK: release per device lock
+            rmGpuGroupLockRelease(gpuMask, GPUS_LOCK_FLAGS_NONE);
+        }
+
+        // UNLOCK: release API lock
+        rmapiLockRelease();
+    }
+
+    threadStateFree(&threadState, THREAD_STATE_FLAGS_NONE);
+    NV_EXIT_RM_RUNTIME(sp,fp);
+
+    powerInfo->vidmem_power_status = pVidmemPowerStatus;
+    powerInfo->dynamic_power_status = pDynamicPowerStatus;
+    powerInfo->gc6_support = pGc6Supported;
+    powerInfo->gcoff_support = pGcoffSupported;
+    powerInfo->s0ix_status = pS0ixStatus;
+}
+
