@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2019-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2019-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -54,6 +54,13 @@ typedef struct GSP_VF_INFO
     NvBool b64bitBar2;
 } GSP_VF_INFO;
 
+// Cache config registers from pcie space
+typedef struct
+{
+    // Link capabilities
+    NvU32 linkCap;
+} GSP_PCIE_CONFIG_REG;
+
 typedef struct GspSMInfo_t
 {
     NvU32 version;
@@ -67,6 +74,13 @@ typedef struct GspSMInfo_t
     NvU32 rtCoreCount;
 } GspSMInfo;
 
+typedef struct
+{
+    NvU32 ecidLow;
+    NvU32 ecidHigh;
+    NvU32 ecidExtended;
+} EcidManufacturingInfo;
+
 // Fetched from GSP-RM into CPU-RM
 typedef struct GspStaticConfigInfo_t
 {
@@ -88,10 +102,10 @@ typedef struct GspStaticConfigInfo_t
     NvBool poisonFuseEnabled;
 
     NvU64 fb_length;
-    NvU32 fbio_mask;
+    NvU64 fbio_mask;
     NvU32 fb_bus_width;
     NvU32 fb_ram_type;
-    NvU32 fbp_mask;
+    NvU64 fbp_mask;
     NvU32 l2_cache_size;
 
     NvU32 gfxpBufferSize[NV2080_CTRL_CMD_GR_CTXSW_PREEMPTION_BIND_BUFFERS_CONTEXT_POOL];
@@ -113,6 +127,21 @@ typedef struct GspStaticConfigInfo_t
     NvBool bIsGcOffRtd3Allowed;
     NvBool bIsGcoffLegacyAllowed;
 
+    /* "Total Board Power" refers to power requirement of GPU,
+     * while in GC6 state. Majority of this power will be used
+     * to keep V-RAM active to preserve its content.
+     * Some energy maybe consumed by Always-on components on GPU chip.
+     * This power will be provided by 3.3v voltage rail.
+     */
+    NvU16  RTD3GC6TotalBoardPower;
+
+    /* PERST# (i.e. PCI Express Reset) is a sideband signal
+     * generated by the PCIe Host to indicate the PCIe devices,
+     * that the power-rails and the reference-clock are stable.
+     * The endpoint device typically uses this signal as a global reset.
+     */
+    NvU16  RTD3GC6PerstDelay;
+
     NvU64 bar1PdeBase;
     NvU64 bar2PdeBase;
 
@@ -143,6 +172,9 @@ typedef struct GspStaticConfigInfo_t
     NvBool bAtsSupported;
 
     NvBool bIsGpuUefi;
+    NvBool bIsEfiInit;
+
+    EcidManufacturingInfo ecidInfo[2];
 } GspStaticConfigInfo;
 
 // Pushed from CPU-RM to GSP-RM
@@ -151,6 +183,7 @@ typedef struct GspSystemInfo
     NvU64 gpuPhysAddr;
     NvU64 gpuPhysFbAddr;
     NvU64 gpuPhysInstAddr;
+    NvU64 gpuPhysIoAddr;
     NvU64 nvDomainBusDeviceFunc;
     NvU64 simAccessBufPhysAddr;
     NvU64 notifyOpSharedSurfacePhysAddr;
@@ -159,6 +192,10 @@ typedef struct GspSystemInfo
     NvU64 maxUserVa;
     NvU32 pciConfigMirrorBase;
     NvU32 pciConfigMirrorSize;
+    NvU32 PCIDeviceID;
+    NvU32 PCISubDeviceID;
+    NvU32 PCIRevisionID;
+    NvU32 pcieAtomicsCplDeviceCapMask;
     NvU8 oorArch;
     NvU64 clPdbProperties;
     NvU32 Chipset;
@@ -170,7 +207,6 @@ typedef struct GspSystemInfo
     NvBool bUpstreamL1Unsupported;
     NvBool bUpstreamL1PorSupported;
     NvBool bUpstreamL1PorMobileOnly;
-    NvBool bSystemHasMux;
     NvU8   upstreamAddressValid;
     BUSINFO FHBBusInfo;
     BUSINFO chipsetIDInfo;
@@ -179,9 +215,14 @@ typedef struct GspSystemInfo
     NvBool bIsPassthru;
     NvU64 sysTimerOffsetNs;
     GSP_VF_INFO gspVFInfo;
+    NvBool bIsPrimary;
     NvBool isGridBuild;
+    GSP_PCIE_CONFIG_REG pcieConfigReg;
     NvU32 gridBuildCsp;
+    NvBool bPreserveVideoMemoryAllocations;
     NvBool bTdrEventSupported;
+    NvBool bFeatureStretchVblankCapable;
+    NvBool bClockBoostSupported;
 } GspSystemInfo;
 
 
