@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -56,6 +56,7 @@
 #include "power/gpu_boost_mgr.h"
 #include "compute/fabric.h"
 #include "gpu_mgr/gpu_db.h"
+#include "core/bin_data.h"
 
 // local static functions
 static NV_STATUS    _sysCreateOs(OBJSYS *);
@@ -93,11 +94,85 @@ static sysChildObject sysChildObjects[] =
     { NV_OFFSETOF(OBJSYS, pGpuDb),          classInfo(GpuDb),           NV_TRUE },
 };
 
+static void
+_sysDestroyMemExportCache(OBJSYS *pSys)
+{
+    if (pSys->pSysMemExportModuleLock != NULL)
+    {
+        portSyncRwLockDestroy(pSys->pSysMemExportModuleLock);
+        pSys->pSysMemExportModuleLock = NULL;
+    }
+
+    NV_ASSERT(multimapCountItems(&pSys->sysMemExportCache) == 0);
+
+    multimapDestroy(&pSys->sysMemExportCache);
+}
+
+static NV_STATUS
+_sysInitMemExportCache(OBJSYS *pSys)
+{
+    multimapInit(&pSys->sysMemExportCache, portMemAllocatorGetGlobalNonPaged());
+
+    pSys->pSysMemExportModuleLock =
+                    portSyncRwLockCreate(portMemAllocatorGetGlobalNonPaged());
+    if (pSys->pSysMemExportModuleLock == NULL)
+    {
+        _sysDestroyMemExportCache(pSys);
+        return NV_ERR_NO_MEMORY;
+    }
+
+    return NV_OK;
+}
+
+static void
+_sysDestroyMemExportClient(OBJSYS *pSys)
+{
+    RM_API *pRmApi;
+
+    if (pSys->hSysMemExportClient == 0)
+        return;
+
+    // Acquire lock to keep rmapiGetInterface() happy.
+    NV_ASSERT(rmapiLockAcquire(API_LOCK_FLAGS_NONE,
+                               RM_LOCK_MODULES_DESTROY) == NV_OK);
+
+    pRmApi = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
+
+    NV_ASSERT(pRmApi->Free(pRmApi, pSys->hSysMemExportClient,
+                           pSys->hSysMemExportClient) == NV_OK);
+
+    rmapiLockRelease();
+
+    pSys->hSysMemExportClient = 0;
+}
+
+static NV_STATUS
+_sysInitMemExportClient(OBJSYS *pSys)
+{
+    RM_API *pRmApi;
+    NV_STATUS status;
+
+    // Acquire lock to keep rmapiGetInterface() happy.
+    status = rmapiLockAcquire(API_LOCK_FLAGS_NONE, RM_LOCK_MODULES_INIT);
+    if (status != NV_OK)
+        return status;
+
+    pRmApi = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
+
+    status = pRmApi->AllocWithHandle(pRmApi, NV01_NULL_OBJECT,
+                                     NV01_NULL_OBJECT, NV01_NULL_OBJECT,
+                                     NV01_ROOT, &pSys->hSysMemExportClient,
+                                     sizeof(pSys->hSysMemExportClient));
+
+    rmapiLockRelease();
+
+    return status;
+}
+
 NV_STATUS
 sysConstruct_IMPL(OBJSYS *pSys)
 {
     NV_STATUS          status;
-    OBJOS             *pOS;
     NvU32              sec = 0;
     NvU32              uSec = 0;
 
@@ -115,22 +190,20 @@ sysConstruct_IMPL(OBJSYS *pSys)
     }
 
     // Use the monotonic system clock for a unique value
-    pOS = SYS_GET_OS(pSys);
     osGetCurrentTime(&sec, &uSec);
     pSys->rmInstanceId = (NvU64)sec * 1000000 + (NvU64)uSec;
 
     {
         // Using Hypervisor native interface to detect
+        OBJOS         *pOS         = SYS_GET_OS(pSys);
         OBJHYPERVISOR *pHypervisor = SYS_GET_HYPERVISOR(pSys);
         if (pHypervisor)
             hypervisorDetection(pHypervisor, pOS);
     }
 
-    if (!pOS->osRmInitRm(pOS))
-    {
-        status = NV_ERR_GENERIC;
+    status = osRmInitRm();
+    if (status != NV_OK)
         goto failed;
-    }
 
     _sysNvSwitchDetection(pSys);
 
@@ -150,10 +223,22 @@ sysConstruct_IMPL(OBJSYS *pSys)
     if (status != NV_OK)
         goto failed;
 
+    status = _sysInitMemExportCache(pSys);
+    if (status != NV_OK)
+        goto failed;
+
+    status = _sysInitMemExportClient(pSys);
+    if (status != NV_OK)
+        goto failed;
+
+    bindataInitialize();
+
     return NV_OK;
 
 failed:
 
+    _sysDestroyMemExportCache(pSys);
+
     _sysDeleteChildObjects(pSys);
 
     g_pSys = NULL;
@@ -170,6 +255,11 @@ void
 sysDestruct_IMPL(OBJSYS *pSys)
 {
 
+    pSys->setProperty(pSys, PDB_PROP_SYS_DESTRUCTING, NV_TRUE);
+
+    _sysDestroyMemExportCache(pSys);
+    _sysDestroyMemExportClient(pSys);
+
     //
     // Any of these operations might fail but go ahead and
     // attempt to free remaining resources before complaining.
@@ -193,6 +283,7 @@ sysDestruct_IMPL(OBJSYS *pSys)
     RMTRACE_DESTROY();
     RMTRACE_DESTROY_NEW();
 
+    bindataDestroy();
 }
 
 //
@@ -324,12 +415,6 @@ _sysRegistryOverrideResourceServer
         pSys->apiLockModuleMask = RM_LOCK_MODULE_GRP(RM_LOCK_MODULES_CLIENT);
     }
 
-    if (osReadRegistryDword(pGpu, NV_REG_STR_RM_CLIENT_HANDLE_LOOKUP,
-                            &data32) == NV_OK)
-    {
-        pSys->setProperty(pSys, PDB_PROP_SYS_CLIENT_HANDLE_LOOKUP, !!data32);
-    }
-
     if (osReadRegistryDword(pGpu, NV_REG_STR_RM_LOCK_TIME_COLLECT,
                             &data32) == NV_OK)
     {
@@ -347,7 +432,6 @@ _sysRegistryOverrideResourceServer
     {
         pSys->clientListDeferredFreeLimit = data32;
     }
-
 }
 
 static void
@@ -693,6 +777,17 @@ sysInitRegistryOverrides_IMPL
         pSys->setProperty(pSys, PDB_PROP_SYS_ROUTE_TO_PHYSICAL_LOCK_BYPASS, !!data32);
     }
 
+    if (osReadRegistryDword(pGpu, NV_REG_STR_RM_GPU_LOCK_MIDPATH, &data32) == NV_OK)
+    {
+        pSys->setProperty(pSys, PDB_PROP_SYS_GPU_LOCK_MIDPATH_ENABLED, !!data32);
+    }
+
+    if (osReadRegistryDword(pGpu, NV_REG_STR_RM_ENABLE_RM_TEST_ONLY_CODE,
+                            &data32) == NV_OK)
+    {
+        pSys->setProperty(pSys, PDB_PROP_SYS_ENABLE_RM_TEST_ONLY_CODE, !!data32);
+    }
+
     gpumgrSetGpuNvlinkBwModeFromRegistry(pGpu);
 }
 
