@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2021-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2021-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -29,16 +29,28 @@
 #include "gpu/fsp/kern_fsp.h"
 #include "platform/chipset/chipset.h"
 #include "ctrl/ctrl2080/ctrl2080bus.h"
+#include "kernel/gpu/nvlink/kernel_nvlink.h"
+
+#include "published/hopper/gh100/dev_xtl_ep_pcfg_gpu.h"
 
 #include "published/hopper/gh100/dev_fb.h"
-#include "published/hopper/gh100/dev_xtl_ep_pri.h"
-#include "published/hopper/gh100/dev_xtl_ep_pcfg_gpu.h"
 #include "published/hopper/gh100/hwproject.h"
-#include "published/hopper/gh100/dev_pmc.h"
+#include "published/hopper/gh100/dev_xtl_ep_pri.h"
+#include "published/hopper/gh100/dev_nv_xpl.h"
 #include "published/hopper/gh100/dev_vm.h"
+#include "published/hopper/gh100/dev_pmc.h"
 
 #include "os/os.h"
 
+static NV_STATUS    _kbifSavePcieConfigRegisters_GH100(OBJGPU *pGpu, KernelBif *pKernelBif, const PKBIF_XVE_REGMAP_REF pRegmapRef);
+static NV_STATUS    _kbifRestorePcieConfigRegisters_GH100(OBJGPU *pGpu, KernelBif *pKernelBif, const PKBIF_XVE_REGMAP_REF pRegmapRef);
+
+// Size of PCIe config space
+#define PCIE_CONFIG_SPACE_MAX_SIZE     0x2FC
+
+// Devinit completion timeout after FLR
+#define BIF_FLR_DEVINIT_COMPLETION_TIMEOUT_DEFAULT      900000
+
 /*!
  * @brief Check if MSI is enabled in HW
  *
@@ -633,10 +645,8 @@ kbifProbePcieReqAtomicCaps_GH100
 )
 {
     NvU32   osAtomicsMask    = 0;
-    OBJSYS *pSys = SYS_GET_INSTANCE();
-    OBJCL  *pCl  = SYS_GET_CL(pSys);
 
-    if (!pCl->getProperty(pCl, PDB_PROP_CL_BUG_3562968_WAR_ALLOW_PCIE_ATOMICS))
+    if (!kbifAllowGpuReqPcieAtomics_HAL(pGpu, pKernelBif))
     {
         NV_PRINTF(LEVEL_INFO, "PCIe atomics not supported in this platform!\n");
         return;
@@ -719,12 +729,18 @@ kbifGetBusOptionsAddr_GH100
         case BUS_OPTIONS_DEV_CONTROL_STATUS:
             *pAddrReg = NV_EP_PCFG_GPU_DEVICE_CONTROL_STATUS;
             break;
+        case BUS_OPTIONS_DEV_CONTROL_STATUS_2:
+            *pAddrReg = NV_EP_PCFG_GPU_DEVICE_CONTROL_STATUS_2;
+            break;
         case BUS_OPTIONS_LINK_CONTROL_STATUS:
             *pAddrReg = NV_EP_PCFG_GPU_LINK_CONTROL_STATUS;
             break;
         case BUS_OPTIONS_LINK_CAPABILITIES:
             *pAddrReg = NV_EP_PCFG_GPU_LINK_CAPABILITIES;
             break;
+        case BUS_OPTIONS_L1_PM_SUBSTATES_CTRL_1:
+            *pAddrReg = NV_EP_PCFG_GPU_L1_PM_SS_CONTROL_1_REGISTER;
+            break;
         default:
             NV_PRINTF(LEVEL_ERROR, "Invalid register type passed 0x%x\n",
                       options);
@@ -734,6 +750,354 @@ kbifGetBusOptionsAddr_GH100
     return status;
 }
 
+/*!
+ * @brief Helper function for _kbifSavePcieConfigRegisters_GH100()
+ *
+ * @param[in]  pGpu         GPU object pointer
+ * @param[in]  pKernelBif  Kernel BIF object pointer
+ * @param[in]  pRegmapRef   Pointer to XVE Register map structure
+ *
+ * @return  'NV_OK' if successful, an RM error code otherwise.
+ */
+static NV_STATUS
+_kbifSavePcieConfigRegisters_GH100
+(
+    OBJGPU    *pGpu,
+    KernelBif *pKernelBif,
+    const PKBIF_XVE_REGMAP_REF pRegmapRef
+)
+{
+    NV_STATUS status;
+    NvU32     regOffset;
+    NvU32     bufOffset = 0;
+
+    // Read and save entire config space
+    for (regOffset = 0x0; regOffset < PCIE_CONFIG_SPACE_MAX_SIZE; regOffset+=0x4)
+    {
+        status = GPU_BUS_CFG_CYCLE_RD32(pGpu, regOffset,
+                                        &pRegmapRef->bufBootConfigSpace[bufOffset]);
+        if (status != NV_OK)
+        {
+            NV_PRINTF(LEVEL_ERROR, "Config read failed.\n");
+            return status;
+        }
+        bufOffset++;
+    }
+
+    pKernelBif->setProperty(pKernelBif, PDB_PROP_KBIF_SECONDARY_BUS_RESET_SUPPORTED, NV_TRUE);
+
+    return NV_OK;
+}
+
+/*!
+ * @brief Helper function for _kbifRestorePcieConfigRegisters_GH100
+ *
+ * @param[in]  pGpu         GPU object pointer
+ * @param[in]  pKernelBif  Kernel BIF object pointer
+ * @param[in]  pRegmapRef   Pointer to XVE Register map structure
+ *
+ * @return  'NV_OK' if successful, an RM error code otherwise.
+ */
+static NV_STATUS
+_kbifRestorePcieConfigRegisters_GH100
+(
+    OBJGPU    *pGpu,
+    KernelBif *pKernelBif,
+    const PKBIF_XVE_REGMAP_REF pRegmapRef
+)
+{
+    NvU32     domain    = gpuGetDomain(pGpu);
+    NvU8      bus       = gpuGetBus(pGpu);
+    NvU8      device    = gpuGetDevice(pGpu);
+    void      *pHandle;
+    NvU32     regOffset;
+    NvU32     bufOffset = 0;
+    NV_STATUS status;
+
+    pHandle = osPciInitHandle(domain, bus, device, 0, NULL, NULL);
+    NV_ASSERT_OR_RETURN(pHandle, NV_ERR_INVALID_POINTER);
+
+    // Restore entire config space
+    for (regOffset = 0x0; regOffset < PCIE_CONFIG_SPACE_MAX_SIZE; regOffset+=0x4)
+    {
+        status = GPU_BUS_CFG_CYCLE_WR32(pGpu, regOffset,
+                     pRegmapRef->bufBootConfigSpace[bufOffset]);
+        if (status != NV_OK)
+        {
+            NV_PRINTF(LEVEL_ERROR, "Config write failed.\n");
+            NV_ASSERT(0);
+            return status;
+        }
+        bufOffset++;
+    }
+
+    return NV_OK;
+}
+
+/*!
+ * @brief  Helper function saves MSIX vector control masks
+ *
+ * @param[in]  pGpu         GPU object pointer
+ * @param[in]  pKernelBif   KernelBif object pointer
+ *
+ * @return  'NV_OK' if successful, an RM error code otherwise.
+ */
+NV_STATUS
+kbifSaveMsixTable_GH100
+(
+    OBJGPU    *pGpu,
+    KernelBif *pKernelBif
+)
+{
+    NvU32 i;
+    NvU32 controlSize = kbifGetMSIXTableVectorControlSize_HAL(pGpu, pKernelBif);
+
+    for (i = 0U; i < controlSize; i++)
+    {
+        // Each MSIX table entry is 4 NvU32s
+        pKernelBif->xveRegmapRef[0].bufMsixTable[(i*4) + 0] = GPU_VREG_RD32(pGpu, NV_VIRTUAL_FUNCTION_PRIV_MSIX_TABLE_ADDR_LO(i));
+        pKernelBif->xveRegmapRef[0].bufMsixTable[(i*4) + 1] = GPU_VREG_RD32(pGpu, NV_VIRTUAL_FUNCTION_PRIV_MSIX_TABLE_ADDR_HI(i));
+        pKernelBif->xveRegmapRef[0].bufMsixTable[(i*4) + 2] = GPU_VREG_RD32(pGpu, NV_VIRTUAL_FUNCTION_PRIV_MSIX_TABLE_DATA(i));
+        pKernelBif->xveRegmapRef[0].bufMsixTable[(i*4) + 3] = GPU_VREG_RD32(pGpu, NV_VIRTUAL_FUNCTION_PRIV_MSIX_TABLE_VECTOR_CONTROL(i));
+    }
+
+    return NV_OK;
+}
+
+/*!
+ * @brief  Helper function to restore MSIX vector control masks
+ *
+ * @param[in]  pGpu         GPU object pointer
+ * @param[in]  pKernelBif   KernelBif object pointer
+ *
+ * @return  'NV_OK' if successful, an RM error code otherwise.
+ */
+NV_STATUS
+kbifRestoreMsixTable_GH100
+(
+    OBJGPU    *pGpu,
+    KernelBif *pKernelBif
+)
+{
+    NvU32 i;
+    NvU32 controlSize = kbifGetMSIXTableVectorControlSize_HAL(pGpu, pKernelBif);
+
+    // Initialize the base offset for the virtual registers for physical function
+    NvU32 vRegOffset = pGpu->sriovState.virtualRegPhysOffset;
+
+    // In FLR path, we don't want to use usual register r/w macros
+    for (i = 0U; i < controlSize; i++)
+    {
+        // Each MSIX table entry is 4 NvU32s
+        osGpuWriteReg032(pGpu, vRegOffset + NV_VIRTUAL_FUNCTION_PRIV_MSIX_TABLE_ADDR_LO(i),        pKernelBif->xveRegmapRef[0].bufMsixTable[(i*4) + 0]);
+        osGpuWriteReg032(pGpu, vRegOffset + NV_VIRTUAL_FUNCTION_PRIV_MSIX_TABLE_ADDR_HI(i),        pKernelBif->xveRegmapRef[0].bufMsixTable[(i*4) + 1]);
+        osGpuWriteReg032(pGpu, vRegOffset + NV_VIRTUAL_FUNCTION_PRIV_MSIX_TABLE_DATA(i),           pKernelBif->xveRegmapRef[0].bufMsixTable[(i*4) + 2]);
+        osGpuWriteReg032(pGpu, vRegOffset + NV_VIRTUAL_FUNCTION_PRIV_MSIX_TABLE_VECTOR_CONTROL(i), pKernelBif->xveRegmapRef[0].bufMsixTable[(i*4) + 3]);
+    }
+
+    return NV_OK;
+}
+
+/*!
+ * @brief Wait for to get config access.
+ *
+ * @param[in]  pGpu     GPU object pointer
+ * @param[in]  pKernelBif  Kernel BIF object pointer
+ * @param[in]  pTimeout Value in microseconds to dev init complete
+ *
+ * @return  NV_OK if successful.
+ */
+NV_STATUS
+kbifConfigAccessWait_GH100
+(
+    OBJGPU    *pGpu,
+    KernelBif *pKernelBif,
+    RMTIMEOUT *pTimeout
+)
+{
+    NvU32     regVal = 0;
+    NV_STATUS status = NV_OK;
+
+    while (NV_TRUE)
+    {
+        //
+        // This read only register should be accessible over config cycle
+        // once devinit is complete without RM having to restore config space.
+        // This register is not really reset on FLR.
+        //
+        if (GPU_BUS_CFG_CYCLE_RD32(pGpu, NV_EP_PCFG_GPU_ID, &regVal) == NV_OK)
+        {
+            if (FLD_TEST_DRF(_EP_PCFG_GPU, _ID, _VENDOR, _NVIDIA, regVal))
+            {
+                break;
+            }
+        }
+        status = gpuCheckTimeout(pGpu, pTimeout);
+
+        if (status == NV_ERR_TIMEOUT)
+        {
+            NV_ASSERT_FAILED("Timed out waiting for devinit to complete\n");
+            return status;
+        }
+        //
+        // Put ourself into wait state for 1ms. This function runs in the context
+        // of DxgkDdiResetFromTimeout which is at PASSIVE_LEVEL(lower priority)
+        // which means it can be in wait state for longer delays of the order of
+        // milliseconds
+        //
+        osDelay(1);
+    }
+
+    return status;
+}
+
+/*!
+ * @brief Do function level reset for Fn0 of GPU
+ *
+ * @param[in]  pGpu  GPU object pointer
+ * @param[in]  pKernelBif  Kernel BIF object pointer
+ *
+ * @return  NV_OK if successful
+ */
+NV_STATUS
+kbifDoFunctionLevelReset_GH100
+(
+    OBJGPU *pGpu,
+    KernelBif *pKernelBif
+)
+{
+    NvBool     bMSIXEnabled;
+    RMTIMEOUT  timeout;
+    NV_STATUS  status = NV_OK;
+    NvU32      flrDevInitTimeout;
+    NvU32      flrDevInitTimeoutScale = pKernelBif->flrDevInitTimeoutScale;
+
+    // If this is non-windows platform or non-ESXi, we already use OS based interface
+    {
+        pKernelBif->bInFunctionLevelReset = NV_TRUE;
+        status = osDoFunctionLevelReset(pGpu);
+        if (status != NV_OK)
+        {
+            NV_ASSERT_FAILED("osDoFunctionLevelReset failed!\n");
+        }
+        goto kbifDoFunctionLevelReset_GH100_exit;
+    }
+
+    pKernelBif->bPreparingFunctionLevelReset = NV_TRUE;
+    status = kbifSavePcieConfigRegisters_HAL(pGpu, pKernelBif);
+    if (status != NV_OK)
+    {
+        NV_ASSERT_FAILED("Config registers save failed!\n");
+        goto kbifDoFunctionLevelReset_GH100_exit;
+    }
+
+    bMSIXEnabled = kbifIsMSIXEnabledInHW_HAL(pGpu, pKernelBif);
+
+    if (bMSIXEnabled)
+    {
+        status = kbifSaveMsixTable_HAL(pGpu, pKernelBif);
+        if (status != NV_OK)
+        {
+            NV_ASSERT_FAILED("MSIX Table save failed!\n");
+            goto kbifDoFunctionLevelReset_GH100_exit;
+        }
+    }
+
+    // Once we save required registers, we are done with prep. phase for FLR
+    pKernelBif->bPreparingFunctionLevelReset = NV_FALSE;
+
+    // Trigger FLR now
+    status = kbifTriggerFlr_HAL(pGpu, pKernelBif);
+
+    if (status != NV_OK)
+    {
+        goto kbifDoFunctionLevelReset_GH100_exit;
+    }
+
+    pKernelBif->bInFunctionLevelReset = NV_TRUE;
+
+    //
+    // Wait for 10ms on Silicon for reset to propagate. Actually, HW is
+    // supposed to return CRS status for config requests until config
+    // space is not ready but we do expect at least 10ms for it to be ready
+    // so why poll? On RTL and emulation, CPU clocks don't scale to GPU clock
+    // so don't wait.
+    //
+    if (IS_SILICON(pGpu))
+    {
+        osDelayUs(10000);
+    }
+
+    // flrTimeoutScale is regkey based for easy debugging
+    flrDevInitTimeout = BIF_FLR_DEVINIT_COMPLETION_TIMEOUT_DEFAULT *
+                        flrDevInitTimeoutScale;
+
+    //
+    // After CRS_TIMEOUT, HW will auto-clear SEND_CRS bit which means config cycles
+    // will succeed but it is still possible that devinit is not complete yet.
+    // OS won't have a way to check that anyway but RM should do that here.
+    // TODO: Should we use this loop or just the PCI spec-defined 100ms delay
+    // now that we have a devinit wait in fspWaitForSecureBoot_HAL below?
+    //
+    gpuSetTimeout(pGpu, flrDevInitTimeout, &timeout, GPU_TIMEOUT_FLAGS_OSTIMER);
+
+    status = kbifConfigAccessWait_HAL(pGpu, pKernelBif, &timeout);
+
+    if (status != NV_OK)
+    {
+        NV_ASSERT_FAILED("Timed out waiting for devinit to complete\n");
+        goto kbifDoFunctionLevelReset_GH100_exit;
+    }
+
+    status = kbifRestorePcieConfigRegisters_HAL(pGpu, pKernelBif);
+    if (status != NV_OK)
+    {
+        NV_ASSERT_FAILED("Config registers restore failed!\n");
+        goto kbifDoFunctionLevelReset_GH100_exit;
+    }
+
+    // As of GH100, miata rom cannot be run on simulation.
+    if (IS_SILICON(pGpu) || IS_EMULATION(pGpu))
+    {
+        // On emulation VBIOS boot can take long so add prints for better visibility
+        if (IS_EMULATION(pGpu))
+        {
+            NV_PRINTF(LEVEL_ERROR, "Entering secure boot completion wait.\n");
+        }
+
+        status = NV_ERR_NOT_SUPPORTED;
+        if (status != NV_OK)
+        {
+            DBG_BREAKPOINT();
+            NV_PRINTF(LEVEL_ERROR, "VBIOS boot failed!!\n");
+            goto kbifDoFunctionLevelReset_GH100_exit;
+        }
+
+        if (IS_EMULATION(pGpu))
+        {
+            NV_PRINTF(LEVEL_ERROR,
+                      "Exited secure boot completion wait with status = NV_OK.\n");
+        }
+    }
+
+    if (bMSIXEnabled)
+    {
+        // TODO-NM: Check if this needed for other NVPM flows like GC6
+        status = kbifRestoreMsixTable_HAL(pGpu, pKernelBif);
+        if (status != NV_OK)
+        {
+            NV_ASSERT_FAILED("MSIX Table save failed!\n");
+        }
+    }
+
+kbifDoFunctionLevelReset_GH100_exit:
+    pKernelBif->bPreparingFunctionLevelReset = NV_FALSE;
+    pKernelBif->bInFunctionLevelReset        = NV_FALSE;
+
+    return status;
+}
+
+
 /*!
  * @brief Returns size of MSIX vector control table
  *
@@ -750,7 +1114,6 @@ kbifGetMSIXTableVectorControlSize_GH100
     return NV_VIRTUAL_FUNCTION_PRIV_MSIX_TABLE_VECTOR_CONTROL__SIZE_1;
 }
 
-
 /*!
  * @brief Check and cache Function level reset support
  *
@@ -990,6 +1353,94 @@ kbifStopSysMemRequests_GH100
     return NV_OK;
 }
 
+/*!
+ * @brief Save boot time PCIe Config space
+ *
+ * @param[in]  pGpu     GPU object pointer
+ * @param[in]  pBif     BIF object pointer
+ *
+ * @return  'NV_OK' if successful, an RM error code otherwise.
+ */
+NV_STATUS
+kbifSavePcieConfigRegisters_GH100
+(
+    OBJGPU    *pGpu,
+    KernelBif *pKernelBif
+)
+{
+
+    NV_STATUS status = NV_OK;
+
+    // Skip config save on FMODEL (200684952), move to regmap (200700271)
+    if (IS_FMODEL(pGpu))
+    {
+        return NV_OK;
+    }
+
+    //
+    // Save config space if GPU is about to enter Function Level Reset
+    // OR if on non-windows platform, FORCE_PCIE_CONFIG_SAVE is set and SBR is
+    //    enabled
+    // OR if on windows platform, SBR is enabled
+    //
+    if (!pKernelBif->bPreparingFunctionLevelReset &&
+        !((RMCFG_FEATURE_PLATFORM_WINDOWS ||
+           pKernelBif->getProperty(pKernelBif, PDB_PROP_KBIF_FORCE_PCIE_CONFIG_SAVE)) &&
+          pKernelBif->getProperty(pKernelBif, PDB_PROP_KBIF_SECONDARY_BUS_RESET_ENABLED)))
+    {
+        NV_PRINTF(LEVEL_ERROR, "Config space save has been skipped.\n");
+        return NV_OK;
+    }
+
+    // Save pcie config space for function 0
+    status = _kbifSavePcieConfigRegisters_GH100(pGpu, pKernelBif,
+                                                &pKernelBif->xveRegmapRef[0]);
+    if (status != NV_OK)
+    {
+        NV_PRINTF(LEVEL_ERROR, "Saving PCIe config space failed for gpu.\n");
+        NV_ASSERT(0);
+        return status;
+    }
+
+    return status;
+}
+
+/*!
+ * @brief Restore boot time PCIe Config space
+ *
+ * @param[in] pGpu  GPU object pointer
+ * @param[in] pBif  BIF object pointer
+ *
+ * @return  'NV_OK' if successful, an RM error code otherwise.
+ */
+NV_STATUS
+kbifRestorePcieConfigRegisters_GH100
+(
+    OBJGPU    *pGpu,
+    KernelBif *pKernelBif
+)
+{
+    NV_STATUS status = NV_OK;
+
+    // Skip config restore on FMODEL (200684952), move to regmap (200700271)
+    if (IS_FMODEL(pGpu))
+    {
+        return NV_OK;
+    }
+
+    // Restore pcie config space for function 0
+    status = _kbifRestorePcieConfigRegisters_GH100(pGpu, pKernelBif,
+                                                   &pKernelBif->xveRegmapRef[0]);
+    if (status != NV_OK)
+    {
+        NV_PRINTF(LEVEL_ERROR, "Restoring PCIe config space failed for gpu.\n");
+        NV_ASSERT(0);
+        return status;
+    }
+
+    return status;
+}
+
 /*!
  * @brief Waits for function issued transaction completions (sysmem to GPU) to arrive
  *
@@ -1109,23 +1560,157 @@ kbifRestoreBarsAndCommand_GH100
 }
 
 /*!
- * @brief HAL specific BIF software state initialization
+ * @brief  Do any work necessary to be able to do a full chip reset.
  *
  * @param[in] pGpu       GPU object pointer
  * @param[in] pKernelBif KernelBif object pointer
  *
- * @return    NV_OK on success
+ * @return  None
  */
-NV_STATUS
-kbifInit_GH100
+void
+kbifPrepareForFullChipReset_GH100
 (
     OBJGPU    *pGpu,
     KernelBif *pKernelBif
 )
 {
-    // Cache the offsets of BAR registers into an array for subsequent use
-    kbifStoreBarRegOffsets_HAL(pGpu, pKernelBif, NV_EP_PCFG_GPU_VF_BAR0);
+    KernelNvlink *pKernelNvlink = GPU_GET_KERNEL_NVLINK(pGpu);
+    NvBool        bFLRSupportedAndEnabled;
 
-    return NV_OK;
+    bFLRSupportedAndEnabled = (pKernelBif->getProperty(pKernelBif, PDB_PROP_KBIF_FLR_SUPPORTED)) &&
+                               !(pKernelBif->bForceDisableFLR);
+
+    if (!bFLRSupportedAndEnabled)
+    {
+        if (!(pKernelBif->getProperty(pKernelBif, PDB_PROP_KBIF_FLR_SUPPORTED)))
+        {
+            NV_PRINTF(LEVEL_ERROR,
+                "FLR is NOT supported! Failing early in fullchip reset sequence\n");
+        }
+        else
+        {
+            NV_PRINTF(LEVEL_ERROR,
+                "FLR is force disabled using regkey/similar mechanism. Failing early.\n");
+        }
+        DBG_BREAKPOINT();
+        goto _bifPrepareForFullChipReset_GH100_exit;
+    }
+
+    pGpu->setProperty(pGpu, PDB_PROP_GPU_PREPARING_FULLCHIP_RESET, NV_TRUE);
+
+    if (pKernelNvlink && pKernelNvlink->getProperty(pKernelNvlink, PDB_PROP_KNVLINK_ENABLED))
+    {
+        if (knvlinkPrepareForXVEReset(pGpu, pKernelNvlink, NV_FALSE) != NV_OK)
+        {
+            NV_PRINTF(LEVEL_ERROR,
+                "NVLink prepare for fullchip reset failed.\n");
+            DBG_BREAKPOINT();
+            goto _bifPrepareForFullChipReset_GH100_exit;
+        }
+    }
+
+_bifPrepareForFullChipReset_GH100_exit:
+    pGpu->setProperty(pGpu, PDB_PROP_GPU_PREPARING_FULLCHIP_RESET, NV_FALSE);
+}
+
+NvU32
+kbifGetEccCounts_GH100
+(
+    OBJGPU *pGpu,
+    KernelBif *pKernelBif
+)
+{
+    NvU32 regVal;
+    NvU32 count = 0;
+
+    // PCIE RBUF
+    regVal = GPU_REG_RD32(pGpu, NV_XPL_BASE_ADDRESS + NV_XPL_DL_ERR_COUNT_RBUF);
+    count += DRF_VAL(_XPL_DL, _ERR_COUNT_RBUF, _UNCORR_ERR, regVal);
+
+    // PCIE SEQ_LUT
+    regVal = GPU_REG_RD32(pGpu, NV_XPL_BASE_ADDRESS + NV_XPL_DL_ERR_COUNT_SEQ_LUT);
+    count += DRF_VAL(_XPL_DL, _ERR_COUNT_SEQ_LUT, _UNCORR_ERR, regVal);
+
+    // PCIE XTL
+    regVal = GPU_REG_RD32(pGpu, NV_XTL_BASE_ADDRESS + NV_XTL_EP_PRI_DED_ERROR_STATUS);
+    if (regVal != 0)
+    {
+        count += 1;
+    }
+
+    // PCIE XTL
+    regVal = GPU_REG_RD32(pGpu, NV_XTL_BASE_ADDRESS + NV_XTL_EP_PRI_RAM_ERROR_INTR_STATUS);
+    if (regVal != 0)
+    {
+        count += 1;
+    }
+
+    return count;
+}
+
+/*!
+ * @brief Use Full Chip Reset to reset the chip.
+ *
+ * @param[in] pGpu       GPU object pointer
+ * @param[in] pKernelBif KernelBif object pointer
+ * 
+ * @return  NV_STATUS
+ */
+NV_STATUS
+kbifDoFullChipReset_GH100
+(
+    OBJGPU *pGpu,
+    KernelBif *pKernelBif
+)
+{
+    NV_STATUS  status     = NV_OK;
+    NvBool     bIsFLRSupportedAndEnabled;
+
+    // We support FLR for SKUs which are FLR capable.
+    bIsFLRSupportedAndEnabled = (pKernelBif->getProperty(pKernelBif, PDB_PROP_KBIF_FLR_SUPPORTED)) &&
+                                !(pKernelBif->bForceDisableFLR);
+
+    if (bIsFLRSupportedAndEnabled)
+    {
+        NV_ASSERT_OK(status = kbifDoFunctionLevelReset_HAL(pGpu, pKernelBif));
+        pGpu->setProperty(pGpu, PDB_PROP_GPU_IN_FULLCHIP_RESET, NV_TRUE);
+    }
+    else
+    {
+        NV_PRINTF(LEVEL_ERROR, "FLR is either not supported or is disabled.\n");
+    }
+
+    return status;
+}
+
+
+NvBool kbifAllowGpuReqPcieAtomics_GH100
+(
+    OBJGPU *pGpu,
+    KernelBif *pKernelBif
+)
+{
+    OBJSYS *pSys = SYS_GET_INSTANCE();
+    OBJCL  *pCl  = SYS_GET_CL(pSys);
+
+    return pCl->getProperty(pCl, PDB_PROP_CL_BUG_3562968_WAR_ALLOW_PCIE_ATOMICS);
+}
+
+/*!
+ * @brief Secondary Bus Reset implementation
+ *
+ * @param[in] pGpu  GPU object pointer
+ * @param[in] pKernelBif KernelBif object pointer
+ *
+ * @return  'NV_OK' if successful, an RM error code otherwise.
+ */
+NV_STATUS
+kbifDoSecondaryBusHotReset_GH100
+(
+    OBJGPU *pGpu,
+    KernelBif *pKernelBif
+)
+{
+    return kbifDoSecondaryBusHotReset_GM107(pGpu, pKernelBif);
 }
 
