@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2013-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2013-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -25,16 +25,24 @@
 /* ------------------------- System Includes -------------------------------- */
 #include "gpu/gpu.h"
 #include "gpu/bif/kernel_bif.h"
+#include "gpu/bus/kern_bus.h"
 #include "platform/chipset/chipset.h"
 #include "nvdevid.h"
+#include "nvmisc.h"
+#include "kernel/gpu/mc/kernel_mc.h"
+#include "kernel/gpu/nvlink/kernel_nvlink.h"
+#include "gpu/mem_sys/kern_mem_sys.h"
 
 #include "published/maxwell/gm107/dev_boot.h"
+#include "published/maxwell/gm107/dev_nv_xp.h"
 #include "published/maxwell/gm107/dev_nv_xve.h"
+#include "published/maxwell/gm107/dev_nv_xve_addendum.h"
 #include "published/maxwell/gm107/dev_nv_xve1.h"
-
 #include "published/maxwell/gm107/dev_nv_pcfg_xve_addendum.h"
 #include "published/maxwell/gm107/dev_nv_pcfg_xve1_addendum.h"
 
+#include "nvpcie.h"
+
 // Defines for C73 chipset registers
 #ifndef NV_XVR_VEND_XP1
 #define NV_XVR_VEND_XP1                                  0x00000F04 /* RW-4R */
@@ -94,14 +102,14 @@ kbifVerifyPcieConfigAccessTestRegisters_GM107
 {
     NvU32 data;
 
-    GPU_BUS_CFG_RD32(pGpu, NV_XVE_ID, &data);
+    NV_ASSERT_OK_OR_RETURN(GPU_BUS_CFG_RD32(pGpu, NV_XVE_ID, &data));
 
     if (FLD_TEST_DRF(_XVE, _ID, _VENDOR, _NVIDIA, data))
     {
         if (data != nvXveId)
             return NV_ERR_NOT_SUPPORTED;
 
-        GPU_BUS_CFG_RD32(pGpu, NV_XVE_VCCAP_HDR, &data);
+        NV_ASSERT_OK_OR_RETURN(GPU_BUS_CFG_RD32(pGpu, NV_XVE_VCCAP_HDR, &data));
 
         if (FLD_TEST_DRF(_XVE, _VCCAP_HDR, _ID, _VC, data) &&
             FLD_TEST_DRF(_XVE, _VCCAP_HDR, _VER, _1, data))
@@ -264,7 +272,7 @@ kbifPcieConfigEnableRelaxedOrdering_GM107
 {
     NvU32 xveDevCtrlStatus;
 
-    if(NV_ERR_GENERIC  == GPU_BUS_CFG_RD32(pGpu, NV_XVE_DEVICE_CONTROL_STATUS, &xveDevCtrlStatus))
+    if (NV_OK != GPU_BUS_CFG_RD32(pGpu, NV_XVE_DEVICE_CONTROL_STATUS, &xveDevCtrlStatus))
     {
         NV_PRINTF(LEVEL_ERROR,
                   "Unable to read NV_XVE_DEVICE_CONTROL_STATUS!\n");
@@ -292,7 +300,7 @@ kbifPcieConfigDisableRelaxedOrdering_GM107
 {
     NvU32 xveDevCtrlStatus;
 
-    if(NV_ERR_GENERIC  == GPU_BUS_CFG_RD32(pGpu, NV_XVE_DEVICE_CONTROL_STATUS, &xveDevCtrlStatus))
+    if (NV_OK != GPU_BUS_CFG_RD32(pGpu, NV_XVE_DEVICE_CONTROL_STATUS, &xveDevCtrlStatus))
     {
         NV_PRINTF(LEVEL_ERROR,
                   "Unable to read NV_XVE_DEVICE_CONTROL_STATUS!\n");
@@ -456,7 +464,9 @@ _kbifRestorePcieConfigRegisters_GM107
     RMTIMEOUT  timeout;
     NvBool     bGcxPmuCfgRestore;
 
-    bGcxPmuCfgRestore = pKernelBif->getProperty(pKernelBif, PDB_PROP_KBIF_GCX_PMU_CFG_SPACE_RESTORE);
+    {
+        bGcxPmuCfgRestore = pKernelBif->getProperty(pKernelBif, PDB_PROP_KBIF_GCX_PMU_CFG_SPACE_RESTORE);
+    }
 
     handle = osPciInitHandle(domain, bus, device, pRegmapRef->nFunc,
                              &vendorId, &deviceId);
@@ -1061,8 +1071,8 @@ kbifInitXveRegMap_GM107
         pKernelBif->xveRegmapRef[0].nFunc              = 0;
         pKernelBif->xveRegmapRef[0].xveRegMapValid     = xveRegMapValid;
         pKernelBif->xveRegmapRef[0].xveRegMapWrite     = xveRegMapWrite;
-        pKernelBif->xveRegmapRef[0].numXveRegMapValid  = sizeof(xveRegMapValid)/sizeof(xveRegMapValid[0]);
-        pKernelBif->xveRegmapRef[0].numXveRegMapWrite  = sizeof(xveRegMapWrite)/sizeof(xveRegMapWrite[0]);
+        pKernelBif->xveRegmapRef[0].numXveRegMapValid  = NV_ARRAY_ELEMENTS(xveRegMapValid);
+        pKernelBif->xveRegmapRef[0].numXveRegMapWrite  = NV_ARRAY_ELEMENTS(xveRegMapWrite);
         pKernelBif->xveRegmapRef[0].bufBootConfigSpace = pKernelBif->cacheData.gpuBootConfigSpace;
         // No MSIX for this GPU
         pKernelBif->xveRegmapRef[0].bufMsixTable       = NULL;
@@ -1072,8 +1082,8 @@ kbifInitXveRegMap_GM107
         pKernelBif->xveRegmapRef[1].nFunc              = 1;
         pKernelBif->xveRegmapRef[1].xveRegMapValid     = xve1RegMapValid;
         pKernelBif->xveRegmapRef[1].xveRegMapWrite     = xve1RegMapWrite;
-        pKernelBif->xveRegmapRef[1].numXveRegMapValid  = sizeof(xve1RegMapValid)/sizeof(xve1RegMapValid[0]);
-        pKernelBif->xveRegmapRef[1].numXveRegMapWrite  = sizeof(xve1RegMapWrite)/sizeof(xve1RegMapWrite[0]);
+        pKernelBif->xveRegmapRef[1].numXveRegMapValid  = NV_ARRAY_ELEMENTS(xve1RegMapValid);
+        pKernelBif->xveRegmapRef[1].numXveRegMapWrite  = NV_ARRAY_ELEMENTS(xve1RegMapWrite);
         pKernelBif->xveRegmapRef[1].bufBootConfigSpace = pKernelBif->cacheData.azaliaBootConfigSpace;
         // No MSIX for this func
         pKernelBif->xveRegmapRef[1].bufMsixTable       = NULL;
@@ -1231,6 +1241,109 @@ kbifRestoreBarsAndCommand_GM107
     return NV_OK;
 }
 
+/*
+*!
+ * @brief  Do any operations to get ready for a XVE sw reset.
+ *
+ * @param[in]  pGpu     GPU object pointer
+ * @param[in]  pBif     BIF object pointer
+ *
+ * @return  NV_STATUS
+ */
+NV_STATUS
+kbifPrepareForXveReset_GM107
+(
+    OBJGPU *    pGpu,
+    KernelBif *pKernelBif
+)
+{
+    NV_STATUS status;
+
+    return status = NV_OK;
+}
+
+
+/*!
+ * @brief  Do any work necessary to be able to do a full chip reset.
+ *
+ * This code needs to accomplish these objectives:
+ *
+ *  - Perform any operations needed to make sure that the HW will not glitch or
+ *  otherwise fail after the reset.  This includes all potential WARs.
+ *  - Perform any operations needed to make sure the devinit scripts can be
+ *  executed properly after the reset.
+ *  - Modify any SW state that could potentially cause errors during/after the reset.
+ *
+ * @param[in] pGpu       GPU object pointer
+ * @param[in] pKernelBif KernelBif object pointer
+ *
+ * @return  None
+ */
+void
+kbifPrepareForFullChipReset_GM107
+(
+    OBJGPU    *pGpu,
+    KernelBif *pKernelBif
+)
+{
+    KernelBus    *pKernelBus                = GPU_GET_KERNEL_BUS(pGpu);
+    KernelMc     *pKernelMc                 = GPU_GET_KERNEL_MC(pGpu);
+    KernelMemorySystem *pKernelMemorySystem = GPU_GET_KERNEL_MEMORY_SYSTEM(pGpu);
+    KernelNvlink *pKernelNvlink             = GPU_GET_KERNEL_NVLINK(pGpu);
+
+    pGpu->setProperty(pGpu, PDB_PROP_GPU_PREPARING_FULLCHIP_RESET, NV_TRUE);
+
+    //
+    // This function must be called before the pmu detach since engines need to
+    // be powered-up before we detach the PMU, and power-up sequence requires PMU
+    //
+    if (kmcPrepareForXVEReset_HAL(pGpu, pKernelMc) != NV_OK)
+    {
+        NV_PRINTF(LEVEL_ERROR, "MC prepare for XVE reset failed.\n");
+
+        DBG_BREAKPOINT();
+    }
+
+    // Set FB_IFACE to disable
+    if (kmemsysPrepareForXVEReset_HAL(pGpu, pKernelMemorySystem) != NV_OK)
+    {
+        NV_PRINTF(LEVEL_ERROR,
+                  "FB_IFACE disable for fullchip reset failed.\n");
+
+        DBG_BREAKPOINT();
+    }
+
+    // Disable P2P on all GPUs before RESET,
+    if (kbusPrepareForXVEReset_HAL(pGpu, pKernelBus) != NV_OK)
+    {
+        NV_PRINTF(LEVEL_ERROR, "BUS prepare for devinit failed.\n");
+
+        DBG_BREAKPOINT();
+    }
+
+    // Disable any NVLinks connected to this GPU
+    if ((pKernelNvlink!= NULL) && pKernelNvlink->getProperty(pKernelNvlink, PDB_PROP_KNVLINK_ENABLED))
+    {
+        if (knvlinkPrepareForXVEReset(pGpu, pKernelNvlink, NV_FALSE) != NV_OK)
+        {
+            NV_PRINTF(LEVEL_ERROR,
+                      "NVLINK prepare for fullchip reset failed.\n");
+
+            DBG_BREAKPOINT();
+        }
+    }
+
+    if (kbifPrepareForXveReset_HAL(pGpu, pKernelBif) != NV_OK)
+    {
+        NV_PRINTF(LEVEL_ERROR, "BIF prepare for devinit failed.\n");
+
+        DBG_BREAKPOINT();
+    }
+
+    pGpu->setProperty(pGpu, PDB_PROP_GPU_PREPARING_FULLCHIP_RESET, NV_FALSE);
+}
+
+
 /*!
  * @brief HAL specific BIF software state initialization
  *
@@ -1272,3 +1385,201 @@ kbifDestruct_GM107
 
 
 
+/*!
+ * @brief  Reset the chip.
+ *
+ * Use the XVE sw reset logic to reset as much of the chip as possible.
+ *
+ * @param[in]  pGpu       GPU object pointer
+ * @param[in]  pKernelBif KernelBif object pointer
+ *
+ * @return  NV_STATUS
+ */
+NV_STATUS
+kbifDoFullChipReset_GM107
+(
+    OBJGPU *pGpu,
+    KernelBif  *pKernelBif
+)
+{
+    OBJSYS *pSys  = SYS_GET_INSTANCE();
+    OBJCL *pCl = SYS_GET_CL(pSys);
+    NvU32  tempRegVal;
+    NvU32  oldPmc, newPmc;
+    NV_STATUS status;
+
+    // First Reset PMC
+    oldPmc = GPU_REG_RD32(pGpu, NV_PMC_ENABLE);
+    newPmc = oldPmc;
+
+    newPmc &= ~(DRF_DEF(_PMC, _ENABLE, _MSPPP, _ENABLE) |
+                DRF_DEF(_PMC, _ENABLE, _PMEDIA, _ENABLED) |
+                DRF_DEF(_PMC, _ENABLE, _CE0, _ENABLED) |
+                DRF_DEF(_PMC, _ENABLE, _CE1, _ENABLED) |
+                DRF_DEF(_PMC, _ENABLE, _PFIFO, _ENABLED) |
+                DRF_DEF(_PMC, _ENABLE, _PGRAPH, _ENABLED) |
+                DRF_DEF(_PMC, _ENABLE, _PWR, _ENABLED) |
+                DRF_DEF(_PMC, _ENABLE, _MSVLD, _ENABLED) |
+                DRF_DEF(_PMC, _ENABLE, _MSPDEC, _ENABLED) |
+                DRF_DEF(_PMC, _ENABLE, _PDISP, _ENABLED));
+
+    GPU_REG_WR32(pGpu, NV_PMC_ENABLE, newPmc);
+    GPU_REG_RD32(pGpu, NV_PMC_ENABLE);
+    GPU_REG_RD32(pGpu, NV_PMC_ENABLE);
+
+    //
+    // Before doing SW_RESET, init NV_XP_PL_CYA_1_BLOCK_HOST2XP_HOLD_LTSSM to 1.
+    // else when NV_XVE_SW_RESET to 0, host will try to do rom init and will assert
+    // HOST2XP_HOLD_LTSSM to 0 which will cause ltssm to goto detect.
+    //
+    tempRegVal = GPU_REG_RD32(pGpu, NV_XP_PL_CYA_1(0));
+    do
+    {
+        tempRegVal |= DRF_NUM(_XP, _PL_CYA_1, _BLOCK_HOST2XP_HOLD_LTSSM, 1);
+        GPU_REG_WR32(pGpu, NV_XP_PL_CYA_1(0), tempRegVal);
+        tempRegVal = GPU_REG_RD32(pGpu, NV_XP_PL_CYA_1(0));
+    } while ((tempRegVal & DRF_NUM(_XP, _PL_CYA_1, _BLOCK_HOST2XP_HOLD_LTSSM, 1)) == 0);
+
+    NV_ASSERT_OK_OR_RETURN(GPU_BUS_CFG_RD32(pGpu, NV_XVE_SW_RESET, &tempRegVal));
+    tempRegVal = FLD_SET_DRF(_XVE, _SW_RESET, _RESET, _ENABLE, tempRegVal);
+    clPcieWriteDword(pCl, gpuGetDomain(pGpu), gpuGetBus(pGpu),
+                     gpuGetDevice(pGpu), 0, NV_XVE_SW_RESET, tempRegVal);
+
+    pGpu->setProperty(pGpu, PDB_PROP_GPU_IN_FULLCHIP_RESET, NV_TRUE);
+
+    // wait a bit to make sure GPU is reset
+    osDelay(1);
+
+    //
+    // Come out of reset. Note that when SW/full-chip reset is triggered by the
+    // above write to NV_XVE_SW_RESET, BAR0 priv writes do not work and thus
+    // this write must be a PCI config bus write.
+    //
+    tempRegVal = FLD_SET_DRF(_XVE, _SW_RESET, _RESET, _DISABLE, tempRegVal);
+    clPcieWriteDword(pCl, gpuGetDomain(pGpu), gpuGetBus(pGpu),
+                     gpuGetDevice(pGpu), 0, NV_XVE_SW_RESET, tempRegVal);
+
+
+    //
+    // When bug 1511451 is present, SW_RESET will clear BAR3, and IO accesses
+    // will fail when legacy VBIOS is called. Apply the related SW WAR now.
+    //
+    status = kbifApplyWarForBug1511451_HAL(pGpu, pKernelBif);
+    if (status != NV_OK)
+    {
+        NV_PRINTF(LEVEL_ERROR, "Failed while applying WAR for Bug 1511451\n");
+        NV_ASSERT(0);
+    }
+
+    return status;
+}
+
+NV_STATUS
+kbifApplyWarForBug1511451_GM107
+(
+    OBJGPU    *pGpu,
+    KernelBif  *pKernelBif
+)
+{
+    NvU32      domain = gpuGetDomain(pGpu);
+    NvU8       bus    = gpuGetBus(pGpu);
+    NvU8       device = gpuGetDevice(pGpu);
+    NvU16      vendorId;
+    NvU16      deviceId;
+    void      *handle;
+
+    // If SBR is not supported, BAR3 will have not been saved and we are at risk.
+    if (!pKernelBif->getProperty(pKernelBif, PDB_PROP_KBIF_SECONDARY_BUS_RESET_SUPPORTED))
+    {
+        NV_PRINTF(LEVEL_ERROR,
+                  "SBR not supported so saved BAR3 is not valid, skipping restore!!!\n");
+        return NV_ERR_INVALID_STATE;
+    }
+
+    handle = osPciInitHandle(domain, bus, device, 0, &vendorId, &deviceId);
+
+    osPciWriteDword(handle, NV_XVE_BAR3, pKernelBif->cacheData.gpuBootConfigSpace[NV_XVE_BAR3/sizeof(NvU32)]);
+
+    return NV_OK;
+}
+
+/**
+ *
+ * @brief : This HAL always issues SBR
+ *
+ */
+NV_STATUS
+kbifDoSecondaryBusResetOrFunctionLevelReset_GM107
+(
+    OBJGPU *pGpu,
+    KernelBif *pKernelBif
+)
+{
+    // Since FLR is only supported for Turing and later, we simply issue SBR here
+    pGpu->setProperty(pGpu, PDB_PROP_GPU_IN_SECONDARY_BUS_RESET, NV_TRUE);
+    return kbifDoSecondaryBusHotReset_HAL(pGpu, pKernelBif);
+}
+
+
+NV_STATUS
+kbifDoSecondaryBusHotReset_GM107
+(
+    OBJGPU *pGpu,
+    KernelBif *pKernelBif
+)
+{
+    {
+        OBJCL  *pCl                 = SYS_GET_CL(SYS_GET_INSTANCE());
+        NBADDR *pUpstreamPort       = &pGpu->gpuClData.upstreamPort.addr;
+        void   *pUpstreamPortHandle = NULL;
+        NvU16   portVendorId;
+        NvU16   portDeviceId;
+        NvU16   data;
+
+        pUpstreamPortHandle = osPciInitHandle(pUpstreamPort->domain,
+                                              pUpstreamPort->bus,
+                                              pUpstreamPort->device,
+                                              0,
+                                              &portVendorId,
+                                              &portDeviceId);
+
+        //
+        // Set secondary bus reset bit, triggering a hot reset on the port the
+        // GPU is plugged into.
+        //
+        data = osPciReadWord(pUpstreamPortHandle, PCI_HEADER_TYPE1_BRIDGE_CONTROL);
+        clPcieWriteWord(pCl,
+                        pGpu->gpuClData.upstreamPort.addr.domain,
+                        pGpu->gpuClData.upstreamPort.addr.bus,
+                        pGpu->gpuClData.upstreamPort.addr.device,
+                        pGpu->gpuClData.upstreamPort.addr.func,
+                        PCI_HEADER_TYPE1_BRIDGE_CONTROL,
+                        (data | 0x40));
+
+        //
+        // We must must ensure a minimum reset duration (Trst) of 1ms, as defined
+        // in the PCI Local Bus Specification, Revision 3.0
+        //
+        osDelayUs(1000);
+
+        // Clear secondary bus reset bit, bringing the port out of hot reset.
+        data &= 0xFFBF;
+        clPcieWriteWord(pCl,
+                        pGpu->gpuClData.upstreamPort.addr.domain,
+                        pGpu->gpuClData.upstreamPort.addr.bus,
+                        pGpu->gpuClData.upstreamPort.addr.device,
+                        pGpu->gpuClData.upstreamPort.addr.func,
+                        PCI_HEADER_TYPE1_BRIDGE_CONTROL,
+                        data);
+        //
+        // We must wait at least 100 ms from the end of the reset before it is
+        // permitted to issue Configuration Requests to the GPU.
+        //
+        osDelayUs(100000);
+    }
+
+    return kbifWaitForConfigAccessAfterReset(pGpu, pKernelBif);
+}
+
+
+
