@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -23,6 +23,7 @@
 
 #include "core/core.h"
 #include "gpu/gpu.h"
+#include "gpu/device/device.h"
 #include "mem_mgr/vaspace.h"
 #include "mem_mgr/io_vaspace.h"
 #include "mem_mgr/gpu_vaspace.h"
@@ -33,13 +34,14 @@
 #include "kernel/gpu/nvbitmask.h"
 #include "platform/chipset/chipset.h"
 #include "rmapi/client.h"
+#include "platform/sli/sli.h"
 #include "nvdevid.h"
+#include "containers/eheap_old.h"
 
-#include "gpu/subdevice/subdevice.h"
 #include "gpu/gsp/gsp_static_config.h"
 #include "vgpu/rpc.h"
 
-#include "nvRmReg.h"
+#include "nvrm_registry.h"
 
 static NV_STATUS kbusInitRegistryOverrides(OBJGPU *pGpu, KernelBus *pKernelBus);
 
@@ -181,7 +183,7 @@ kbusInitRegistryOverrides(OBJGPU *pGpu, KernelBus *pKernelBus)
             break;
     }
 
-    if (RMCFG_FEATURE_PLATFORM_WINDOWS_LDDM && !pGpu->getProperty(pGpu, PDB_PROP_GPU_IN_TCC_MODE))
+    if (RMCFG_FEATURE_PLATFORM_WINDOWS && !pGpu->getProperty(pGpu, PDB_PROP_GPU_IN_TCC_MODE))
     {
         //
         // Aligns to unlinked SLI: Volta and up
@@ -201,6 +203,12 @@ kbusInitRegistryOverrides(OBJGPU *pGpu, KernelBus *pKernelBus)
         pKernelBus->setProperty(pKernelBus, PDB_PROP_KBUS_RESTORE_BAR1_SIZE_BUG_3249028_WAR, !!data32);
     }
 
+    if (osReadRegistryDword(pGpu, NV_REG_STR_RM_FORCE_BAR_ACCESS_ON_HCC, &data32) == NV_OK &&
+        data32 == NV_REG_STR_RM_FORCE_BAR_ACCESS_ON_HCC_YES  && gpuIsCCDevToolsModeEnabled(pGpu))
+    {
+        pKernelBus->bForceBarAccessOnHcc = NV_TRUE;
+    }
+
     return NV_OK;
 }
 
@@ -312,12 +320,31 @@ kbusDestruct_IMPL(KernelBus *pKernelBus)
 {
     OBJGPU *pGpu = ENG_GET_GPU(pKernelBus);
 
+    memdescFree(pKernelBus->bar1[GPU_GFID_PF].pInstBlkMemDesc);
+    memdescDestroy(pKernelBus->bar1[GPU_GFID_PF].pInstBlkMemDesc);
+    pKernelBus->bar1[GPU_GFID_PF].pInstBlkMemDesc = NULL;
+    memdescFree(pKernelBus->bar2[GPU_GFID_PF].pInstBlkMemDesc);
+    memdescDestroy(pKernelBus->bar2[GPU_GFID_PF].pInstBlkMemDesc);
+    pKernelBus->bar2[GPU_GFID_PF].pInstBlkMemDesc = NULL;
+
     //
     // We need to clean-up the memory resources for BAR2 as late as possible,
     // and after all memory descriptors have been reclaimed.
     //
     kbusDestructVirtualBar2_HAL(pGpu, pKernelBus, NV_TRUE, GPU_GFID_PF);
 
+    // Unmap BAR2 mapping that was retained for RPC
+    if (IS_VGPU_GSP_PLUGIN_OFFLOAD_ENABLED(pGpu) &&
+        IS_VIRTUAL_WITH_SRIOV(pGpu) &&
+        pKernelBus->virtualBar2[GPU_GFID_PF].pCpuMapping != NULL)
+    {
+        // vgpuDestructObject should have been run
+        OBJVGPU *pVGpu = GPU_GET_VGPU(pGpu);
+        NV_ASSERT(pVGpu == NULL);
+
+        osUnmapPciMemoryKernelOld(pGpu, pKernelBus->virtualBar2[GPU_GFID_PF].pCpuMapping);
+    }
+
     return;
 }
 
@@ -613,7 +640,7 @@ kbusCpuOffsetInBar2WindowGet_IMPL
  * @return VA limit of BAR2
  */
 NvU64
-kbusGetVaLimitForBar2_KERNEL
+kbusGetVaLimitForBar2_FWCLIENT
 (
     OBJGPU    *pGpu,
     KernelBus *pKernelBus
@@ -632,6 +659,56 @@ kbusGetVaLimitForBar2_KERNEL
     return limit;
 }
 
+/*!
+ * @brief Calculates the BAR2 VA limit (in Byte units)
+ * Can be safely called only after kbusSetBarsApertureSize_HAL is executed.
+ */
+NvU64 kbusGetVaLimitForBar2_IMPL
+(
+    OBJGPU    *pGpu,
+    KernelBus *pKernelBus
+)
+{
+    NvU64 limit = 0;
+    NvU32 gfid;
+
+    NV_ASSERT_OR_RETURN(vgpuGetCallingContextGfid(pGpu, &gfid) == NV_OK, 0);
+
+    //
+    // Return zero from the guest in the paravirtualization case or
+    // if guest is running in SRIOV heavy mode.
+    //
+    if (IS_VIRTUAL_WITHOUT_SRIOV(pGpu) ||
+        (IS_VIRTUAL(pGpu) && gpuIsWarBug200577889SriovHeavyEnabled(pGpu)))
+    {
+        return 0;
+    }
+
+    kbusCalcCpuInvisibleBar2Range_HAL(pGpu, pKernelBus, gfid);
+
+    //
+    // we are just accounting here for possibility that
+    // we are on pre_PASCAL and so we set the va limit
+    // to account only for the cpuVisible Aperture
+    //
+    if (pKernelBus->bar2[gfid].cpuInvisibleLimit > pKernelBus->bar2[gfid].cpuInvisibleBase)
+    {
+        limit = pKernelBus->bar2[gfid].cpuInvisibleLimit;
+    }
+    else
+    {
+        limit = pKernelBus->bar2[gfid].cpuVisibleLimit;
+    }
+    NV_PRINTF(LEVEL_INFO, "va limit: 0x%llx\n", limit);
+    //
+    // pKernelBus->bar2.vaLimit is set by this function.
+    // Assert to ensure that this value doesn't get changed.
+    //
+    NV_ASSERT(pKernelBus->bar2[gfid].vaLimit == 0 || pKernelBus->bar2[gfid].vaLimit == limit);
+
+    return limit;
+}
+
 /*!
  * Patch CPU-RM's SW cache of BAR1 PDB to GSP-RM's BAR1 PDB so that CPU-RM can
  * do TLB invalidation to correct VA space.
@@ -655,7 +732,7 @@ kbusPatchBar1Pdb_GSPCLIENT
     MEMORY_DESCRIPTOR   *pMemDesc  = NULL;
     GVAS_GPU_STATE      *pGpuState = gvaspaceGetGpuState(pGVAS, pGpu);
     const MMU_FMT_LEVEL *pRootFmt  = pGpuState->pFmt->pRoot;
-    NvU32                rootSize  = pRootFmt->entrySize;
+    NvU32                rootSize  = mmuFmtLevelSize(pRootFmt);
     MMU_WALK_USER_CTX    userCtx   = {0};
     GspStaticConfigInfo *pGSCI     = GPU_GET_GSP_STATIC_INFO(pGpu);
 
@@ -666,7 +743,9 @@ kbusPatchBar1Pdb_GSPCLIENT
     memdescDescribe(pMemDesc, ADDR_FBMEM, pGSCI->bar1PdeBase, rootSize);
     memdescSetPageSize(pMemDesc, VAS_ADDRESS_TRANSLATION(pKernelBus->bar1[GPU_GFID_PF].pVAS), RM_PAGE_SIZE);
 
-    gvaspaceWalkUserCtxAcquire(pGVAS, pGpu, NULL, &userCtx);
+    NV_ASSERT_OK_OR_GOTO(status,
+        gvaspaceWalkUserCtxAcquire(pGVAS, pGpu, NULL, &userCtx),
+        done);
 
     //
     // Modify the CPU-RM's walker state with the new backing memory.
@@ -676,18 +755,22 @@ kbusPatchBar1Pdb_GSPCLIENT
                                         pRootFmt,
                                         vaspaceGetVaStart(pKernelBus->bar1[GPU_GFID_PF].pVAS),
                                         (MMU_WALK_MEMDESC*)pMemDesc,
-                                        mmuFmtLevelSize(pRootFmt),
+                                        rootSize,
                                         NV_TRUE,
                                         NV_TRUE,
                                         NV_FALSE);
     gvaspaceWalkUserCtxRelease(pGVAS, &userCtx);
-    if (NV_OK != status)
+
+done:
+    if (status != NV_OK)
     {
         NV_PRINTF(LEVEL_ERROR, "Failed to modify CPU-RM's BAR1 PDB to GSP-RM's BAR1 PDB.\n");
-        return status;
+        memdescDestroy(pMemDesc);
+    }
+    else
+    {
+        gvaspaceInvalidateTlb(pGVAS, pGpu, PTE_DOWNGRADE);
     }
-
-    gvaspaceInvalidateTlb(pGVAS, pGpu, PTE_DOWNGRADE);
 
     return status;
 }
@@ -767,6 +850,16 @@ kbusPatchBar2Pdb_GSPCLIENT
     return NV_OK;
 }
 
+NvBool
+kbusCheckEngine_KERNEL
+(
+    OBJGPU        *pGpu,
+    KernelBus     *pKernelBus,
+    ENGDESCRIPTOR  engDesc
+)
+{
+    return kbusCheckEngineWithOrderList_KERNEL(pGpu, pKernelBus, engDesc, NV_TRUE);
+}
 
 /*!
  * @brief Checks whether an engine is available or not.
@@ -776,23 +869,25 @@ kbusPatchBar2Pdb_GSPCLIENT
  * rely on timeouts after a read of a register in the reg space for engine.
  * Instead, it
  *  - Return TRUE for all engines which are must present in GPU.
- *  - Get information about CE, MSENC, NVJPG and OFA engines from plugin.
- *  - Rest engines are determined from HAL creation data.
+ *  - Get information about CE, MSENC, NVJPG and OFA engines from plugin or GSP-RM.
+ *  - If bCheckEngineOrder is true, the remaining engines are searched for in gpuChildOrderList_HAL.
  *
  * @param[in] pGpu       OBJGPU pointer
  * @param[in] pKernelBus KernelBus pointer
  * @param[in] engDesc    ENGDESCRIPTOR pointer used to check Engine presence
+ * @param[in] bCheckEngineOrder If true, check gpuChildOrderList_HAL for engDesc as well
  *
  * @returns NV_TRUE if engine is available.
  *          NV_FALSE if engine is not available or floorswept.
  *
  */
 NvBool
-kbusCheckEngine_KERNEL
+kbusCheckEngineWithOrderList_KERNEL
 (
     OBJGPU        *pGpu,
     KernelBus     *pKernelBus,
-    ENGDESCRIPTOR  engDesc
+    ENGDESCRIPTOR  engDesc,
+    NvBool         bCheckEngineOrder
 )
 {
     NvU32     rmEngineCaps[NVGPU_ENGINE_CAPS_MASK_ARRAY_MAX] = {0};
@@ -800,6 +895,31 @@ kbusCheckEngine_KERNEL
     NvBool    bSupported;
     NV_STATUS status;
 
+    if (IS_VIRTUAL(pGpu))
+    {
+        NvU32             i     = 0;
+        NvU32             j     = 0;
+        VGPU_STATIC_INFO *pVSI  = GPU_GET_STATIC_INFO(pGpu);
+
+        if (pVSI == NULL)
+        {
+            return NV_FALSE;
+        }
+
+        //
+        // vGPU Enginelist can accommodate maximum of NVGPU_VGPU_ENGINE_LIST_LAST engine mask.
+        // Currently, vGPU plugin advertises the same in NvU64 bitmask.
+        //
+        ct_assert(RM_ENGINE_TYPE_LAST <= NVGPU_VGPU_ENGINE_LIST_LAST);
+        ct_assert(NVGPU_ENGINE_CAPS_MASK_ARRAY_MAX <= (NVGPU_VGPU_ENGINE_LIST_MASK_ARRAY_MAX * 2));
+
+        for (i = 0; i < (NVGPU_ENGINE_CAPS_MASK_ARRAY_MAX/2); i++)
+        {
+            nv2080EngineCaps[j++] = NvU64_LO32(pVSI->engineList[i]);
+            nv2080EngineCaps[j++] = NvU64_HI32(pVSI->engineList[i]);
+        }
+    }
+    else
     {
         NvU32 i;
         GspStaticConfigInfo *pGSCI = GPU_GET_GSP_STATIC_INFO(pGpu);
@@ -863,6 +983,17 @@ kbusCheckEngine_KERNEL
         case ENG_CE(7):
         case ENG_CE(8):
         case ENG_CE(9):
+            // Bug 3748354
+        case ENG_CE(10):
+        case ENG_CE(11):
+        case ENG_CE(12):
+        case ENG_CE(13):
+        case ENG_CE(14):
+        case ENG_CE(15):
+        case ENG_CE(16):
+        case ENG_CE(17):
+        case ENG_CE(18):
+        case ENG_CE(19):
             return !!NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps,
                 RM_ENGINE_TYPE_COPY(GET_CE_IDX(engDesc)));
 
@@ -886,9 +1017,10 @@ kbusCheckEngine_KERNEL
                 RM_ENGINE_TYPE_NVDEC(GET_NVDEC_IDX(engDesc)));
 
    case ENG_OFA(0):
+   case ENG_OFA(1):
             return !!NVGPU_GET_ENGINE_CAPS_MASK(rmEngineCaps,
                 RM_ENGINE_TYPE_OFA(GET_OFA_IDX(engDesc)));
-        
+
         case ENG_NVJPEG(0):
         case ENG_NVJPEG(1):
         case ENG_NVJPEG(2):
@@ -929,15 +1061,23 @@ kbusCheckEngine_KERNEL
         // HAL lists of GPU. So Add ASSERT there.
         //
         default:
-            bSupported = gpuIsEngDescSupported(pGpu, engDesc);
-
-            if (!bSupported)
+        {
+            if (bCheckEngineOrder)
             {
-                NV_PRINTF(LEVEL_ERROR, "Unable to check engine ID: %d\n",
-                          engDesc);
-                NV_ASSERT(bSupported);
+                bSupported = gpuIsEngDescSupported(pGpu, engDesc);
+
+                if (!bSupported)
+                {
+                    NV_PRINTF(LEVEL_ERROR, "Unable to check engine ID: 0x%x\n",
+                              engDesc);
+                    NV_ASSERT(bSupported);
+                }
             }
+            else
+                bSupported = NV_FALSE;
+
             return bSupported;
+        }
     }
 }
 
@@ -983,9 +1123,6 @@ kbusGetDeviceCaps_IMPL
         RMCTRL_SET_CAP(tempCaps, NV0080_CTRL_HOST_CAPS, _CPU_WRITE_WAR_BUG_420495);
     }
 
-    // the RM always supports GPU-coherent mappings
-    RMCTRL_SET_CAP(tempCaps, NV0080_CTRL_HOST_CAPS, _GPU_COHERENT_MAPPING_SUPPORTED);
-
     // If we don't have existing caps with which to reconcile, then just return
     if (!bCapsInitialized)
     {
@@ -993,44 +1130,42 @@ kbusGetDeviceCaps_IMPL
         return;
     }
 
-    // factor in this GPUs caps: all these are feature caps, so use AND
-    RMCTRL_AND_CAP(pHostCaps, tempCaps, temp,
-                   NV0080_CTRL_HOST_CAPS, _P2P_4_WAY);
-    RMCTRL_AND_CAP(pHostCaps, tempCaps, temp,
-                   NV0080_CTRL_HOST_CAPS, _P2P_8_WAY);
-    RMCTRL_AND_CAP(pHostCaps, tempCaps, temp,
-                   NV0080_CTRL_HOST_CAPS, _GPU_COHERENT_MAPPING_SUPPORTED);
-
-    RMCTRL_OR_CAP(pHostCaps, tempCaps, temp,
-                  NV0080_CTRL_HOST_CAPS, _SEMA_ACQUIRE_BUG_105665);
-    RMCTRL_OR_CAP(pHostCaps, tempCaps, temp,
-                  NV0080_CTRL_HOST_CAPS, _SYS_SEMA_DEADLOCK_BUG_148216);
-    RMCTRL_OR_CAP(pHostCaps, tempCaps, temp,
-                  NV0080_CTRL_HOST_CAPS, _SLOWSLI);
-    RMCTRL_OR_CAP(pHostCaps, tempCaps, temp,
-                  NV0080_CTRL_HOST_CAPS, _SEMA_READ_ONLY_BUG);
-    RMCTRL_OR_CAP(pHostCaps, tempCaps, temp,
-                  NV0080_CTRL_HOST_CAPS, _MEM2MEM_BUG_365782);
-    RMCTRL_OR_CAP(pHostCaps, tempCaps, temp,
-                  NV0080_CTRL_HOST_CAPS, _LARGE_NONCOH_UPSTR_WRITE_BUG_114871);
-    RMCTRL_OR_CAP(pHostCaps, tempCaps, temp,
-                  NV0080_CTRL_HOST_CAPS, _LARGE_UPSTREAM_WRITE_BUG_115115);
-    RMCTRL_OR_CAP(pHostCaps, tempCaps, temp,
-                  NV0080_CTRL_HOST_CAPS, _SEP_VIDMEM_PB_NOTIFIERS_BUG_83923);
-    RMCTRL_OR_CAP(pHostCaps, tempCaps, temp,
-                  NV0080_CTRL_HOST_CAPS, _P2P_DEADLOCK_BUG_203825);
-    RMCTRL_OR_CAP(pHostCaps, tempCaps, temp,
-                  NV0080_CTRL_HOST_CAPS, _COMPRESSED_BL_P2P_BUG_257072);
-    RMCTRL_OR_CAP(pHostCaps, tempCaps, temp,
-                  NV0080_CTRL_HOST_CAPS, _CROSS_BLITS_BUG_270260);
     RMCTRL_OR_CAP(pHostCaps, tempCaps, temp,
                   NV0080_CTRL_HOST_CAPS, _CPU_WRITE_WAR_BUG_420495);
-    RMCTRL_OR_CAP(pHostCaps, tempCaps, temp,
-                  NV0080_CTRL_HOST_CAPS, _BAR1_READ_DEADLOCK_BUG_511418);
 
     return;
 }
 
+NV_STATUS
+kbusMapFbApertureSingle_IMPL
+(
+    OBJGPU *pGpu,
+    KernelBus *pKernelBus,
+    MEMORY_DESCRIPTOR *pMemDesc,
+    NvU64 offset,
+    NvU64 *pAperOffset,
+    NvU64 *pLength,
+    NvU32 flags,
+    Device *pDevice
+)
+{
+    return kbusMapFbAperture_HAL(pGpu, pKernelBus, pMemDesc, offset, pAperOffset, pLength, flags, pDevice);
+}
+    
+NV_STATUS
+kbusUnmapFbApertureSingle_IMPL
+(
+    OBJGPU *pGpu,
+    KernelBus *pKernelBus,
+    MEMORY_DESCRIPTOR *pMemDesc,
+    NvU64 aperOffset,
+    NvU64 length,
+    NvU32 flags
+)
+{
+    return kbusUnmapFbAperture_HAL(pGpu, pKernelBus, pMemDesc, aperOffset, length, flags);
+}
+
 NV_STATUS
 kbusMapFbApertureByHandle_IMPL
 (
@@ -1073,9 +1208,9 @@ kbusMapFbApertureByHandle_IMPL
         return NV_ERR_INVALID_ARGUMENT;
     }
 
-    status = kbusMapFbAperture_HAL(pGpu, pKernelBus, pMemDesc, offset,
-                                   &fbApertureOffset, &fbApertureLength,
-                                   BUS_MAP_FB_FLAGS_MAP_UNICAST, pDevice);
+    status = kbusMapFbApertureSingle(pGpu, pKernelBus, pMemDesc, offset,
+                                     &fbApertureOffset, &fbApertureLength,
+                                     BUS_MAP_FB_FLAGS_MAP_UNICAST, pDevice);
     if (status != NV_OK)
     {
         return status;
@@ -1101,10 +1236,10 @@ kbusMapFbApertureByHandle_IMPL
     return NV_OK;
 
 failed:
-    // Note: fbApertureLength is not used by kbusUnmapFbAperture_HAL(), so it's passed as 0
-    kbusUnmapFbAperture_HAL(pGpu, pKernelBus, pMemDesc,
-                            fbApertureOffset, 0,
-                            BUS_MAP_FB_FLAGS_MAP_UNICAST);
+    // Note: fbApertureLength is not used by kbusUnmapFbApertureSingle(), so it's passed as 0
+    kbusUnmapFbApertureSingle(pGpu, pKernelBus, pMemDesc,
+                              fbApertureOffset, 0,
+                              BUS_MAP_FB_FLAGS_MAP_UNICAST);
 
     return status;
 }
@@ -1141,10 +1276,10 @@ kbusUnmapFbApertureByHandle_IMPL
 
     pMemDesc = pSrcMemory->pMemDesc;
 
-    // Note: fbApertureLength is not used by kbusUnmapFbAperture_HAL(), so it's passed as 0
-    status = kbusUnmapFbAperture_HAL(pGpu, pKernelBus, pMemDesc,
-                                     bar1Va - gpumgrGetGpuPhysFbAddr(pGpu),
-                                     0, BUS_MAP_FB_FLAGS_MAP_UNICAST);
+    // Note: fbApertureLength is not used by kbusUnmapFbApertureSingle(), so it's passed as 0
+    status = kbusUnmapFbApertureSingle(pGpu, pKernelBus, pMemDesc,
+                                       bar1Va - gpumgrGetGpuPhysFbAddr(pGpu),
+                                       0, BUS_MAP_FB_FLAGS_MAP_UNICAST);
     if (status != NV_OK)
     {
         return status;
@@ -1172,20 +1307,33 @@ kbusSendBusInfo_IMPL
 )
 {
     NV_STATUS status = NV_OK;
-    NV2080_CTRL_BUS_GET_INFO_V2_PARAMS busGetInfoParams = {0};
 
-    busGetInfoParams.busInfoList[0] = *pBusInfo;
-    busGetInfoParams.busInfoListSize = 1;
+    if (IS_VIRTUAL(pGpu))
+    {
+        VGPU_STATIC_INFO *pVSI = GPU_GET_STATIC_INFO(pGpu);
 
-    NV_RM_RPC_CONTROL(pGpu,
-                      pGpu->hInternalClient,
-                      pGpu->hInternalSubdevice,
-                      NV2080_CTRL_CMD_BUS_GET_INFO_V2,
-                      &busGetInfoParams,
-                      sizeof(busGetInfoParams),
-                      status);
+        NV_ASSERT_OR_RETURN(pVSI != NULL, NV_ERR_INVALID_STATE);
+
+        pBusInfo->data = pVSI->busGetInfoV2.busInfoList[pBusInfo->index].data;
+    }
+    else
+    {
+        RM_API *pRmApi = GPU_GET_PHYSICAL_RMAPI(pGpu);
+        NV2080_CTRL_BUS_GET_INFO_V2_PARAMS busGetInfoParams = {0};
+
+        busGetInfoParams.busInfoList[0] = *pBusInfo;
+        busGetInfoParams.busInfoListSize = 1;
+
+        status = pRmApi->Control(pRmApi,
+                                 pGpu->hInternalClient,
+                                 pGpu->hInternalSubdevice,
+                                 NV2080_CTRL_CMD_BUS_GET_INFO_V2,
+                                 &busGetInfoParams,
+                                 sizeof(busGetInfoParams));
+
+        pBusInfo->data = busGetInfoParams.busInfoList[0].data;
+    }
 
-    pBusInfo->data = busGetInfoParams.busInfoList[0].data;
     return status;
 }
 
@@ -1266,3 +1414,68 @@ kbusIsGpuP2pAlive_IMPL
 {
     return (pKernelBus->totalP2pObjectsAliveRefCount > 0);
 }
+
+/**
+ * @brief     Gets BAR0 size in bytes for VF
+ *
+ * @param[in] pGpu
+ * @param[in] pKernelBus
+ */
+NvU64 kbusGetVfBar0SizeBytes_IMPL
+(
+    OBJGPU    *pGpu,
+    KernelBus *pKernelBus
+)
+{
+    // Bar 0 size for VF is always 16MB
+    return 16llu << 20;
+}
+
+/**
+ * @brief Update BAR1 size and availability in RUSD
+ *
+ * @param[in] pGpu
+ */
+NV_STATUS
+kbusUpdateRusdStatistics_IMPL
+(
+    OBJGPU *pGpu
+)
+{
+    KernelBus *pKernelBus = GPU_GET_KERNEL_BUS(pGpu);
+    OBJVASPACE *pBar1VAS;
+    OBJEHEAP *pVASHeap;
+    RUSD_BAR1_MEMORY_INFO *pSharedData;
+    NvU64 bar1Size = 0;
+    NvU64 bar1AvailSize = 0;
+    NV_RANGE bar1VARange = NV_RANGE_EMPTY;
+    NvBool bZeroRusd = kbusIsBar1Disabled(pKernelBus);
+
+    bZeroRusd = bZeroRusd || IS_MIG_ENABLED(pGpu);
+
+    if (!bZeroRusd)
+    {
+        pBar1VAS = kbusGetBar1VASpace_HAL(pGpu, pKernelBus);
+        NV_ASSERT_OR_RETURN(pBar1VAS != NULL, NV_ERR_INVALID_STATE);
+        pVASHeap = vaspaceGetHeap(pBar1VAS);
+        bar1VARange = rangeMake(vaspaceGetVaStart(pBar1VAS), vaspaceGetVaLimit(pBar1VAS));
+
+        bar1Size = (NvU32)(rangeLength(bar1VARange) / 1024);
+
+        if (pVASHeap != NULL)
+        {
+            NvU64 freeSize = 0;
+
+            pVASHeap->eheapInfoForRange(pVASHeap, bar1VARange, NULL, NULL, NULL, &freeSize);
+            bar1AvailSize = (NvU32)(freeSize / 1024);
+        }
+    }
+
+    // Minimize critical section, write data once we have it.
+    pSharedData = gpushareddataWriteStart(pGpu, bar1MemoryInfo);
+    pSharedData->bar1Size = bar1Size;
+    pSharedData->bar1AvailSize = bar1AvailSize;
+    gpushareddataWriteFinish(pGpu, bar1MemoryInfo);
+
+    return NV_OK;
+}
