@@ -67,8 +67,11 @@ typedef cryptoBundle_t keySlot_t[CC_KEYSPACE_TOTAL_SIZE];
 static NV_STATUS checkSlot(ConfidentialCompute *pConfCompute, NvU32 slotNumber);
 static void incrementChannelCounter(ConfidentialCompute *pConfCompute, NvU32 slotNumber);
 static NvU64 getChannelCounter(ConfidentialCompute *pConfCompute, NvU32 slotNumber);
+static NV_STATUS getKeyIdLce(KernelChannel *pKernelChannel, ROTATE_IV_TYPE rotateOperation,
+                             NvU16 *keyId);
 static NV_STATUS getKeyIdSec2(KernelChannel *pKernelChannel, ROTATE_IV_TYPE rotateOperation,
                               NvU16 *keyId);
+static NV_STATUS getKeyspaceLce(KernelChannel *pKernelChannel, NvU16 *keyspace);
 
 NV_STATUS
 confComputeKeyStoreInit_GH100(ConfidentialCompute *pConfCompute)
@@ -124,6 +127,7 @@ confComputeKeyStoreDeinit_GH100(ConfidentialCompute *pConfCompute)
         portMemSet(pConfCompute->m_keySlot, 0, (NvLength) sizeof(keySlot_t));
         confComputeKeyStoreClearExportMasterKey_HAL(pConfCompute);
         portMemFree(pConfCompute->m_keySlot);
+        pConfCompute->m_keySlot = NULL;
     }
 }
 
@@ -173,7 +177,7 @@ confComputeKeyStoreDeriveKey_GH100(ConfidentialCompute *pConfCompute, NvU32 glob
 
     // LCEs will return an error / interrupt if the key is all 0s.
     if ((CC_GKEYID_GET_KEYSPACE(globalKeyId) >= CC_KEYSPACE_LCE0) &&
-        (CC_GKEYID_GET_KEYSPACE(globalKeyId) <= confComputeGetMaxCeKeySpaceIdx(pConfCompute)))
+        (CC_GKEYID_GET_KEYSPACE(globalKeyId) <= CC_KEYSPACE_LCE7))
     {
         for (NvU32 index = 0; index < CC_AES_256_GCM_KEY_SIZE_DWORD; index++)
         {
@@ -232,21 +236,19 @@ confComputeKeyStoreRetrieveViaChannel_GH100
 
     if (RM_ENGINE_TYPE_IS_COPY(kchannelGetEngineType(pKernelChannel)))
     {
-        NvU16 keySpace;
+        NvU16 keyspace;
 
-        if (confComputeGetKeySpaceFromKChannel_HAL(pConfCompute, pKernelChannel,
-                                                   &keySpace) != NV_OK)
+        if (getKeyspaceLce(pKernelChannel, &keyspace) != NV_OK)
         {
             return NV_ERR_INVALID_PARAMETER;
         }
 
-        if (confComputeGetLceKeyIdFromKChannel_HAL(pConfCompute, pKernelChannel,
-                                                   rotateOperation, &keyId) != NV_OK)
+        if (getKeyIdLce(pKernelChannel, rotateOperation, &keyId) != NV_OK)
         {
             return NV_ERR_INVALID_PARAMETER;
         }
 
-        globalKeyId = CC_GKEYID_GEN(keySpace, keyId);
+        globalKeyId = CC_GKEYID_GEN(keyspace, keyId);
     }
     else if (kchannelGetEngineType(pKernelChannel) == RM_ENGINE_TYPE_SEC2)
     {
@@ -432,7 +434,7 @@ confComputeKeyStoreUpdateKey_GH100(ConfidentialCompute *pConfCompute, NvU32 glob
 
     // LCEs will return an error / interrupt if the key is all 0s.
     if ((CC_GKEYID_GET_KEYSPACE(globalKeyId) >= CC_KEYSPACE_LCE0) &&
-        (CC_GKEYID_GET_KEYSPACE(globalKeyId) <= confComputeGetMaxCeKeySpaceIdx(pConfCompute)))
+        (CC_GKEYID_GET_KEYSPACE(globalKeyId) <= CC_KEYSPACE_LCE7))
     {
         for (NvU32 index = 0; index < CC_AES_256_GCM_KEY_SIZE_DWORD; index++)
         {
@@ -479,12 +481,9 @@ confComputeGetKeyPairByChannel_GH100
     }
     else
     {
-        NV_ASSERT_OK_OR_RETURN(confComputeGetKeySpaceFromKChannel_HAL(pConfCompute,
-                                    pKernelChannel, &keySpace) != NV_OK);
-        NV_ASSERT_OK_OR_RETURN(confComputeGetLceKeyIdFromKChannel_HAL(pConfCompute, pKernelChannel,
-                                    ROTATE_IV_ENCRYPT, &lh2dKeyId));
-        NV_ASSERT_OK_OR_RETURN(confComputeGetLceKeyIdFromKChannel_HAL(pConfCompute, pKernelChannel,
-                                    ROTATE_IV_DECRYPT, &ld2hKeyId));
+        NV_ASSERT_OK_OR_RETURN(getKeyspaceLce(pKernelChannel, &keySpace));
+        NV_ASSERT_OK_OR_RETURN(getKeyIdLce(pKernelChannel, ROTATE_IV_ENCRYPT, &lh2dKeyId));
+        NV_ASSERT_OK_OR_RETURN(getKeyIdLce(pKernelChannel, ROTATE_IV_DECRYPT, &ld2hKeyId));
     }
 
     if (pH2DKey != NULL)
@@ -510,49 +509,41 @@ confComputeKeyStoreIsValidGlobalKeyId_GH100
     return (globalKeyIdString != NULL);
 }
 
-/*!
- * Return the key ID for a given LCE channel and rotation operation.
- * If rotateOperation is ROTATE_IV_ALL_VALID then it will return the least
- * key ID of the key pair; ie the one that corresponds to an even numbered slot.
- *
- * @param[in]   pConfCompute    : conf comp pointer
- * @param[in]   pKernelChannel  : KernelChannel pointer
- * @param[in]   rotateOperation : The type of rotation operation
- * @param[out]  pKeyId          : pointer to keyId
- */
-NV_STATUS
-confComputeGetLceKeyIdFromKChannel_GH100
+//
+// Return the key ID for a given LCE channel and rotation operation.
+// If rotateOperation is ROTATE_IV_ALL_VALID then it will return the least
+// key ID of the key pair; ie the one that corresponds to an even numbered slot.
+//
+static NV_STATUS
+getKeyIdLce
 (
-    ConfidentialCompute *pConfCompute,
-    KernelChannel       *pKernelChannel,
-    ROTATE_IV_TYPE       rotateOperation,
-    NvU16               *pKeyId
+    KernelChannel  *pKernelChannel,
+    ROTATE_IV_TYPE  rotateOperation,
+    NvU16          *keyId
 )
 {
     if (kchannelCheckIsUserMode(pKernelChannel))
     {
-        if ((rotateOperation == ROTATE_IV_ENCRYPT) ||
-            (rotateOperation == ROTATE_IV_ALL_VALID))
+        if ((rotateOperation == ROTATE_IV_ENCRYPT) || (rotateOperation == ROTATE_IV_ALL_VALID))
         {
-            *pKeyId = CC_LKEYID_LCE_H2D_USER;
+            *keyId = CC_LKEYID_LCE_H2D_USER;
         }
         else
         {
-            *pKeyId = CC_LKEYID_LCE_D2H_USER;
+            *keyId = CC_LKEYID_LCE_D2H_USER;
         }
 
         return NV_OK;
     }
     else if (kchannelCheckIsKernel(pKernelChannel))
     {
-        if ((rotateOperation == ROTATE_IV_ENCRYPT) ||
-            (rotateOperation == ROTATE_IV_ALL_VALID))
+        if ((rotateOperation == ROTATE_IV_ENCRYPT) || (rotateOperation == ROTATE_IV_ALL_VALID))
         {
-            *pKeyId = CC_LKEYID_LCE_H2D_KERN;
+            *keyId = CC_LKEYID_LCE_H2D_KERN;
         }
         else
         {
-            *pKeyId = CC_LKEYID_LCE_D2H_KERN;
+            *keyId = CC_LKEYID_LCE_D2H_KERN;
         }
 
         return NV_OK;
@@ -609,20 +600,11 @@ getKeyIdSec2
     return NV_ERR_GENERIC;
 }
 
-/*!
- * Returns a key space corresponding to a channel
- *
- * @param[in]   pConfCompute               : ConfidentialCompute pointer
- * @param[in]   pKernelChannel             : KernelChannel pointer
- * @param[out]  keySpace                   : value of keyspace from cc_keystore.h
- *
- */
-NV_STATUS
-confComputeGetKeySpaceFromKChannel_GH100
+static NV_STATUS
+getKeyspaceLce
 (
-    ConfidentialCompute *pConfCompute,
-    KernelChannel       *pKernelChannel,
-    NvU16               *keyspace
+    KernelChannel *pKernelChannel,
+    NvU16         *keyspace
 )
 {
     // The actual copy engine (2 through 9) is normalized to start at 0.
