@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2021-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2021-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -43,6 +43,7 @@
 #include "gpu/mem_mgr/vaspace_api.h"
 #include "vgpu/rpc.h"
 #include "rmapi/rs_utils.h"
+#include "containers/eheap_old.h"
 
 NV_STATUS
 kchangrpapiConstruct_IMPL
@@ -234,9 +235,7 @@ kchangrpapiConstruct_IMPL
 
     // vGpu plugin context flag should only be set on host if context is plugin
     if (gpuIsSriovEnabled(pGpu))
-    {
         pKernelChannelGroup->bIsCallingContextVgpuPlugin = pAllocParams->bIsCallingContextVgpuPlugin;
-    }
 
     if (pKernelChannelGroup->bIsCallingContextVgpuPlugin)
         gfid = GPU_GFID_PF;
@@ -1434,19 +1433,3 @@ kchangrpapiCtrlCmdGetInterleaveLevel_IMPL
     return NV_OK;
 }
 
-/*!
- * @brief Handler for NVA06C_CTRL_CMD_INTERNAL_PROMOTE_FAULT_METHOD_BUFFERS
- *
- * This is currently un-implemented as split change for bug 200691429
- */
-NV_STATUS
-kchangrpapiCtrlCmdInternalPromoteFaultMethodBuffers_IMPL
-(
-    KernelChannelGroupApi *pKernelChannelGroupApi,
-    NVA06C_CTRL_INTERNAL_PROMOTE_FAULT_METHOD_BUFFERS_PARAMS *pParams
-)
-{
-    NV_PRINTF(LEVEL_INFO,
-        "bug 200691429: kchangrpapiCtrlCmdInternalPromoteFaultMethodBuffers_IMPL received\n");
-    return NV_OK;
-}
