@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2006-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2006-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -28,13 +28,18 @@
 #include "gpu/mem_sys/kern_mem_sys.h"
 #include "gpu/mem_mgr/heap.h"
 #include "gpu/mem_mgr/mem_desc.h"
+#include "gpu/mem_mgr/fermi_dma.h"
 #include "gpu/mem_mgr/virt_mem_allocator.h"
 #include "kernel/gpu/gr/kernel_graphics.h"
 #include "gpu/mmu/kern_gmmu.h"
 #include "gpu/bus/kern_bus.h"
+#include "containers/eheap_old.h"
 #include "kernel/gpu/mig_mgr/kernel_mig_manager.h"
+#include "platform/sli/sli.h"
+#include "nvrm_registry.h"
 
 #include "gpu/bif/kernel_bif.h"
+#include "gpu/device/device.h"
 #include "gpu/subdevice/subdevice.h"
 #include "gpu/disp/inst_mem/disp_inst_mem.h"
 
@@ -181,6 +186,11 @@ memmgrAllocDetermineAlignment_GM107
                               NV_ERR_INVALID_ARGUMENT);
             hwAlignment = NV_MAX(hwAlignment, RM_PAGE_SIZE_512M - 1);
             break;
+        case RM_ATTR_PAGE_SIZE_256GB:
+            NV_ASSERT_OR_RETURN(kgmmuIsPageSize256gbSupported(pKernelGmmu),
+                                NV_ERR_INVALID_ARGUMENT);
+            hwAlignment = NV_MAX(hwAlignment, RM_PAGE_SIZE_256G - 1);
+            break;
         case RM_ATTR_PAGE_SIZE_DEFAULT:
         case RM_ATTR_PAGE_SIZE_INVALID:
             NV_PRINTF(LEVEL_ERROR, "- invalid page size specified\n");
@@ -477,7 +487,6 @@ memmgrAllocHal_GM107
     if (FLD_TEST_DRF(OS32, _ATTR2, _ZBC_SKIP_ZBCREFCOUNT, _NO, pFbAllocInfo->pageFormat->attr2))
     {
         if (
-            !IS_MIG_ENABLED(pGpu) &&
             memmgrIsKind_HAL(pMemoryManager, FB_IS_KIND_ZBC, pFbAllocInfo->pageFormat->kind) &&
             !(pFbAllocInfo->pageFormat->flags & NVOS32_ALLOC_FLAGS_VIRTUAL))
         {
@@ -542,13 +551,6 @@ memmgrFreeHal_GM107
         return NV_OK;
     }
 
-    // We might want to move this check to higher-level
-    if (IS_MIG_ENABLED(pGpu))
-    {
-        // In SMC mode, we do not program ZCULL or ZBC
-        return NV_OK;
-    }
-
     kmemsysFreeComprResources_HAL(pGpu, GPU_GET_KERNEL_MEMORY_SYSTEM(pGpu), commitResId);
 
     if (FLD_TEST_DRF(OS32, _ATTR2, _ZBC_SKIP_ZBCREFCOUNT, _NO, pFbAllocInfo->pageFormat->attr2) &&
@@ -628,7 +630,7 @@ memmgrGetBAR1InfoForDevice_GM107
         return status;
     }
 
-    if (!KBUS_CPU_VISIBLE_BAR12_DISABLED(pGpu))
+    if (!kbusIsBar1Disabled(pKernelBus))
     {
         pBar1VAS = kbusGetBar1VASpace_HAL(pGpu, pKernelBus);
         NV_ASSERT_OR_RETURN(pBar1VAS != NULL, NV_ERR_INVALID_STATE);
@@ -658,7 +660,7 @@ memmgrGetBAR1InfoForDevice_GM107
     return NV_OK;
 }
 
-NvU32
+NvU64
 memmgrGetReservedHeapSizeMb_GM107
 (
     OBJGPU        *pGpu,
@@ -682,7 +684,7 @@ memmgrGetReservedHeapSizeMb_GM107
 
     rsvdSize = rsvdSize / (1024 * 1024);   // convert byte to MB
 
-    return (NvU64_LO32(rsvdSize));
+    return rsvdSize;
 }
 
 /*!
@@ -695,7 +697,7 @@ memmgrStateInitReservedMemory
     MemoryManager *pMemoryManager
 )
 {
-    if (IS_GSP_CLIENT(pGpu))
+    if (IS_GSP_CLIENT(pGpu) || IS_VIRTUAL(pGpu))
         return;
 
 }
@@ -712,6 +714,100 @@ memmgrStateInitAdjustReservedMemory
 )
 {
 
+    if (!IS_GSP_CLIENT(pGpu))
+    {
+        NV_STATUS              status;
+        KernelMemorySystem    *pKernelMemorySystem = GPU_GET_KERNEL_MEMORY_SYSTEM(pGpu);
+        KernelBus             *pKernelBus          = GPU_GET_KERNEL_BUS(pGpu);
+        NvU32                  allocFlags          = MEMDESC_FLAGS_NONE;
+        const MEMORY_SYSTEM_STATIC_CONFIG *pMemorySystemConfig =
+            kmemsysGetStaticConfig(pGpu, pKernelMemorySystem);
+
+        // check for allocating VPR memory
+        if (pKernelBus->bInstProtectedMem)
+            allocFlags |= MEMDESC_ALLOC_FLAGS_PROTECTED;
+
+        if ((status = memdescCreate(&pKernelBus->bar1[GPU_GFID_PF].pInstBlkMemDesc, pGpu,
+                                    GF100_BUS_INSTANCEBLOCK_SIZE,
+                                    GF100_BUS_INSTANCEBLOCK_SIZE,
+                                    NV_TRUE,
+                                    pKernelBus->InstBlkAperture,
+                                    pKernelBus->InstBlkAttr,
+                                    allocFlags)) != NV_OK)
+        {
+            return status;
+        }
+
+        if ((memdescGetAddressSpace(pKernelBus->bar1[GPU_GFID_PF].pInstBlkMemDesc) == ADDR_SYSMEM) &&
+            (gpuIsInstanceMemoryAlwaysCached(pGpu)))
+        {
+            memdescSetGpuCacheAttrib(pKernelBus->bar1[GPU_GFID_PF].pInstBlkMemDesc, NV_MEMORY_CACHED);
+        }
+
+        if ((status = memdescCreate(&pKernelBus->bar2[GPU_GFID_PF].pInstBlkMemDesc, pGpu,
+                                    GF100_BUS_INSTANCEBLOCK_SIZE,
+                                    GF100_BUS_INSTANCEBLOCK_SIZE,
+                                    NV_TRUE,
+                                    pKernelBus->InstBlkAperture,
+                                    pKernelBus->InstBlkAttr,
+                                    allocFlags)) != NV_OK)
+        {
+            return status;
+        }
+
+        if ((memdescGetAddressSpace(pKernelBus->bar2[GPU_GFID_PF].pInstBlkMemDesc) == ADDR_SYSMEM) &&
+            (gpuIsInstanceMemoryAlwaysCached(pGpu)))
+        {
+            memdescSetGpuCacheAttrib(pKernelBus->bar2[GPU_GFID_PF].pInstBlkMemDesc, NV_MEMORY_CACHED);
+        }
+
+        switch (pKernelBus->InstBlkAperture)
+        {
+            default:
+            case ADDR_FBMEM:
+                pKernelBus->bar1[GPU_GFID_PF].instBlockBase += pMemoryManager->rsvdMemoryBase;
+                memdescDescribe(pKernelBus->bar1[GPU_GFID_PF].pInstBlkMemDesc,
+                                pKernelBus->InstBlkAperture,
+                                pKernelBus->bar1[GPU_GFID_PF].instBlockBase,
+                                GF100_BUS_INSTANCEBLOCK_SIZE);
+                pKernelBus->bar2[GPU_GFID_PF].instBlockBase += pMemoryManager->rsvdMemoryBase;
+                memdescDescribe(pKernelBus->bar2[GPU_GFID_PF].pInstBlkMemDesc,
+                                pKernelBus->InstBlkAperture,
+                                pKernelBus->bar2[GPU_GFID_PF].instBlockBase,
+                                GF100_BUS_INSTANCEBLOCK_SIZE);
+
+                // Pre-fill cache to prevent FB read accesses if in cache only mode and not doing one time pre-fill
+                if (gpuIsCacheOnlyModeEnabled(pGpu) &&
+                    !pMemorySystemConfig->bL2PreFill)
+                {
+                    kmemsysPreFillCacheOnlyMemory_HAL(pGpu, pKernelMemorySystem, pKernelBus->bar1[GPU_GFID_PF].instBlockBase, GF100_BUS_INSTANCEBLOCK_SIZE);
+                    kmemsysPreFillCacheOnlyMemory_HAL(pGpu, pKernelMemorySystem, pKernelBus->bar2[GPU_GFID_PF].instBlockBase, GF100_BUS_INSTANCEBLOCK_SIZE);
+                }
+                break;
+
+            case ADDR_SYSMEM:
+                memdescTagAlloc(status, NV_FB_ALLOC_RM_INTERNAL_OWNER_UNNAMED_TAG_135, 
+                                (pKernelBus->bar1[GPU_GFID_PF].pInstBlkMemDesc));
+                if (status != NV_OK)
+                {
+                    NV_PRINTF(LEVEL_ERROR,
+                              "couldn't allocate BAR1 instblk in sysmem\n");
+                    return status;
+                }
+                pKernelBus->bar1[GPU_GFID_PF].instBlockBase = memdescGetPhysAddr(pKernelBus->bar1[GPU_GFID_PF].pInstBlkMemDesc, AT_GPU, 0);
+                memdescTagAlloc(status, NV_FB_ALLOC_RM_INTERNAL_OWNER_UNNAMED_TAG_136, 
+                                (pKernelBus->bar2[GPU_GFID_PF].pInstBlkMemDesc));
+                if (status != NV_OK)
+                {
+                    NV_PRINTF(LEVEL_ERROR,
+                              "couldn't allocate BAR2 instblk in sysmem\n");
+                    return status;
+                }
+                pKernelBus->bar2[GPU_GFID_PF].instBlockBase = memdescGetPhysAddr(pKernelBus->bar2[GPU_GFID_PF].pInstBlkMemDesc, AT_GPU, 0);
+                break;
+        }
+    }
+
     return NV_OK;
 }
 
@@ -754,8 +850,6 @@ memmgrInitReservedMemory_GM107
     NvU64                  rsvdTopOfMem     = 0;
     NvU64                  rsvdAlignment    = 0;
     NvBool                 bMemoryProtectionEnabled = NV_FALSE;
-    const MEMORY_SYSTEM_STATIC_CONFIG *pMemorySystemConfig =
-        kmemsysGetStaticConfig(pGpu, GPU_GET_KERNEL_MEMORY_SYSTEM(pGpu));
 
     if (!IS_VIRTUAL(pGpu) && !IS_GSP_CLIENT(pGpu))
     {
@@ -774,7 +868,7 @@ memmgrInitReservedMemory_GM107
     // Align reserved memory to 64K granularity
     pMemoryManager->rsvdMemorySize = NV_ALIGN_UP(pMemoryManager->rsvdMemorySize, 0x10000);
 
-    NV_PRINTF(LEVEL_INFO, "Final reserved memory size = 0x%x\n", pMemoryManager->rsvdMemorySize);
+    NV_PRINTF(LEVEL_INFO, "Final reserved memory size = 0x%llx\n", pMemoryManager->rsvdMemorySize);
 
     if (!IS_VIRTUAL(pGpu))
     {
@@ -786,7 +880,7 @@ memmgrInitReservedMemory_GM107
             memmgrCheckReservedMemorySize_HAL(pGpu, pMemoryManager) == NV_OK, NV_ERR_INSUFFICIENT_RESOURCES);
     }
 
-    NV_PRINTF(LEVEL_INFO, "RESERVED Memory size: 0x%x\n", pMemoryManager->rsvdMemorySize);
+    NV_PRINTF(LEVEL_INFO, "RESERVED Memory size: 0x%llx\n", pMemoryManager->rsvdMemorySize);
 
     // ***************************************************************
     // Done sizing reserved memory
@@ -834,8 +928,16 @@ memmgrInitReservedMemory_GM107
     // In L2 cache only mode, base this off the size of L2 cache
     // If reserved memory at top of FB, base this off the size of FB
     //
-    if (gpuIsCacheOnlyModeEnabled(pGpu) || !pMemorySystemConfig->bReservedMemAtBottom)
+    if (gpuIsCacheOnlyModeEnabled(pGpu) || !pMemoryManager->bReservedMemAtBottom)
     {
+        const MEMORY_SYSTEM_STATIC_CONFIG *pMemorySystemConfig =
+            kmemsysGetStaticConfig(pGpu, GPU_GET_KERNEL_MEMORY_SYSTEM(pGpu));
+
+        if (gpuIsCacheOnlyModeEnabled(pGpu))
+        {
+            rsvdTopOfMem = pMemorySystemConfig->l2CacheSize;
+        }
+        else
         {
             rsvdTopOfMem = pMemoryManager->Ram.fbAddrSpaceSizeMb << 20;
 
@@ -861,7 +963,7 @@ memmgrInitReservedMemory_GM107
         }
         tmpAddr = rsvdTopOfMem - pMemoryManager->rsvdMemorySize;
         pMemoryManager->rsvdMemoryBase = RM_ALIGN_DOWN(tmpAddr, rsvdAlignment);
-        pMemoryManager->rsvdMemorySize = NvU64_LO32(rsvdTopOfMem - pMemoryManager->rsvdMemoryBase);
+        pMemoryManager->rsvdMemorySize = rsvdTopOfMem - pMemoryManager->rsvdMemoryBase;
 
         // make sure we didn't just blindly truncate that...
         NV_ASSERT(0 == NvU64_HI32(rsvdTopOfMem - pMemoryManager->rsvdMemoryBase));
@@ -902,6 +1004,16 @@ memmgrInitReservedMemory_GM107
         rsvdFbRegion.bProtected = bMemoryProtectionEnabled;
         rsvdFbRegion.bInternalHeap = NV_TRUE;
 
+        if (RMCFG_FEATURE_PLATFORM_GSP)
+        {
+            rsvdFbRegion.bPreserveOnSuspend = NV_TRUE;
+        }
+        else
+        {
+            // Reserved region is explicitly saved as before/after Bar2 PTE region.
+            rsvdFbRegion.bLostOnSuspend = NV_TRUE;
+        }
+
         memmgrInsertFbRegion(pGpu, pMemoryManager, &rsvdFbRegion);
     }
 
@@ -1129,6 +1241,13 @@ memmgrSetMemDescPageSize_GM107
                 NV_ASSERT_OR_RETURN(0 == (physAddr & (RM_PAGE_SIZE_512M - 1)), NV_ERR_INVALID_OFFSET);
                 newPageSize = RM_PAGE_SIZE_512M;
                 break;
+            case RM_ATTR_PAGE_SIZE_256GB:
+                NV_ASSERT_OR_RETURN(kgmmuIsPageSize256gbSupported(pKernelGmmu),
+                                  NV_ERR_NOT_SUPPORTED);
+                // If forcing the 256GB page size the underlying memory must be aligned
+                NV_ASSERT_OR_RETURN(0 == (physAddr & (RM_PAGE_SIZE_256G - 1)), NV_ERR_INVALID_OFFSET);
+                newPageSize = RM_PAGE_SIZE_256G;
+                break;
         }
     }
     else if (ADDR_FBMEM == addrSpace)
@@ -1182,6 +1301,13 @@ memmgrSetMemDescPageSize_GM107
                 NV_ASSERT_OR_RETURN(0 == (physAddr & (RM_PAGE_SIZE_512M - 1)), NV_ERR_INVALID_OFFSET);
                 newPageSize = RM_PAGE_SIZE_512M;
                 break;
+            case RM_ATTR_PAGE_SIZE_256GB:
+                NV_ASSERT_OR_RETURN(kgmmuIsPageSize256gbSupported(pKernelGmmu),
+                                  NV_ERR_NOT_SUPPORTED);
+                // If forcing the 256GB page size the underlying memory must be aligned
+                NV_ASSERT_OR_RETURN(0 == (physAddr & (RM_PAGE_SIZE_256G - 1)), NV_ERR_INVALID_OFFSET);
+                newPageSize = RM_PAGE_SIZE_256G;
+                break;
         }
     }
 
@@ -1251,11 +1377,12 @@ memmgrGetRsvdSizeForSr_GM107
     MemoryManager *pMemoryManager
 )
 {
-    if ((pMemoryManager->Ram.fbTotalMemSizeMb >> 10) > 32)
+    if (((pMemoryManager->Ram.fbTotalMemSizeMb >> 10) > 32) || IS_GSP_CLIENT(pGpu))
     {
         //
-        // For SKUs with more than 32GB FB, need to reserve for more memory for S/R
-        // Bug Id:2468357
+        // We need to reserve more memory for S/R if
+        // 1. FB size is > 32GB  Bug Id: 2468357
+        // 2. Or GSP is enabled  Bug Id: 4312881
         //
         return 512 * 1024 * 1024;
     }
@@ -1334,7 +1461,7 @@ memmgrScrubRegistryOverrides_GM107
  *  @returns the physical address space size of FB, which is greater
  *      than or equal to the populated FB memory size
  */
-NvU32
+NvU64
 memmgrGetAddrSpaceSizeMB_GM107
 (
     OBJGPU        *pGpu,
@@ -1343,13 +1470,13 @@ memmgrGetAddrSpaceSizeMB_GM107
 {
     NV_ASSERT(pMemoryManager->Ram.fbAddrSpaceSizeMb != 0);
 
-    return NvU64_LO32(pMemoryManager->Ram.fbAddrSpaceSizeMb);
+    return pMemoryManager->Ram.fbAddrSpaceSizeMb;
 }
 
 //
 // Get fb ram size (usable and mappable).
 //
-NvU32
+NvU64
 memmgrGetUsableMemSizeMB_GM107
 (
     OBJGPU        *pGpu,
@@ -1360,7 +1487,7 @@ memmgrGetUsableMemSizeMB_GM107
 
     // we shouldn't ever need this, but...
     NV_ASSERT(0 == NvU64_HI32(pMemoryManager->Ram.fbUsableMemSize >> 20));
-    return NvU64_LO32(pMemoryManager->Ram.fbUsableMemSize >> 20);
+    return (pMemoryManager->Ram.fbUsableMemSize >> 20);
 }
 
 #define _MAX_COVG (100*NVOS32_ALLOC_COMPR_COVG_SCALE)
@@ -1547,15 +1674,13 @@ memmgrGetBlackListPagesForHeap_GM107
     Heap          *pHeap
 )
 {
+    KernelMemorySystem *pKernelMemorySystem = GPU_GET_KERNEL_MEMORY_SYSTEM(pGpu);
     BLACKLIST_ADDRESS  *pBlAddrs;
     NvU32               idx;
     NV_STATUS           status;
     NvU32               count;
 
-    const MEMORY_SYSTEM_STATIC_CONFIG *pMemorySystemConfig =
-        kmemsysGetStaticConfig(pGpu, GPU_GET_KERNEL_MEMORY_SYSTEM(pGpu));
-
-    count = pMemorySystemConfig->maximumBlacklistPages;
+    count = kmemsysGetMaximumBlacklistPages(pGpu, pKernelMemorySystem);
     pBlAddrs = portMemAllocNonPaged(sizeof(BLACKLIST_ADDRESS) * count);
     if (pBlAddrs == NULL)
     {
@@ -1600,3 +1725,295 @@ memmgrGetFBEndReserveSizeEstimate_GM107
 
     return ESTIMATED_RESERVE_FB;
 }
+
+/*!
+ *  @brief Calculate the reserved memory requirement
+ *         per FB region for mixed type/density
+ *
+ *  @param[out] rsvdFastSize   generic reserved RM memory needed in fast region
+ *  @param[out] rsvdSlowSize   generic reserved RM memory needed in slow region
+ *  @param[out] rsvdISOSize    ISO-specific reserved RM memory needed
+ *
+ *  @returns void
+ *
+ */
+void
+memmgrCalcReservedFbSpaceHal_GM107
+(
+    OBJGPU        *pGpu,
+    MemoryManager *pMemoryManager,
+    NvU64         *rsvdFastSize,
+    NvU64         *rsvdSlowSize,
+    NvU64         *rsvdISOSize
+)
+{
+    KernelGmmu            *pKernelGmmu   = GPU_GET_KERNEL_GMMU(pGpu);
+    KernelFifo            *pKernelFifo   = GPU_GET_KERNEL_FIFO(pGpu);
+    KernelGraphics        *pKernelGraphics = GPU_GET_KERNEL_GRAPHICS(pGpu, 0);
+    NvU64                  smallPagePte  = 0;
+    NvU64                  bigPagePte    = 0;
+    NvU32                  attribBufferSize;
+    NvU64                  maxContextSize = 0;
+    NvU64                  userdReservedSize = 0;
+    NvU64                  runlistEntriesReservedSize = 0;
+    NvU64                  mmuFaultBufferSize = 0;
+    NvU64                  faultMethodBufferSize = 0;
+    NV_STATUS              status = NV_OK;
+
+    // Initialize reserved block logging data structure
+    NV_FB_RSVD_BLOCK_LOG_INIT(pMemoryManager);
+
+    if (IS_VIRTUAL_WITHOUT_SRIOV(pGpu) ||
+        (IS_VIRTUAL(pGpu) && gpuIsWarBug200577889SriovHeavyEnabled(pGpu)))
+    {
+        //
+        // 4MB of reserved memory for vGPU.
+        // Mainly to satisfy KMD memory allocations.
+        //
+        *rsvdFastSize = 4 * 1024 * 1024;
+        *rsvdSlowSize = 0;
+        *rsvdISOSize = 0;
+        return;
+    }
+
+#if defined(NV_UNIX)
+    if (IS_VIRTUAL_WITH_SRIOV(pGpu) && pMemoryManager->Ram.fbTotalMemSizeMb <= 1024)
+    {
+        //
+        // 88MB of reserved memory for vGPU guest with lower FB size(1GB)
+        // in full SRIOV mode. On lower vGPU profiles, available FB memory
+        // is already very less. To compensate for that, reducing the guest
+        // reserved FB memory.
+        //
+        *rsvdFastSize = 88 * 1024 * 1024;
+        *rsvdSlowSize = 0;
+        *rsvdISOSize = 0;
+        return;
+    }
+#endif
+
+    {
+        *rsvdFastSize = 0;
+
+        // Allow reservation up to half of usable FB size
+        if (pMemoryManager->rsvdMemorySizeIncrement > (pMemoryManager->Ram.fbUsableMemSize / 2))
+        {
+            pMemoryManager->rsvdMemorySizeIncrement = pMemoryManager->Ram.fbUsableMemSize / 2;
+            NV_PRINTF(LEVEL_ERROR,
+                      "RM can only increase reserved heap by 0x%llx bytes\n",
+                      pMemoryManager->rsvdMemorySizeIncrement);
+        }
+        NV_PRINTF(LEVEL_INFO, "RT::: incrementing the reserved size by: %llx\n",
+                   pMemoryManager->rsvdMemorySizeIncrement);
+        *rsvdSlowSize = pMemoryManager->rsvdMemorySizeIncrement;
+        *rsvdISOSize = 0;
+    }
+
+    if (RMCFG_FEATURE_PLATFORM_WINDOWS && pMemoryManager->bBug2301372IncreaseRmReserveMemoryWar)
+    {
+        *rsvdFastSize += 30 * 1024 * 1024;
+    }
+
+    attribBufferSize = memmgrGetGrHeapReservationSize_HAL(pGpu, pMemoryManager);
+
+    // Fast: Attribute buffer
+    NV_FB_RSVD_BLOCK_LOG_ENTRY_ADD(status, pMemoryManager, NV_FB_ALLOC_RM_INTERNAL_OWNER_ATTR_BUFFER,
+                         attribBufferSize);
+    *rsvdFastSize += attribBufferSize;
+
+    // Fast: Circular buffer & fudge
+    NV_FB_RSVD_BLOCK_LOG_ENTRY_ADD(status, pMemoryManager, NV_FB_ALLOC_RM_INTERNAL_OWNER_CIRCULAR_BUFFER,
+                        1 *1024 *1024);
+    *rsvdFastSize += 1 *1024 *1024;
+
+    if (!RMCFG_FEATURE_PLATFORM_GSP)
+    {
+        // smallPagePte = FBSize /4k * 8 (Small page PTE for whole FB)
+        smallPagePte = NV_ROUNDUP((pMemoryManager->Ram.fbUsableMemSize / FERMI_SMALL_PAGESIZE) * 8, RM_PAGE_SIZE);
+
+        // bigPagePte = FBSize /bigPageSize * 8 (Big page PTE for whole FB)
+        bigPagePte = NV_ROUNDUP((pMemoryManager->Ram.fbUsableMemSize / (kgmmuGetMaxBigPageSize_HAL(pKernelGmmu))) * 8,
+                                     RM_PAGE_SIZE);
+
+        NV_FB_RSVD_BLOCK_LOG_ENTRY_ADD(status, pMemoryManager, NV_FB_ALLOC_RM_INTERNAL_OWNER_PAGE_PTE,
+                    (smallPagePte + bigPagePte));
+    }
+
+    userdReservedSize = memmgrGetUserdReservedFbSpace_HAL(pGpu, pMemoryManager);
+    NV_FB_RSVD_BLOCK_LOG_ENTRY_ADD(status, pMemoryManager, NV_FB_ALLOC_RM_INTERNAL_OWNER_USERD_BUFFER,
+                         userdReservedSize);
+
+    runlistEntriesReservedSize = memmgrGetRunlistEntriesReservedFbSpace_HAL(pGpu, pMemoryManager);
+    NV_FB_RSVD_BLOCK_LOG_ENTRY_ADD(status, pMemoryManager, NV_FB_ALLOC_RM_INTERNAL_OWNER_RUNLIST_ENTRIES,
+                         runlistEntriesReservedSize);
+
+    maxContextSize = memmgrGetMaxContextSize_HAL(pGpu, pMemoryManager);
+    NV_FB_RSVD_BLOCK_LOG_ENTRY_ADD(status, pMemoryManager, NV_FB_ALLOC_RM_INTERNAL_OWNER_CONTEXT_BUFFER,
+                         maxContextSize);
+    *rsvdSlowSize +=
+             userdReservedSize +  // Kepler USERD
+             runlistEntriesReservedSize +   // Kepler Runlist entries
+             smallPagePte +       // small page Pte
+             bigPagePte +         // big page pte
+             maxContextSize;
+
+    // Reserve FB for UVM on WDDM
+    memmgrCalcReservedFbSpaceForUVM_HAL(pGpu, pMemoryManager, rsvdSlowSize);
+
+    // Reserve FB for MMU fault buffers
+    mmuFaultBufferSize = kgmmuGetFaultBufferReservedFbSpaceSize(pGpu, pKernelGmmu);
+    NV_FB_RSVD_BLOCK_LOG_ENTRY_ADD(status, pMemoryManager, NV_FB_ALLOC_RM_INTERNAL_OWNER_MMU_FAULT_BUFFER,
+                         mmuFaultBufferSize);
+    *rsvdSlowSize += mmuFaultBufferSize;
+
+    // Reserve FB for Fault method buffers
+    if (!RMCFG_FEATURE_PLATFORM_GSP)
+    {
+        faultMethodBufferSize = kfifoCalcTotalSizeOfFaultMethodBuffers_HAL(pGpu, pKernelFifo, NV_TRUE);
+        NV_FB_RSVD_BLOCK_LOG_ENTRY_ADD(status, pMemoryManager, NV_FB_ALLOC_RM_INTERNAL_OWNER_FAULT_METHOD,
+                        faultMethodBufferSize);
+        *rsvdSlowSize += faultMethodBufferSize;
+    }
+
+    // The access map is fairly large (512KB) so we account for it specifically
+    if (kgraphicsDoesUcodeSupportPrivAccessMap(pGpu, pKernelGraphics))
+    {
+        *rsvdSlowSize += pGpu->userRegisterAccessMapSize;
+        NV_FB_RSVD_BLOCK_LOG_ENTRY_ADD(status, pMemoryManager, NV_FB_ALLOC_RM_INTERNAL_OWNER_ACCESS_MAP,
+                        pGpu->userRegisterAccessMapSize);
+    }
+
+    if (*rsvdFastSize + *rsvdSlowSize > pMemoryManager->Ram.fbUsableMemSize / 2)
+    {
+        NV_PRINTF(LEVEL_ERROR,
+                  "Before capping: rsvdFastSize = 0x%llx bytes rsvdSlowSize = 0x%llx "
+                  "bytes Usable FB = 0x%llx bytes\n", *rsvdFastSize,
+                  *rsvdSlowSize, pMemoryManager->Ram.fbUsableMemSize);
+        if (pMemoryManager->rsvdMemorySizeIncrement > 0)
+        {
+            NV_PRINTF(LEVEL_ERROR,
+                      "Fail the rsvd memory capping in case of user specified increase = %llx bytes\n",
+                      pMemoryManager->rsvdMemorySizeIncrement);
+            *rsvdFastSize = 0;
+            *rsvdSlowSize = 0;
+            NV_ASSERT(0);
+            return;
+        }
+        // Scale down fast and slow proportionally
+        *rsvdFastSize = *rsvdFastSize * pMemoryManager->Ram.fbUsableMemSize / 2
+            / (*rsvdFastSize + *rsvdSlowSize);
+        *rsvdSlowSize = pMemoryManager->Ram.fbUsableMemSize / 2 - *rsvdFastSize;
+        NV_PRINTF(LEVEL_ERROR,
+                  "After capping: rsvdFastSize = 0x%llx bytes rsvdSlowSize = 0x%llx bytes\n",
+                  *rsvdFastSize, *rsvdSlowSize);
+    }
+
+    if (!pMemoryManager->bPreferSlowRegion)
+    {
+        *rsvdFastSize = *rsvdFastSize + *rsvdSlowSize;
+        *rsvdSlowSize = 0;
+    }
+
+    //
+    // Memory should be blocked off with 64K granularity.  This makes PMA and
+    // and VA space management more efficient.
+    //
+    *rsvdFastSize = NV_ROUNDUP(*rsvdFastSize, RM_PAGE_SIZE_64K);
+    *rsvdSlowSize = NV_ROUNDUP(*rsvdSlowSize, RM_PAGE_SIZE_64K);
+    *rsvdISOSize  = NV_ROUNDUP(*rsvdISOSize, RM_PAGE_SIZE_64K);
+
+    // If any of the reservation logging fails then print error message
+    if (status != NV_OK)
+    {
+        NV_PRINTF(LEVEL_WARNING, "Error logging the FB reservation entries\n");
+    }
+}
+
+/*!
+ * Set up RM reserved memory space
+ */
+NV_STATUS
+memmgrPreInitReservedMemory_GM107
+(
+    OBJGPU        *pGpu,
+    MemoryManager *pMemoryManager
+)
+{
+    KernelDisplay         *pKernelDisplay      = GPU_GET_KERNEL_DISPLAY(pGpu);
+    KernelBus             *pKernelBus          = GPU_GET_KERNEL_BUS(pGpu);
+    NvU64                  tmpAddr             = 0;
+    NV_STATUS              status              = NV_OK;
+    NvU32                  instBlkBarOverride  = 0;
+
+    // ***************************************************************
+    // Determine the size of reserved memory & tell the HW where it is
+    // Note that the order of these matters for optimum alignment &
+    // FB usage. The order must be highest alignment requirement to
+    // lowest alignment requirement, with the last item being the
+    // vbios image / workspace area
+    // ***************************************************************
+
+    if (IS_GSP_CLIENT(pGpu) && pKernelDisplay != NULL)
+    {
+        // TODO: Determine the correct size of display instance memory
+        // via instmemGetSize_HAL(), as well as other parameters.
+        // I.e. refactor and leverage the code performing these tasks
+        // in memmgrPreInitReservedMemory_GM107() today.
+        tmpAddr += 0x10000;
+    }
+
+    {
+        instBlkBarOverride = DRF_VAL(_REG_STR_RM, _INST_LOC, _INSTBLK, pGpu->instLocOverrides);
+    }
+
+    pKernelBus->InstBlkAperture = ADDR_FBMEM;
+    pKernelBus->InstBlkAttr = NV_MEMORY_WRITECOMBINED;
+
+    memdescOverrideInstLoc(instBlkBarOverride, "BAR instblk",
+                           &pKernelBus->InstBlkAperture,
+                           &pKernelBus->InstBlkAttr);
+
+    if (pKernelBus->InstBlkAperture == ADDR_FBMEM)
+    {
+        // Reserve space for BAR1 and BAR2 instance blocks
+        tmpAddr = NV_ROUNDUP(tmpAddr, GF100_BUS_INSTANCEBLOCK_SIZE);
+        pKernelBus->bar1[GPU_GFID_PF].instBlockBase = tmpAddr;
+        tmpAddr += GF100_BUS_INSTANCEBLOCK_SIZE;
+
+        tmpAddr = NV_ROUNDUP(tmpAddr, GF100_BUS_INSTANCEBLOCK_SIZE);
+        pKernelBus->bar2[GPU_GFID_PF].instBlockBase = tmpAddr;
+        tmpAddr += GF100_BUS_INSTANCEBLOCK_SIZE;
+
+        NV_PRINTF(LEVEL_INFO, "Reserve space for Bar1 inst block offset = 0x%llx size = 0x%x\n",
+            pKernelBus->bar1[GPU_GFID_PF].instBlockBase, GF100_BUS_INSTANCEBLOCK_SIZE);
+
+        NV_PRINTF(LEVEL_INFO, "Reserve space for Bar2 inst block offset = 0x%llx size = 0x%x\n",
+            pKernelBus->bar2[GPU_GFID_PF].instBlockBase, GF100_BUS_INSTANCEBLOCK_SIZE);
+    }
+
+    if (gpuIsSelfHosted(pGpu) && !RMCFG_FEATURE_PLATFORM_GSP)
+    {
+        //
+        // Reserve space for the test buffer used in coherent link test
+        // that is run early when memory allocation is not ready yet.
+        //
+        pKernelBus->coherentLinkTestBufferBase = tmpAddr;
+        tmpAddr += BUS_COHERENT_LINK_TEST_BUFFER_SIZE;
+    }
+
+    //
+    // This has to be the very *last* thing in reserved memory as it
+    // will may grow past the 1MB reserved memory window.  We cannot
+    // size it until memsysStateInitLockedHal_GM107.
+    //
+    memmgrReserveBar2BackingStore(pGpu, pMemoryManager, &tmpAddr);
+
+    //
+    // Store the size of rsvd memory excluding VBIOS space. Size finalized in memmgrStateInitReservedMemory.
+    //
+    pMemoryManager->rsvdMemorySize = tmpAddr;
+
+    NV_PRINTF(LEVEL_INFO, "Calculated size of reserved memory = 0x%llx. Size finalized in StateInit.\n", pMemoryManager->rsvdMemorySize);
+
+    return status;
+}
