@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -52,6 +52,8 @@
 #include "gpu/device/device.h"
 #include "gpu/subdevice/subdevice.h"
 #include "gpu/bus/kern_bus.h"
+#include "platform/sli/sli.h"
+#include "containers/eheap_old.h"
 
 /*!
  * @brief Allocate mapping.
@@ -1135,6 +1137,23 @@ dmaPageArrayInit
     pPageArray->count = pageCount;
 }
 
+/*!
+ * Initialize an abstracted page array with opaque page array data.
+ *
+ *  TODO: Deprecate dmaPageArrayInit once moving all the use cases.
+ */
+void
+dmaPageArrayInitWithFlags
+(
+    DMA_PAGE_ARRAY *pPageArray,    //!< [out] Abstracted page array.
+    void           *pPageData,     //!< [in] Opaque page array data.
+    NvU32           pageCount,     //!< [in] Number of pages represented
+    NvU64           pageArrayFlags //!< [in] Flags of type DMA_PAGE_ARRARY_FLAGS
+)
+{
+    dmaPageArrayInit(pPageArray, pPageData, pageCount);
+}
+
 /*!
  * Initialize an abstracted page array from a memory descriptor.
  */
@@ -1176,18 +1195,16 @@ dmaPageArrayGetPhysAddr
 
     if (pPageArray->bOsFormat)
     {
-        OBJSYS *pSys = SYS_GET_INSTANCE();
-        OBJOS *pOS = SYS_GET_OS(pSys);
-        addr = pOS->osPageArrayGetPhysAddr(pPageArray->pOsGpuInfo,
+        addr = osPageArrayGetPhysAddr(pPageArray->pOsGpuInfo,
             pPageArray->pData, pPageArray->startIndex + pageIndex);
     }
     else
     {
         RmPhysAddr *pPteArray = pPageArray->pData;
-        addr = pPteArray[pPageArray->startIndex + pageIndex];
+        {
+            addr = pPteArray[pPageArray->startIndex + pageIndex];
+        }
     }
 
-    addr |= pPageArray->orMask;
-
     return addr;
 }
