@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2020-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2020-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -66,7 +66,15 @@ memmgrGetMemTransferType
     OBJGPU       *pGpu                = ENG_GET_GPU(pMemoryManager);
     KernelBus    *pKernelBus          = GPU_GET_KERNEL_BUS(pGpu);
 
-    if (flags & TRANSFER_FLAGS_PREFER_CE)
+    if ((pDst == NULL || memdescGetAddressSpace(pDst->pMemDesc) == ADDR_SYSMEM) &&
+        (pSrc == NULL || memdescGetAddressSpace(pSrc->pMemDesc) == ADDR_SYSMEM))
+    {
+        //
+        // If the operation only touches sysmem, use processor copy
+        //
+        transferType = TRANSFER_TYPE_PROCESSOR;
+    }
+    else if (flags & TRANSFER_FLAGS_PREFER_CE)
     {
         if (IS_SIMULATION(pGpu) && pSrc != NULL)
         {
@@ -78,21 +86,25 @@ memmgrGetMemTransferType
             return TRANSFER_TYPE_BAR0;
         }
 
-        if (pMemoryManager->pCeUtils != NULL)
+        CeUtils *pCeUtils = pMemoryManager->pCeUtils;
+
+        if (pCeUtils != NULL)
         {
-            return TRANSFER_TYPE_CE;
+            NvBool bDstCompr = pDst != NULL && memmgrIsKind_HAL(pMemoryManager, FB_IS_KIND_COMPRESSIBLE,
+                                                                memdescGetPteKind(pDst->pMemDesc));
+            NvBool bSrcCompr = pSrc != NULL && memmgrIsKind_HAL(pMemoryManager, FB_IS_KIND_COMPRESSIBLE,
+                                                                memdescGetPteKind(pSrc->pMemDesc));
+            if (!(bSrcCompr || bDstCompr) || pCeUtils->bUseVasForCeCopy)
+            {
+                return TRANSFER_TYPE_CE;
+            }
         }
 
-        NV_PRINTF(LEVEL_WARNING, "Can't copy using CE, falling back to other methods\n");
-    }
-
-    if ((pDst == NULL || memdescGetAddressSpace(pDst->pMemDesc) == ADDR_SYSMEM) &&
-        (pSrc == NULL || memdescGetAddressSpace(pSrc->pMemDesc) == ADDR_SYSMEM))
-    {
-        //
-        // If the operation only touches sysmem, use processor copy
-        //
-        transferType = TRANSFER_TYPE_PROCESSOR;
+        // On Emulation we may lack CE support so preventing excessive debug spew.
+        if (!IS_EMULATION(pGpu))
+        {
+            NV_PRINTF(LEVEL_WARNING, "Can't copy using CE, falling back to other methods\n");
+        }
     }
     else if (kbusIsBarAccessBlocked(pKernelBus) &&
             (!gpuIsCCDevToolsModeEnabled(pGpu) || !(flags & TRANSFER_FLAGS_PREFER_PROCESSOR)))
@@ -140,11 +152,11 @@ _memmgrAllocAndMapSurface
 
     NV_ASSERT_OK_OR_RETURN(
         memdescCreate(ppMemDesc, pGpu, size, RM_PAGE_SIZE, NV_TRUE,
-                      ADDR_SYSMEM, NV_MEMORY_UNCACHED, flags));
+                      ADDR_SYSMEM, NV_MEMORY_CACHED, flags));
 
     memdescTagAlloc(status, NV_FB_ALLOC_RM_INTERNAL_OWNER_UNNAMED_TAG_77,
                     (*ppMemDesc));
-    NV_ASSERT_OK_OR_GOTO(status, status, failed);
+    NV_CHECK_OK_OR_GOTO(status, LEVEL_ERROR, status, failed);
 
     NV_ASSERT_OK_OR_GOTO(status,
         memdescMapOld(*ppMemDesc, 0, size, NV_TRUE, NV_PROTECT_READ_WRITE,
@@ -212,7 +224,7 @@ _memmgrMemReadOrWriteWithGsp
         return NV_ERR_NOT_SUPPORTED;
 
     // Allocate and map the staging buffer
-    NV_ASSERT_OK_OR_RETURN(
+    NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
         _memmgrAllocAndMapSurface(pGpu, size, &pStagingBuf, &pStagingBufMap,
                                   &pStagingBufPriv));
 
@@ -357,7 +369,7 @@ _memmgrMemcpyWithGsp
     }
 
     // Allocate and map the bounce buffer
-    NV_ASSERT_OK_OR_RETURN(
+    NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
         _memmgrAllocAndMapSurface(pGpu, size, &pStagingBuf, &pStagingBufMap,
                                   &pStagingBufPriv));
 
@@ -553,8 +565,6 @@ memmgrMemCopyWithTransferType
 )
 {
     OBJGPU *pGpu = ENG_GET_GPU(pMemoryManager);
-    NvU8 *pSrc;
-    NvU8 *pDst;
 
     // Sanitize the input
     NV_ASSERT_OK_OR_RETURN(memmgrCheckSurfaceBounds(pDstInfo, size));
@@ -565,19 +575,21 @@ memmgrMemCopyWithTransferType
     switch (transferType)
     {
         case TRANSFER_TYPE_PROCESSOR:
-            pDst = memdescMapInternal(pGpu, pDstInfo->pMemDesc, TRANSFER_FLAGS_NONE);
-            NV_ASSERT_OR_RETURN(pDst != NULL, NV_ERR_INSUFFICIENT_RESOURCES);
-            pSrc = memdescMapInternal(pGpu, pSrcInfo->pMemDesc, TRANSFER_FLAGS_NONE);
-            if (pSrc == NULL)
             {
-                memdescUnmapInternal(pGpu, pDstInfo->pMemDesc, 0);
-                NV_ASSERT_OR_RETURN(0, NV_ERR_INSUFFICIENT_RESOURCES);
+                NvU32 mappingFlags = flags | TRANSFER_FLAGS_ALLOW_MAPPING_REUSE;
+                NvU8 *pDst = memmgrMemBeginTransfer(pMemoryManager, pDstInfo, size, mappingFlags);
+                NvU8 *pSrc = memmgrMemBeginTransfer(pMemoryManager, pSrcInfo, size, mappingFlags);
+
+                if (pDst != NULL && pSrc != NULL)
+                {
+                    portMemCopy(pDst, size, pSrc, size);
+                }
+
+                memmgrMemEndTransfer(pMemoryManager, pSrcInfo, size, mappingFlags);
+                memmgrMemEndTransfer(pMemoryManager, pDstInfo, size, mappingFlags);
+
+                NV_CHECK_OR_RETURN(LEVEL_ERROR, pDst != NULL && pSrc != NULL, NV_ERR_INSUFFICIENT_RESOURCES);
             }
-
-            portMemCopy(pDst + pDstInfo->offset, size, pSrc + pSrcInfo->offset, size);
-
-            memdescUnmapInternal(pGpu, pSrcInfo->pMemDesc, TRANSFER_FLAGS_NONE);
-            memdescUnmapInternal(pGpu, pDstInfo->pMemDesc, flags);
             break;
         case TRANSFER_TYPE_GSP_DMA:
             if (IS_GSP_CLIENT(pGpu))
@@ -635,8 +647,7 @@ _memmgrMemReadOrWriteUsingStagingBuffer
     TRANSFER_SURFACE *pDst = bRead ? &staging : pAlloc;
     NV_STATUS status = NV_OK;
 
-
-    NV_ASSERT_OK_OR_RETURN(
+    NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
         _memmgrAllocAndMapSurface(ENG_GET_GPU(pMemoryManager), size, &pStagingBuf, &pStagingBufMap,
                                   &pStagingBufPriv));
     staging.pMemDesc = pStagingBuf;
@@ -681,7 +692,6 @@ memmgrMemSetWithTransferType
 )
 {
     OBJGPU *pGpu = ENG_GET_GPU(pMemoryManager);
-    NvU8 *pDst;
 
     // Sanitize the input
     NV_ASSERT_OK_OR_RETURN(memmgrCheckSurfaceBounds(pDstInfo, size));
@@ -689,12 +699,13 @@ memmgrMemSetWithTransferType
     switch (transferType)
     {
         case TRANSFER_TYPE_PROCESSOR:
-            pDst = memdescMapInternal(pGpu, pDstInfo->pMemDesc, TRANSFER_FLAGS_NONE);
-            NV_ASSERT_OR_RETURN(pDst != NULL, NV_ERR_INSUFFICIENT_RESOURCES);
-
-            portMemSet(pDst + pDstInfo->offset, value, size);
-
-            memdescUnmapInternal(pGpu, pDstInfo->pMemDesc, flags);
+            {
+                NvU32 mappingFlags = flags | TRANSFER_FLAGS_ALLOW_MAPPING_REUSE;
+                NvU8 *pDst = memmgrMemBeginTransfer(pMemoryManager, pDstInfo, size, mappingFlags);
+                NV_CHECK_OR_RETURN(LEVEL_ERROR, pDst != NULL, NV_ERR_INSUFFICIENT_RESOURCES);
+                portMemSet(pDst, value, size);
+                memmgrMemEndTransfer(pMemoryManager, pDstInfo, size, mappingFlags);
+            }
             break;
         case TRANSFER_TYPE_GSP_DMA:
             if (IS_GSP_CLIENT(pGpu))
@@ -734,89 +745,6 @@ memmgrMemSetWithTransferType
     return NV_OK;
 }
 
-/*!
- * @brief This function is used to map the appropriate memory descriptor,
- *        copy the memory from the given buffer, and then unmap.
- *
- * @param[in] pMemDesc Memory descriptor of buffer to write
- * @param[in] pBuf     Buffer allocated by caller
- * @param[in] offset   Offset of buffer to write
- * @param[in] size     Size in bytes of the buffer
- * @param[in] flags    Flags
- */
-static NV_STATUS
-memmgrMemWriteMapAndCopy
-(
-    MemoryManager     *pMemoryManager,
-    MEMORY_DESCRIPTOR *pMemDesc,
-    void              *pBuf,
-    NvU64              offset,
-    NvU64              size,
-    NvU32              flags
-)
-{
-    NvU8   *pDst = NULL;
-    OBJGPU *pGpu = ENG_GET_GPU(pMemoryManager);
-
-    pDst = memdescMapInternal(pGpu, pMemDesc, TRANSFER_FLAGS_NONE);
-    NV_CHECK_OR_RETURN(LEVEL_SILENT, pDst != NULL, NV_ERR_INSUFFICIENT_RESOURCES);
-
-    portMemCopy(pDst + offset, size, pBuf, size);
-    memdescUnmapInternal(pGpu, pMemDesc, flags);
-
-    return NV_OK;
-}
-
-/*!
- * @brief This function is used for writing data placed in a caller passed buffer
- *        to a given memory region while only mapping regions as large as the given
- *        block size.
- *
- * @param[in] pMemDesc   Memory descriptor of buffer to write
- * @param[in] pBuf       Buffer allocated by caller
- * @param[in] baseOffset Offset of entire buffer to write
- * @param[in] size       Size in bytes of the buffer
- * @param[in] flags      Flags
- * @param[in] blockSize  Maximum size of a mapping to use
- */
-static NV_STATUS
-memmgrMemWriteInBlocks
-(
-    MemoryManager     *pMemoryManager,
-    MEMORY_DESCRIPTOR *pMemDesc,
-    void              *pBuf,
-    NvU64              baseOffset,
-    NvU64              size,
-    NvU32              flags,
-    NvU32              blockSize
-)
-{
-    NV_STATUS  status    = NV_OK;
-    OBJGPU    *pGpu      = ENG_GET_GPU(pMemoryManager);
-    NvU64      remaining = size;
-    NvU64      offset    = 0;
-
-    while ((remaining > 0) && (status == NV_OK))
-    {
-        MEMORY_DESCRIPTOR *pSubMemDesc = NULL;
-        NvU32              mapSize     = NV_MIN(blockSize, remaining);
-
-        NV_CHECK_OK_OR_RETURN(LEVEL_SILENT, memdescCreateSubMem(&pSubMemDesc, pMemDesc, pGpu, offset + baseOffset, mapSize));
-
-        // Set the offset to 0, as the sub descriptor already starts at the offset
-        status = memmgrMemWriteMapAndCopy(pMemoryManager, pSubMemDesc, (NvU8 *)pBuf + offset,
-                                          0, mapSize, flags);
-
-        memdescFree(pSubMemDesc);
-        memdescDestroy(pSubMemDesc);
-
-        offset += mapSize;
-        remaining -= mapSize;
-    }
-
-    return status;
-}
-
 /*!
  * @brief This function is used for writing data placed in a caller passed buffer
  *        to a given memory region using the specified memory transfer technique
@@ -838,34 +766,28 @@ memmgrMemWriteWithTransferType
     NvU32             flags
 )
 {
-    NvU8 *pMapping = memdescGetKernelMapping(pDstInfo->pMemDesc);
     OBJGPU *pGpu = ENG_GET_GPU(pMemoryManager);
 
     // Sanitize the input
     NV_ASSERT_OK_OR_RETURN(memmgrCheckSurfaceBounds(pDstInfo, size));
     NV_ASSERT_OR_RETURN(pBuf != NULL, NV_ERR_INVALID_ARGUMENT);
 
-    if (pMapping != NULL)
-    {
-        portMemCopy(pMapping + pDstInfo->offset, size, pBuf, size);
-        return NV_OK;
-    }
-
     switch (transferType)
     {
         case TRANSFER_TYPE_PROCESSOR:
-            if (memmgrMemWriteMapAndCopy(pMemoryManager, pDstInfo->pMemDesc, pBuf, pDstInfo->offset, size, flags) != NV_OK)
             {
-                // If we fail to map a block large enough for the entire transfer, split up the mapping.
-                NV_ASSERT_OK_OR_RETURN(memmgrMemWriteInBlocks(pMemoryManager, pDstInfo->pMemDesc, pBuf,
-                                                              pDstInfo->offset, size, flags, MEMORY_COPY_BLOCK_SIZE));
+                NvU32 mappingFlags = flags | TRANSFER_FLAGS_ALLOW_MAPPING_REUSE;
+                NvU8 *pDst = memmgrMemBeginTransfer(pMemoryManager, pDstInfo, size, mappingFlags);
+                NV_CHECK_OR_RETURN(LEVEL_ERROR, pDst != NULL, NV_ERR_INSUFFICIENT_RESOURCES);
+                portMemCopy(pDst, size, pBuf, size);
+                memmgrMemEndTransfer(pMemoryManager, pDstInfo, size, mappingFlags);
             }
             break;
         case TRANSFER_TYPE_GSP_DMA:
             if (IS_GSP_CLIENT(pGpu))
             {
                 NV_PRINTF(LEVEL_INFO, "Calling GSP DMA task\n");
-                NV_ASSERT_OK_OR_RETURN(
+                NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
                     _memmgrMemReadOrWriteWithGsp(pGpu, pDstInfo, pBuf, size,
                                                  NV_FALSE /* bRead */));
             }
@@ -877,7 +799,7 @@ memmgrMemWriteWithTransferType
         case TRANSFER_TYPE_CE:
         case TRANSFER_TYPE_CE_PRI:
         case TRANSFER_TYPE_BAR0:
-            NV_ASSERT_OK_OR_RETURN(
+            NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
                 _memmgrMemReadOrWriteUsingStagingBuffer(pMemoryManager, pDstInfo, pBuf, size,
                                                         transferType, NV_FALSE /* bRead */));
             break;
@@ -909,34 +831,26 @@ memmgrMemReadWithTransferType
 )
 {
     OBJGPU *pGpu = ENG_GET_GPU(pMemoryManager);
-    NvU8   *pSrc;
-    NvU8   *pMapping = memdescGetKernelMapping(pSrcInfo->pMemDesc);
-
 
     // Sanitize the input
     NV_ASSERT_OK_OR_RETURN(memmgrCheckSurfaceBounds(pSrcInfo, size));
     NV_ASSERT_OR_RETURN(pBuf != NULL, NV_ERR_INVALID_ARGUMENT);
 
-    if (pMapping != NULL)
-    {
-        portMemCopy(pBuf, size, pMapping + pSrcInfo->offset, size);
-        return NV_OK;
-    }
-
     switch (transferType)
     {
         case TRANSFER_TYPE_PROCESSOR:
-            pSrc = memdescMapInternal(pGpu, pSrcInfo->pMemDesc, TRANSFER_FLAGS_NONE);
-            NV_ASSERT_OR_RETURN(pSrc != NULL, NV_ERR_INSUFFICIENT_RESOURCES);
-
-            portMemCopy(pBuf, size, pSrc + pSrcInfo->offset, size);
-
-            memdescUnmapInternal(pGpu, pSrcInfo->pMemDesc, 0);
+            {
+                NvU32 mappingFlags = flags | TRANSFER_FLAGS_ALLOW_MAPPING_REUSE;
+                NvU8 *pSrc = memmgrMemBeginTransfer(pMemoryManager, pSrcInfo, size, mappingFlags);
+                NV_CHECK_OR_RETURN(LEVEL_INFO, pSrc != NULL, NV_ERR_INSUFFICIENT_RESOURCES);
+                portMemCopy(pBuf, size, pSrc, size);
+                memmgrMemEndTransfer(pMemoryManager, pSrcInfo, size, mappingFlags);
+            }
             break;
         case TRANSFER_TYPE_GSP_DMA:
             if (IS_GSP_CLIENT(pGpu))
             {
-                NV_ASSERT_OK_OR_RETURN(
+                NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
                     _memmgrMemReadOrWriteWithGsp(pGpu, pSrcInfo, pBuf, size,
                                                  NV_TRUE /* bRead */));
             }
@@ -948,9 +862,9 @@ memmgrMemReadWithTransferType
         case TRANSFER_TYPE_CE:
         case TRANSFER_TYPE_CE_PRI:
         case TRANSFER_TYPE_BAR0:
-            NV_ASSERT_OK_OR_RETURN(
-            _memmgrMemReadOrWriteUsingStagingBuffer(pMemoryManager, pSrcInfo, pBuf, size,
-                                                    transferType, NV_TRUE /* bRead */));
+            NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
+                _memmgrMemReadOrWriteUsingStagingBuffer(pMemoryManager, pSrcInfo, pBuf, size,
+                                                        transferType, NV_TRUE /* bRead */));
             break;
     }
 
@@ -1080,6 +994,66 @@ memmgrMemUtilsGetMemDescFromHandle_IMPL
     return pMemory->pMemDesc;
 }
 
+/*!
+ * @brief This function is used for write a value placed in a caller passed buffer
+ *        to a given memory region while only mapping regions as large as the given
+ *        block size.
+ *
+ * @param[in] pMemDesc   Memory descriptor of buffer to write
+ * @param[in] value      Value to be written
+ * @param[in] baseOffset Offset of entire buffer to write
+ * @param[in] size       Size in bytes of the buffer
+ * @param[in] flags      Flags
+ * @param[in] blockSize  Maximum size of a mapping to use
+ */
+NV_STATUS
+memmgrMemsetInBlocks_IMPL
+(
+    MemoryManager     *pMemoryManager,
+    MEMORY_DESCRIPTOR *pMemDesc,
+    NvU32              value,
+    NvU64              baseOffset,
+    NvU64              size,
+    NvU32              flags,
+    NvU32              blockSize
+)
+{
+    NV_STATUS  status    = NV_OK;
+    OBJGPU    *pGpu      = ENG_GET_GPU(pMemoryManager);
+    NvU64      remaining = size;
+    NvU64      offset    = 0;
+
+    if (blockSize == 0)
+    {
+        blockSize = MEMORY_COPY_BLOCK_SIZE;
+    }
+
+    while ((remaining > 0) && (status == NV_OK))
+    {
+        MEMORY_DESCRIPTOR *pSubMemDesc = NULL;
+        NvU32              mapSize     = NV_MIN(blockSize, remaining);
+        TRANSFER_SURFACE   surf = {0};
+
+        NV_CHECK_OK_OR_RETURN(LEVEL_SILENT, memdescCreateSubMem(&pSubMemDesc, pMemDesc, pGpu, offset + baseOffset, mapSize));
+
+        surf.pMemDesc = pSubMemDesc;
+        surf.offset = 0;
+
+        NV_ASSERT_OK_OR_RETURN(
+            memmgrMemSet(pMemoryManager, &surf, value,
+                         pSubMemDesc->Size,
+                         flags));
+
+        memdescFree(pSubMemDesc);
+        memdescDestroy(pSubMemDesc);
+
+        offset += mapSize;
+        remaining -= mapSize;
+    }
+
+    return status;
+}
+
 /*!
  * @brief This function is used for copying data b/w two memory regions
  *        Both memory regions can be in the same aperture of different apertures
@@ -1156,6 +1130,70 @@ memmgrMemDescMemSet_IMPL
                                         transferType, flags);
 }
 
+static NV_STATUS
+memmgrMemReadOrWriteInBlocks
+(
+    MemoryManager    *pMemoryManager,
+    TRANSFER_SURFACE *pSurf,
+    void             *pBuf,
+    NvU64             size,
+    TRANSFER_TYPE     transferType,
+    NvU32             flags,
+    NvBool            bRead
+)
+{
+    NvBool             bCreateSubMemDesc = (transferType == TRANSFER_TYPE_PROCESSOR) &&
+                                           !(flags & TRANSFER_FLAGS_USE_BAR1);
+    NvU64              remainingSize     = size;
+    NvU64              baseOffset        = pSurf->offset;
+    MEMORY_DESCRIPTOR *pMemDesc          = pSurf->pMemDesc;
+    NvU64              offset            = 0;
+    NV_STATUS          status            = NV_OK;
+
+    while ((remainingSize > 0) && (status == NV_OK))
+    {
+        MEMORY_DESCRIPTOR *pSubMemDesc = NULL;
+        NvU64              copySize    = NV_MIN(MEMORY_COPY_BLOCK_SIZE, remainingSize);
+        TRANSFER_SURFACE   tmpSurf     = { 0 };
+
+        if (bCreateSubMemDesc)
+        {
+            NV_ASSERT_OK_OR_RETURN(
+                memdescCreateSubMem(&pSubMemDesc, pMemDesc, pMemDesc->pGpu, offset + baseOffset, copySize));
+            tmpSurf.pMemDesc = pSubMemDesc;
+            tmpSurf.offset = 0;
+        }
+        else
+        {
+            tmpSurf.pMemDesc = pMemDesc;
+            tmpSurf.offset = offset + baseOffset;
+        }
+
+        if (bRead)
+        {
+            status = memmgrMemReadWithTransferType(pMemoryManager, &tmpSurf, (NvU8 *)pBuf + offset,
+                                                   copySize, transferType, flags);
+        }
+        else
+        {
+            status = memmgrMemWriteWithTransferType(pMemoryManager, &tmpSurf, (NvU8 *)pBuf + offset,
+                                                    copySize, transferType, flags);
+        }
+        NV_ASSERT_OK(status);
+
+        if (bCreateSubMemDesc)
+        {
+            memdescFree(pSubMemDesc);
+            memdescDestroy(pSubMemDesc);
+        }
+
+        offset += copySize;
+        remainingSize -= copySize;
+    }
+
+    return status;
+}
+
 /*!
  * @brief This function is used for writing data placed in a user buffer
  *        to a given memory region
@@ -1178,8 +1216,13 @@ memmgrMemWrite_IMPL
     TRANSFER_TYPE transferType = memmgrGetMemTransferType(pMemoryManager,
                                                           pDstInfo, NULL, flags);
 
-    return memmgrMemWriteWithTransferType(pMemoryManager, pDstInfo, pBuf,
-                                          size, transferType, flags);
+    NV_STATUS status = memmgrMemWriteWithTransferType(pMemoryManager, pDstInfo, pBuf,
+                                                      size, transferType, flags);
+    if (status == NV_OK)
+        return NV_OK;
+
+    return memmgrMemReadOrWriteInBlocks(pMemoryManager, pDstInfo, pBuf,
+                                        size, transferType, flags, NV_FALSE);
 }
 
 /*!
@@ -1204,8 +1247,13 @@ memmgrMemRead_IMPL
     TRANSFER_TYPE transferType = memmgrGetMemTransferType(pMemoryManager,
                                                           NULL, pSrcInfo, flags);
 
-    return memmgrMemReadWithTransferType(pMemoryManager, pSrcInfo, pBuf,
-                                         size, transferType, flags);
+    NV_STATUS status = memmgrMemReadWithTransferType(pMemoryManager, pSrcInfo, pBuf,
+                                                     size, transferType, flags);
+    if (status == NV_OK)
+        return NV_OK;
+
+    return memmgrMemReadOrWriteInBlocks(pMemoryManager, pSrcInfo, pBuf,
+                                        size, transferType, flags, NV_TRUE);
 }
 
 /*!
@@ -1229,18 +1277,26 @@ memmgrMemBeginTransfer_IMPL
     MEMORY_DESCRIPTOR *pMemDesc     = pTransferInfo->pMemDesc;
     NvU64              offset       = pTransferInfo->offset;
     OBJGPU            *pGpu         = ENG_GET_GPU(pMemoryManager);
-    NvU8              *pPtr         = NULL;
+    void              *pPtr         = NULL;
+    void              *pPriv        = NULL;
     NvU64              memSz        = shadowBufSize;
+    NvU8              *pMapping     = memdescGetKernelMapping(pTransferInfo->pMemDesc);
 
     NV_ASSERT_OR_RETURN(memmgrCheckSurfaceBounds(pTransferInfo, memSz) == NV_OK, NULL);
-    NV_ASSERT_OR_RETURN(memdescGetKernelMapping(pMemDesc) == NULL, NULL);
+    NV_ASSERT_OR_RETURN(pTransferInfo->pMapping == NULL, NULL);
+    NV_ASSERT_OR_RETURN(pTransferInfo->pMappingPriv == NULL, NULL);
+
+    if ((flags & TRANSFER_FLAGS_ALLOW_MAPPING_REUSE) && pMapping != NULL)
+    {
+        // keep TRANSFER_SURFACE's pMapping NULL, as mapping has a different owner
+        return pMapping + pTransferInfo->offset;
+    }
 
     switch (transferType)
     {
         case TRANSFER_TYPE_PROCESSOR:
             if (flags & TRANSFER_FLAGS_USE_BAR1)
             {
-                NvP64 pPriv;
                 NvU32 protect = NV_PROTECT_READ_WRITE;
 
                 if (flags & TRANSFER_FLAGS_MAP_PROTECT_READABLE)
@@ -1252,14 +1308,15 @@ memmgrMemBeginTransfer_IMPL
                     protect = NV_PROTECT_WRITEABLE;
                 }
 
-                NV_ASSERT_OR_RETURN(memdescMap(pMemDesc, offset, memSz, NV_TRUE, protect,
-                    (NvP64*) &pPtr, &pPriv) == NV_OK, NULL);
-                memdescSetKernelMappingPriv(pMemDesc, pPriv);
-                break;
+                NV_CHECK_OR_RETURN(LEVEL_ERROR,
+                    memdescMapOld(pMemDesc, offset, memSz, NV_TRUE, protect, &pPtr, &pPriv) == NV_OK, NULL);
             }
-            NV_ASSERT_OR_RETURN((pPtr = memdescMapInternal(pGpu, pMemDesc, flags)) != NULL, NULL);
-            pPtr = &pPtr[offset];
+            else
+            {
+                NV_CHECK_OR_RETURN(LEVEL_INFO, (pPtr = memdescMapInternal(pGpu, pMemDesc, flags)) != NULL, NULL);
 
+                pPtr = (NvU8 *)pPtr + offset;
+            }
             break;
         case TRANSFER_TYPE_GSP_DMA:
         case TRANSFER_TYPE_CE:
@@ -1277,7 +1334,9 @@ memmgrMemBeginTransfer_IMPL
         default:
             NV_ASSERT(0);
     }
-    memdescSetKernelMapping(pMemDesc, pPtr);
+
+    pTransferInfo->pMapping = pPtr;
+    pTransferInfo->pMappingPriv = pPriv;
     return pPtr;
 }
 
@@ -1301,44 +1360,41 @@ memmgrMemEndTransfer_IMPL
                                                                pTransferInfo, NULL, flags);
     MEMORY_DESCRIPTOR *pMemDesc     = pTransferInfo->pMemDesc;
     OBJGPU            *pGpu         = ENG_GET_GPU(pMemoryManager);
-    NvU8              *pMapping     = NULL;
     NvU64              memSz        = shadowBufSize;
 
     NV_ASSERT_OR_RETURN_VOID(memmgrCheckSurfaceBounds(pTransferInfo, memSz) == NV_OK);
 
-    pMapping = memdescGetKernelMapping(pMemDesc);
-
-    memdescSetKernelMapping(pMemDesc, NULL);
+    if (pTransferInfo->pMapping == NULL)
+    {
+        // Normal for full memdesc mapping reuse
+        return;
+    }
 
     switch (transferType)
     {
         case TRANSFER_TYPE_PROCESSOR:
             if (flags & TRANSFER_FLAGS_USE_BAR1)
             {
-                NvP64 pPriv = memdescGetKernelMappingPriv(pMemDesc);
-                memdescSetKernelMappingPriv(pMemDesc, NULL);
-                if (pMapping != NULL)
-                {
-                    memdescUnmap(pMemDesc, NV_TRUE, 0, pMapping, pPriv);
-                }
-                return;
+                memdescUnmap(pMemDesc, NV_TRUE, 0, pTransferInfo->pMapping, pTransferInfo->pMappingPriv);
             }
-            memdescUnmapInternal(pGpu, pMemDesc, flags);
-            return;
+            else
+            {
+                memdescUnmapInternal(pGpu, pMemDesc, flags);
+            }
+            break;
         case TRANSFER_TYPE_GSP_DMA:
         case TRANSFER_TYPE_CE:
         case TRANSFER_TYPE_CE_PRI:
         case TRANSFER_TYPE_BAR0:
-            if (pMapping != NULL)
-            {
-                NV_ASSERT_OK(memmgrMemWrite(pMemoryManager, pTransferInfo, pMapping, memSz, flags));
-                portMemFree(pMapping);
-            }
-            return;
+            NV_ASSERT_OK(memmgrMemWrite(pMemoryManager, pTransferInfo, pTransferInfo->pMapping, memSz, flags));
+            portMemFree(pTransferInfo->pMapping);
+            break;
         default:
             NV_ASSERT(0);
     }
-    return;
+
+    pTransferInfo->pMapping = NULL;
+    pTransferInfo->pMappingPriv = NULL;
 }
 
 /*!
@@ -1360,7 +1416,15 @@ memmgrMemDescEndTransfer_IMPL
         return;
     }
 
-    TRANSFER_SURFACE transferSurface = {.offset = 0, .pMemDesc = pMemDesc};
+    TRANSFER_SURFACE transferSurface = { 0 };
+    transferSurface.offset          = 0;
+    transferSurface.pMemDesc        = pMemDesc;
+    transferSurface.pMapping        = memdescGetKernelMapping(pMemDesc);
+    transferSurface.pMappingPriv    = memdescGetKernelMappingPriv(pMemDesc);
+
+    memdescSetKernelMapping(pMemDesc, NULL);
+    memdescSetKernelMappingPriv(pMemDesc, NULL);
+
     memmgrMemEndTransfer(pMemoryManager, &transferSurface, memdescGetSize(pMemDesc), flags);
 }
 
@@ -1379,8 +1443,21 @@ memmgrMemDescBeginTransfer_IMPL
 )
 {
     NV_ASSERT_OR_RETURN(pMemDesc != NULL, NULL);
+    NV_ASSERT_OR_RETURN(memdescGetKernelMapping(pMemDesc) == NULL, NULL);
+    NV_ASSERT_OR_RETURN(memdescGetKernelMappingPriv(pMemDesc) == NULL, NULL);
+    NV_ASSERT_OR_RETURN(!(flags & TRANSFER_FLAGS_ALLOW_MAPPING_REUSE), NULL);
+
     TRANSFER_SURFACE transferSurface = {.offset = 0, .pMemDesc = pMemDesc};
-    return memmgrMemBeginTransfer(pMemoryManager, &transferSurface, memdescGetSize(pMemDesc), flags);
+    NvU8 *pMapping = memmgrMemBeginTransfer(pMemoryManager, &transferSurface, memdescGetSize(pMemDesc), flags);
+
+    NV_ASSERT_OR_RETURN(pMapping != NULL, NULL);
+    NV_ASSERT_OR_RETURN(transferSurface.pMapping == pMapping, NULL);
+
+    // Set mapping and priv to reuse during later operations, or at unmap time
+    memdescSetKernelMapping(pMemDesc, pMapping);
+    memdescSetKernelMappingPriv(pMemDesc, transferSurface.pMappingPriv);
+
+    return pMapping;
 }
 
 /*!
