@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2023-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -24,11 +24,14 @@
 #include "gpu/mem_mgr/sem_surf.h"
 #include "os/os.h" // NV_MEMORY_NONCONTIGUOUS, osEventNotification
 #include "gpu/device/device.h"
+#include "gpu_mgr/gpu_mgr.h"
 #include "gpu/mem_mgr/mem_mgr.h"
 #include "gpu/mem_mgr/mem_desc.h"
+#include "gpu/fifo/kernel_channel.h"
 #include "gpu/gpu.h"
 #include "rmapi/client.h"
 #include "rmapi/rs_utils.h"
+#include "platform/sli/sli.h"
 
 #include "class/cl0080.h"
 #include "class/cl2080.h"
@@ -43,6 +46,7 @@ _semsurfFreeRmClient
 )
 {
     RM_API *pRmApi = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
+    NvU32 i;
 
     if (pShared->hClient != NV01_NULL_OBJECT)
     {
@@ -52,27 +56,27 @@ _semsurfFreeRmClient
     }
 
     pShared->hClient    = NV01_NULL_OBJECT;
-    pShared->hDevice    = NV01_NULL_OBJECT;
-    pShared->hSubDevice = NV01_NULL_OBJECT;
+    for (i = 0; i < NV_MAX_DEVICES; i++) {
+        pShared->hDevice[i]     = NV01_NULL_OBJECT;
+        pShared->hSubDevice[i]  = NV01_NULL_OBJECT;
+    }
 }
 
 static void
 _semsurfUnregisterCallback
 (
-    SEM_SHARED_DATA *pShared
+    SEM_SHARED_DATA *pShared,
+    NvHandle hEvent
 )
 {
     RM_API *pRmApi = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
 
-    if ((pShared->hClient != NV01_NULL_OBJECT) &&
-        (pShared->hEvent != NV01_NULL_OBJECT))
+    if (pShared->hClient != NV01_NULL_OBJECT)
     {
         pRmApi->Free(pRmApi,
                      pShared->hClient,
-                     pShared->hEvent);
+                     hEvent);
     }
-
-    pShared->hEvent     = NV01_NULL_OBJECT;
 }
 
 static NvU64
@@ -323,7 +327,7 @@ _semsurfEventCallback
 
     NV_PRINTF(LEVEL_INFO, "SemMem(0x%08x, 0x%08x): Got a callback\n", pShared->hClient, pShared->hSemaphoreMem);
     NV_PRINTF(LEVEL_INFO, "  hEvent: 0x%08x surf event: 0x%08x, data 0x%08x, status 0x%08x\n",
-              hEvent, pShared->hEvent, data, status);
+              hEvent, hEvent, data, status);
 
     while (valuesChanged)
     {
@@ -399,6 +403,50 @@ _semsurfEventCallback
     }
 }
 
+static NV_STATUS
+_semsurfAllocDevObjs(SEM_SHARED_DATA *pShared, OBJGPU *pGpu, RM_API *pRmApi)
+{
+    NV0080_ALLOC_PARAMETERS nv0080AllocParams;
+    NV2080_ALLOC_PARAMETERS nv2080AllocParams;
+    NvU32 gpuIdx = gpuGetInstance(pGpu);
+
+    if (pShared->hDevice[gpuIdx] != NV01_NULL_OBJECT)
+    {
+        NV_ASSERT_OR_RETURN(pShared->hSubDevice[gpuIdx] != NV01_NULL_OBJECT,
+                            NV_ERR_INVALID_STATE);
+        return NV_OK;
+    }
+
+    portMemSet(&nv0080AllocParams, 0, sizeof(nv0080AllocParams));
+    nv0080AllocParams.deviceId =
+        gpuGetDeviceInstance(pGpu);
+
+    NV_ASSERT_OK_OR_RETURN(
+        pRmApi->Alloc(pRmApi,
+                      pShared->hClient,
+                      pShared->hClient,
+                      &pShared->hDevice[gpuIdx],
+                      NV01_DEVICE_0,
+                      &nv0080AllocParams,
+                      sizeof(nv0080AllocParams)));
+
+    // Allocate a subDevice
+    portMemSet(&nv2080AllocParams, 0, sizeof(nv2080AllocParams));
+    nv2080AllocParams.subDeviceId =
+        gpumgrGetSubDeviceInstanceFromGpu(pGpu);
+
+    NV_ASSERT_OK_OR_RETURN(
+        pRmApi->Alloc(pRmApi,
+                      pShared->hClient,
+                      pShared->hDevice[gpuIdx],
+                      &pShared->hSubDevice[gpuIdx],
+                      NV20_SUBDEVICE_0,
+                      &nv2080AllocParams,
+                      sizeof(nv2080AllocParams)));
+
+    return NV_OK;
+}
+
 static NV_STATUS
 _semsurfAllocRmClient
 (
@@ -406,14 +454,14 @@ _semsurfAllocRmClient
 )
 {
     SEM_SHARED_DATA *pShared = pSemSurf->pShared;
-    NV0080_ALLOC_PARAMETERS nv0080AllocParams;
-    NV2080_ALLOC_PARAMETERS nv2080AllocParams;
     RM_API *pRmApi = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
 
+    //
     // Allocate an internal client, device, and subDevice for the semaphore
     // surface. These will be used to allocate the internally-managed memory
     // object wrapped by the semaphore surface, and to to register callbacks
     // on the GPU for semaphore awaken/notification interrupts.
+    //
     NV_ASSERT_OK_OR_RETURN(
         pRmApi->AllocWithHandle(pRmApi,
                                 NV01_NULL_OBJECT,
@@ -423,34 +471,7 @@ _semsurfAllocRmClient
                                 &pShared->hClient,
                                 sizeof(pShared->hClient)));
 
-    portMemSet(&nv0080AllocParams, 0, sizeof(nv0080AllocParams));
-    nv0080AllocParams.deviceId =
-        gpuGetDeviceInstance(GPU_RES_GET_GPU(pSemSurf));
-
-    NV_ASSERT_OK_OR_RETURN(
-        pRmApi->Alloc(pRmApi,
-                      pShared->hClient,
-                      pShared->hClient,
-                      &pShared->hDevice,
-                      NV01_DEVICE_0,
-                      &nv0080AllocParams,
-                      sizeof(nv0080AllocParams)));
-
-    // Allocate a subDevice
-    portMemSet(&nv2080AllocParams, 0, sizeof(nv2080AllocParams));
-    nv2080AllocParams.subDeviceId =
-        gpumgrGetSubDeviceInstanceFromGpu(GPU_RES_GET_GPU(pSemSurf));
-
-    NV_ASSERT_OK_OR_RETURN(
-        pRmApi->Alloc(pRmApi,
-                      pShared->hClient,
-                      pShared->hDevice,
-                      &pShared->hSubDevice,
-                      NV20_SUBDEVICE_0,
-                      &nv2080AllocParams,
-                      sizeof(nv2080AllocParams)));
-
-    return NV_OK;
+    return _semsurfAllocDevObjs(pShared, GPU_RES_GET_GPU(pSemSurf), pRmApi);
 }
 
 static void
@@ -488,11 +509,12 @@ _semsurfDupMemory
 {
     SEM_SHARED_DATA *pShared = pSemSurf->pShared;
     RM_API *pRmApi = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
+    NvU32 gpuIdx = gpuGetInstance(GPU_RES_GET_GPU(pSemSurf));
 
     NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
         pRmApi->DupObject(pRmApi,
                           pShared->hClient,
-                          pShared->hDevice,
+                          pShared->hDevice[gpuIdx],
                           &pShared->hSemaphoreMem,
                           RES_GET_CLIENT_HANDLE(pSemSurf),
                           pAllocParams->hSemaphoreMem,
@@ -509,7 +531,7 @@ _semsurfDupMemory
             NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
                 pRmApi->DupObject(pRmApi,
                                   pShared->hClient,
-                                  pShared->hDevice,
+                                  pShared->hDevice[gpuIdx],
                                   &pShared->hMaxSubmittedMem,
                                   RES_GET_CLIENT_HANDLE(pSemSurf),
                                   pAllocParams->hMaxSubmittedMem,
@@ -532,34 +554,109 @@ _semsurfDupMemory
 static NV_STATUS
 _semsurfRegisterCallback
 (
-    SemaphoreSurface *pSemSurf
+    SemaphoreSurface *pSemSurf,
+    OBJGPU *pGpu,
+    NvU32 notifyIndex,
+    NvU32 *phEvent
 )
 {
     SEM_SHARED_DATA *pShared = pSemSurf->pShared;
     NV0005_ALLOC_PARAMETERS nv0005AllocParams;
     RM_API *pRmApi = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
-
-    pShared->callback.func = _semsurfEventCallback;
-    pShared->callback.arg = pShared;
+    NvHandle hEvent = 0;
 
     portMemSet(&nv0005AllocParams, 0, sizeof(nv0005AllocParams));
     nv0005AllocParams.hParentClient = pShared->hClient;
     nv0005AllocParams.hClass        = NV01_EVENT_KERNEL_CALLBACK_EX;
-    nv0005AllocParams.notifyIndex   = NV2080_NOTIFIERS_FIFO_EVENT_MTHD |
+    nv0005AllocParams.data          = NV_PTR_TO_NvP64(&pShared->callback);
+    nv0005AllocParams.notifyIndex = notifyIndex |
         NV01_EVENT_NONSTALL_INTR |
         NV01_EVENT_WITHOUT_EVENT_DATA |
         NV01_EVENT_SUBDEVICE_SPECIFIC |
         DRF_NUM(0005, _NOTIFY_INDEX, _SUBDEVICE,
-                gpumgrGetSubDeviceInstanceFromGpu(GPU_RES_GET_GPU(pSemSurf)));
-    nv0005AllocParams.data          = NV_PTR_TO_NvP64(&pShared->callback);
+                gpumgrGetSubDeviceInstanceFromGpu(pGpu));
 
-    return pRmApi->Alloc(pRmApi,
-                         pShared->hClient,
-                         pShared->hSubDevice,
-                         &pShared->hEvent,
-                         NV01_EVENT_KERNEL_CALLBACK_EX,
-                         &nv0005AllocParams,
-                         sizeof(nv0005AllocParams));
+    NV_ASSERT_OK_OR_RETURN(_semsurfAllocDevObjs(pShared, pGpu, pRmApi));
+
+    NV_ASSERT_OK_OR_RETURN(pRmApi->Alloc(pRmApi,
+                                         pShared->hClient,
+                                         pShared->hSubDevice[gpuGetInstance(pGpu)],
+                                         &hEvent,
+                                         NV01_EVENT_KERNEL_CALLBACK_EX,
+                                         &nv0005AllocParams,
+                                         sizeof(nv0005AllocParams)));
+
+    *phEvent = hEvent;
+
+    return NV_OK;
+}
+
+NV_STATUS
+_semsurfRemoveNotifyBinding
+(
+    SemaphoreSurface *pSemSurf,
+    NvU32 gpuIdx,
+    NvU32 notifyIndex
+)
+{
+    const NvU64 mapKey = ((NvU64)gpuIdx << 32) | notifyIndex;
+    SEM_NOTIFIER_NODE *pNotNode = mapFind(&pSemSurf->pShared->notifierMap, mapKey);
+
+    NV_ASSERT_OR_RETURN(pNotNode != NULL, NV_ERR_INVALID_STATE);
+    NV_ASSERT_OR_RETURN(pNotNode->nUsers >= 1, NV_ERR_INVALID_STATE);
+    pNotNode->nUsers--;
+
+    if (pNotNode->nUsers != 0)
+    {
+        return NV_OK;
+    }
+
+    _semsurfUnregisterCallback(pSemSurf->pShared, pNotNode->hEvent);
+    mapRemove(&pSemSurf->pShared->notifierMap, pNotNode);
+    portMemFree(pNotNode);
+
+    return NV_OK;
+}
+
+static NV_STATUS
+_semsurfUnbindChannel
+(
+    SemaphoreSurface *pSemSurf,
+    SEM_CHANNEL_NODE *pChannelNode
+)
+{
+    NV_STATUS rmStatus = NV_OK;
+    NvU32 i;
+
+    for (i = 0; i < pChannelNode->numNotifyIndices; i++)
+    {
+        //
+        // Soldier on if removal of one notification index fails, but preserve
+        // the first failure status code.
+        //
+        NV_STATUS tmpStatus =
+            _semsurfRemoveNotifyBinding(pSemSurf,
+                                        pChannelNode->gpuIdx,
+                                        pChannelNode->notifyIndices[i]);
+
+        if (tmpStatus == NV_OK)
+        {
+            NV_PRINTF(LEVEL_INFO,
+                      "SemSurf(0x%08x, 0x%08x): "
+                      "Unbound event for GPU instance 0x%08x notify index 0x%08x\n",
+                      RES_GET_CLIENT_HANDLE(pSemSurf), RES_GET_HANDLE(pSemSurf),
+                      pChannelNode->gpuIdx, pChannelNode->notifyIndices[i]);
+        }
+        else if (rmStatus == NV_OK)
+        {
+            rmStatus = tmpStatus;
+        }
+    }
+
+    mapRemove(&pSemSurf->boundChannelMap, pChannelNode);
+    portMemFree(pChannelNode);
+
+    return rmStatus;
 }
 
 static NV_STATUS
@@ -593,11 +690,25 @@ _semsurfDestroyShared
 )
 {
     RM_API *pRmApi = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
+    SEM_NOTIFIERIter notIter;
 
     if (pShared == NULL)
         return;
 
-    _semsurfUnregisterCallback(pShared);
+    //
+    // Any notifier entries should have been cleaned up as part of the
+    // destructors of the semaphore surface objects sharing this data.
+    //
+    NV_ASSERT(mapCount(&pShared->notifierMap) == 0);
+
+    for (notIter = mapIterAll(&pShared->notifierMap);
+         mapIterNext(&notIter);
+         notIter = mapIterAll(&pShared->notifierMap))
+    {
+        _semsurfUnregisterCallback(pShared, notIter.pValue->hEvent);
+        mapRemove(&pShared->notifierMap, notIter.pValue);
+        portMemFree(notIter.pValue);
+    }
 
     if (pShared->pMaxSubmittedMem)
     {
@@ -605,7 +716,7 @@ _semsurfDestroyShared
         {
             pRmApi->UnmapFromCpu(pRmApi,
                                  pShared->hClient,
-                                 pShared->hDevice,
+                                 pShared->hDevice[pShared->memGpuIdx],
                                  pShared->hMaxSubmittedMem,
                                  pShared->maxSubmittedKernAddr,
                                  0, 0);
@@ -621,7 +732,7 @@ _semsurfDestroyShared
         {
             pRmApi->UnmapFromCpu(pRmApi,
                                  pShared->hClient,
-                                 pShared->hDevice,
+                                 pShared->hDevice[pShared->memGpuIdx],
                                  pShared->hSemaphoreMem,
                                  pShared->semKernAddr,
                                  0, 0);
@@ -650,9 +761,7 @@ _semsurfValidateIndex
     NvU64 index
 )
 {
-    const NvU64 slotSize = pShared->layout.size;
-
-    if (((index * slotSize) + slotSize) <= pShared->pSemaphoreMem->pMemDesc->Size)
+    if (index < pShared->slotCount)
         return NV_TRUE;
     else
         return NV_FALSE;
@@ -692,7 +801,9 @@ semsurfConstruct_IMPL
 
     portMemSet(pShared, 0, sizeof(*pShared));
     pShared->refCount = 1;
+    pShared->memGpuIdx = gpuGetInstance(GPU_RES_GET_GPU(pSemSurf));
     mapInitIntrusive(&pShared->listenerMap);
+    mapInitIntrusive(&pShared->notifierMap);
 
     pShared->pSpinlock = portSyncSpinlockCreate(portMemAllocatorGetGlobalNonPaged());
     NV_ASSERT_TRUE_OR_GOTO(status, pShared->pSpinlock != NULL, NV_ERR_NO_MEMORY, ctorFailed);
@@ -702,7 +813,7 @@ semsurfConstruct_IMPL
     NV_ASSERT_OK_OR_GOTO(status,
         pRmApi->Control(pRmApi,
                         pShared->hClient,
-                        pShared->hSubDevice,
+                        pShared->hSubDevice[pShared->memGpuIdx],
                         NV2080_CTRL_CMD_FB_GET_SEMAPHORE_SURFACE_LAYOUT,
                        &pShared->layout,
                         sizeof pShared->layout),
@@ -737,7 +848,7 @@ semsurfConstruct_IMPL
     NV_ASSERT_OK_OR_GOTO(status,
         pRmApi->MapToCpu(pRmApi,
                          pShared->hClient,
-                         pShared->hDevice,
+                         pShared->hDevice[pShared->memGpuIdx],
                          pShared->hSemaphoreMem,
                          0,
                          pShared->pSemaphoreMem->pMemDesc->Size,
@@ -745,6 +856,8 @@ semsurfConstruct_IMPL
                          0),
         ctorFailed);
 
+    pShared->slotCount = pShared->pSemaphoreMem->pMemDesc->Size / pShared->layout.size;
+
     pShared->pSem = KERNEL_POINTER_FROM_NvP64(NvU8 *, pShared->semKernAddr);
 
     if (!pShared->bIs64Bit)
@@ -765,7 +878,7 @@ semsurfConstruct_IMPL
             NV_ASSERT_OK_OR_GOTO(status,
                 pRmApi->MapToCpu(pRmApi,
                                  pShared->hClient,
-                                 pShared->hDevice,
+                                 pShared->hDevice[pShared->memGpuIdx],
                                  pShared->hMaxSubmittedMem,
                                  0,
                                  pShared->pMaxSubmittedMem->pMemDesc->Size,
@@ -773,6 +886,9 @@ semsurfConstruct_IMPL
                                  0),
                 ctorFailed);
 
+            pShared->slotCount =
+                NV_MIN(pShared->slotCount, pShared->pMaxSubmittedMem->pMemDesc->Size / pShared->layout.size);
+
             pShared->pMaxSubmitted =
                 KERNEL_POINTER_FROM_NvP64(NvU8 *, pShared->maxSubmittedKernAddr);
         }
@@ -806,8 +922,8 @@ semsurfConstruct_IMPL
                          goto ctorFailed);
     }
 
-
-    NV_ASSERT_OK_OR_GOTO(status, _semsurfRegisterCallback(pSemSurf), ctorFailed);
+    pShared->callback.func = _semsurfEventCallback;
+    pShared->callback.arg = pShared;
 
     for (i = 0; _semsurfValidateIndex(pShared, i); i++)
     {
@@ -840,6 +956,7 @@ semsurfDestruct_IMPL
     SEM_INDEX_LISTENERS_NODE *pIndexListeners;
     SEM_VALUE_LISTENERS_NODE *pValueListeners;
     SEM_VALUE_LISTENERS_NODE *pNextValueListeners;
+    SEM_CHANNEL_NODE *pChannelNode;
     EVENTNOTIFICATION *pListener;
     EVENTNOTIFICATION *pNextListener;
     NvU64 minWaitValue;
@@ -931,6 +1048,10 @@ semsurfDestruct_IMPL
     NV_PRINTF(LEVEL_INFO, "SemMem(0x%08x, 0x%08x): Exited spinlock\n",
               hSharedClient, hSharedMem);
 
+    while ((pChannelNode = mapFindGEQ(&pSemSurf->boundChannelMap, 0)))
+    {
+        (void)_semsurfUnbindChannel(pSemSurf, pChannelNode);
+    }
 skipRemoveListeners:
     NV_ASSERT(pShared->refCount > 0);
     --pShared->refCount;
@@ -1033,7 +1154,196 @@ semsurfCtrlCmdBindChannel_IMPL
     NV_SEMAPHORE_SURFACE_CTRL_BIND_CHANNEL_PARAMS *pParams
 )
 {
-    return NV_ERR_NOT_SUPPORTED;
+    SEM_NOTIFIER_NODE *pNotNode;
+    SEM_CHANNEL_NODE *pChannelNode;
+    OBJGPU *pGpu;
+    RsClient *pRsClient = staticCast(RES_GET_CLIENT(pSemSurf), RsClient);
+    KernelChannel *pKernelChannel;
+    NvHandle hEvent;
+    NvU64 mapKey;
+    NV_STATUS rmStatus = NV_OK;
+    NvU32 gpuIdx;
+    NvU32 i, j;
+
+    // The channel isn't actually used for anything here yet. However, validate
+    // it to ensure using it in the future (E.g., for robust channel error
+    // notification registration) won't break buggy clients.
+    NV_CHECK_OK_OR_RETURN(LEVEL_INFO, CliGetKernelChannel(pRsClient,
+                                                          pParams->hChannel,
+                                                          &pKernelChannel));
+
+    pGpu = GPU_RES_GET_GPU(pKernelChannel);
+
+    NV_CHECK_OR_RETURN(LEVEL_INFO, !IsSLIEnabled(pGpu), NV_ERR_INVALID_STATE);
+
+    gpuIdx = gpuGetInstance(pGpu);
+
+
+    if (pParams->numNotifyIndices > NV_ARRAY_ELEMENTS(pParams->notifyIndices))
+    {
+        return NV_ERR_INVALID_PARAMETER;
+    }
+
+    for (i = 0; i < pParams->numNotifyIndices; i++)
+    {
+        mapKey = ((NvU64)gpuIdx << 32) | pParams->notifyIndices[i];
+        pNotNode = mapFind(&pSemSurf->pShared->notifierMap, mapKey);
+
+        if (pNotNode != NULL)
+        {
+            NV_ASSERT(pNotNode->nUsers >= 1);
+
+            if (pNotNode->nUsers == NV_U32_MAX)
+            {
+                NV_PRINTF(LEVEL_WARNING,
+                          "SemSurf(0x%08x, 0x%08x): "
+                          "GPU instance 0x%08x notify index 0x%08x number of bound channels at max\n",
+                          RES_GET_CLIENT_HANDLE(pSemSurf),
+                          RES_GET_HANDLE(pSemSurf),
+                          gpuIdx, pParams->notifyIndices[i]);
+
+                rmStatus = NV_ERR_INVALID_STATE;
+                goto undoBound;
+            }
+
+            pNotNode->nUsers++;
+
+            NV_PRINTF(LEVEL_INFO,
+                      "SemSurf(0x%08x, 0x%08x): "
+                      "Bound to existing event for GPU instance 0x%08x notify index 0x%08x\n",
+                      RES_GET_CLIENT_HANDLE(pSemSurf),
+                      RES_GET_HANDLE(pSemSurf),
+                      gpuIdx, pParams->notifyIndices[i]);
+
+            continue;
+        }
+
+        NV_CHECK_OK_OR_GOTO(rmStatus,
+                            LEVEL_SILENT,
+                            _semsurfRegisterCallback(pSemSurf,
+                                                     pGpu,
+                                                     pParams->notifyIndices[i],
+                                                     &hEvent),
+                            undoBound);
+
+        pNotNode = portMemAllocNonPaged(sizeof(*pNotNode));
+        if (pNotNode == NULL)
+        {
+            NV_PRINTF(LEVEL_WARNING,
+                      "SemSurf(0x%08x, 0x%08x): "
+                      "Failed to allocate an event notifier map node\n",
+                      RES_GET_CLIENT_HANDLE(pSemSurf),
+                      RES_GET_HANDLE(pSemSurf));
+
+            _semsurfUnregisterCallback(pSemSurf->pShared, hEvent);
+            rmStatus = NV_ERR_NO_MEMORY;
+            goto undoBound;
+        }
+
+        pNotNode->hEvent = hEvent;
+        pNotNode->nUsers = 1;
+
+        if (!mapInsertExisting(&pSemSurf->pShared->notifierMap,
+                               mapKey,
+                               pNotNode))
+        {
+            NV_PRINTF(LEVEL_WARNING,
+                      "SemSurf(0x%08x, 0x%08x): "
+                      "Duplicate entry found for new event notifier map node\n",
+                      RES_GET_CLIENT_HANDLE(pSemSurf),
+                      RES_GET_HANDLE(pSemSurf));
+
+            _semsurfUnregisterCallback(pSemSurf->pShared, hEvent);
+            portMemFree(pNotNode);
+            rmStatus = NV_ERR_INVALID_STATE;
+            goto undoBound;
+        }
+
+        NV_PRINTF(LEVEL_INFO,
+                  "SemSurf(0x%08x, 0x%08x): "
+                  "Bound to new event for GPU instance 0x%08x notify index 0x%08x\n",
+                  RES_GET_CLIENT_HANDLE(pSemSurf), RES_GET_HANDLE(pSemSurf),
+                  gpuIdx, pParams->notifyIndices[i]);
+    }
+
+    pChannelNode = portMemAllocNonPaged(sizeof(*pChannelNode));
+    if (pChannelNode == NULL)
+    {
+        NV_PRINTF(LEVEL_WARNING,
+                  "SemSurf(0x%08x, 0x%08x): "
+                  "Failed to allocate an channel binding map node for channel 0x%08x\n",
+                  RES_GET_CLIENT_HANDLE(pSemSurf),
+                  RES_GET_HANDLE(pSemSurf),
+                  pParams->hChannel);
+
+        rmStatus = NV_ERR_NO_MEMORY;
+        goto undoBound;
+    }
+
+    pChannelNode->gpuIdx = gpuIdx;
+    pChannelNode->numNotifyIndices = pParams->numNotifyIndices;
+    (void)portMemCopy(pChannelNode->notifyIndices,
+                      pChannelNode->numNotifyIndices *
+                      sizeof(pChannelNode->notifyIndices[0]),
+                      pParams->notifyIndices,
+                      pParams->numNotifyIndices *
+                      sizeof(pParams->notifyIndices[0]));
+
+    if (!mapInsertExisting(&pSemSurf->boundChannelMap, pParams->hChannel, pChannelNode))
+    {
+        NV_PRINTF(LEVEL_WARNING,
+                  "SemSurf(0x%08x, 0x%08x): "
+                  "Attempt to register duplicate channel binding for channel 0x%08x\n",
+                  RES_GET_CLIENT_HANDLE(pSemSurf),
+                  RES_GET_HANDLE(pSemSurf),
+                  pParams->hChannel);
+
+        portMemFree(pChannelNode);
+        rmStatus = NV_ERR_INSERT_DUPLICATE_NAME;
+        goto undoBound;
+    }
+
+    return NV_OK;
+
+undoBound:
+    for (j = 0; j < i; j++)
+    {
+        _semsurfRemoveNotifyBinding(pSemSurf,
+                                    gpuIdx,
+                                    pParams->notifyIndices[j]);
+
+    }
+
+    return rmStatus;
+}
+
+NV_STATUS
+semsurfCtrlCmdUnbindChannel_IMPL
+(
+    SemaphoreSurface *pSemSurf,
+    NV_SEMAPHORE_SURFACE_CTRL_UNBIND_CHANNEL_PARAMS *pParams
+)
+{
+    RsClient *pRsClient = staticCast(RES_GET_CLIENT(pSemSurf), RsClient);
+    KernelChannel *pKernelChannel;
+    SEM_CHANNEL_NODE *pChannelNode;
+
+    // The channel isn't actually used for anything here yet. However, validate
+    // it to ensure using it in the future (E.g., for robust channel error
+    // notification registration) won't break buggy clients.
+    NV_CHECK_OK_OR_RETURN(LEVEL_INFO, CliGetKernelChannel(pRsClient,
+                                                          pParams->hChannel,
+                                                          &pKernelChannel));
+
+    if (pParams->numNotifyIndices > NV_ARRAY_ELEMENTS(pParams->notifyIndices))
+    {
+        return NV_ERR_INVALID_PARAMETER;
+    }
+
+    pChannelNode = mapFind(&pSemSurf->boundChannelMap, pParams->hChannel);
+    NV_CHECK_OR_RETURN(LEVEL_INFO, pChannelNode != NULL, NV_ERR_INVALID_OBJECT);
+
+    return _semsurfUnbindChannel(pSemSurf, pChannelNode);
 }
 
 NV_STATUS
@@ -1159,6 +1469,9 @@ _semsurfAddWaiter
         return NV_ERR_INVALID_STATE;
     }
 
+    NV_PRINTF(LEVEL_INFO, "SemMem(0x%08x, 0x%08x): Entering spinlock\n",
+              pSemSurf->pShared->hClient,
+              pSemSurf->pShared->hSemaphoreMem);
     portSyncSpinlockAcquire(pSemSurf->pShared->pSpinlock);
 
     pIndexListeners = mapFind(&pSemSurf->pShared->listenerMap, index);
@@ -1379,14 +1692,16 @@ _semsurfAddWaiter
     }
 
     portSyncSpinlockRelease(pSemSurf->pShared->pSpinlock);
+    NV_PRINTF(LEVEL_INFO, "SemMem(0x%08x, 0x%08x): Exited spinlock\n",
+              pSemSurf->pShared->hClient, pSemSurf->pShared->hSemaphoreMem);
 
     NV_PRINTF(LEVEL_INFO,
               "SemSurf(0x%08x, 0x%08x): "
               "Registered semaphore surface value listener at index %"
               NvU64_fmtu ", value %" NvU64_fmtu " current value %" NvU64_fmtu
-              " post-wait value %" NvU64_fmtu " notification: %" NvU64_fmtx "\n",
+              " post-wait value %" NvU64_fmtu " notification: " NvP64_fmt "\n",
               hClient, hSemaphoreSurf, index, waitValue, semValue, newValue,
-              (NvU64)notificationHandle);
+              notificationHandle);
 
     return rmStatus;
 
@@ -1412,6 +1727,8 @@ failureUnlock:
     }
 
     portSyncSpinlockRelease(pSemSurf->pShared->pSpinlock);
+    NV_PRINTF(LEVEL_INFO, "SemMem(0x%08x, 0x%08x): Exited spinlock\n",
+              pSemSurf->pShared->hClient, pSemSurf->pShared->hSemaphoreMem);
 
     // There's no point of going through the trouble of notifying the waiter in
     // this case, but it is worth immediately running the auto-update code here
@@ -1639,3 +1956,24 @@ semsurfCtrlCmdUnregisterWaiter_IMPL
 
     return rmStatus;
 }
+
+NvU64
+semsurfGetValue_IMPL
+(
+    SemaphoreSurface *pSemSurf,
+    NvU64 index
+)
+{
+    NV_ASSERT_OR_RETURN(_semsurfValidateIndex(pSemSurf->pShared, index), 0);
+    return _semsurfGetValue(pSemSurf->pShared, index);
+}
+
+NvBool
+semsurfValidateIndex_IMPL
+(
+    SemaphoreSurface *pSemSurf,
+    NvU64 index
+)
+{
+    return _semsurfValidateIndex(pSemSurf->pShared, index);
+}
