@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2018-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2018-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -688,6 +688,7 @@ gisubscriptionCtrlCmdExecPartitionsGet_IMPL
         pOutInfo->gpcCount = pMIGComputeInstance->resourceAllocation.gpcCount;
         pOutInfo->gfxGpcCount = pMIGComputeInstance->resourceAllocation.gfxGpcCount;
         pOutInfo->veidCount = pMIGComputeInstance->resourceAllocation.veidCount;
+
         pOutInfo->ceCount = kmigmgrCountEnginesOfType(&pMIGComputeInstance->resourceAllocation.engines,
                                                       RM_ENGINE_TYPE_COPY(0));
         pOutInfo->nvEncCount = kmigmgrCountEnginesOfType(&pMIGComputeInstance->resourceAllocation.engines,
@@ -819,6 +820,7 @@ gisubscriptionCtrlCmdExecPartitionsExport_IMPL
     portMemCopy(pParams->info.uuid, sizeof(pParams->info.uuid),
                 pMIGComputeInstance->uuid.uuid, sizeof(pMIGComputeInstance->uuid.uuid));
     pParams->info.sharedEngFlags = pMIGComputeInstance->sharedEngFlag;
+    pParams->info.gfxGpcCount    = pMIGComputeInstance->resourceAllocation.gfxGpcCount;
     pParams->info.veidOffset     = pMIGComputeInstance->resourceAllocation.veidOffset;
     pParams->info.veidCount      = pMIGComputeInstance->resourceAllocation.veidCount;
     pParams->info.smCount        = pMIGComputeInstance->resourceAllocation.smCount;
@@ -1152,7 +1154,7 @@ gisubscriptionCtrlCmdExecPartitionsGetProfileCapacity_IMPL
         NvU64 GPUInstancePseudoMask;
         NvU32 availableSpanCount;
         NvU32 totalSpanCount;
-        NvU32 veidStepSize;
+        NvU32 veidSizePerSpan;
         NvU32 veidSlotCount;
         NvU32 count;
         NvU32 i;
@@ -1160,7 +1162,7 @@ gisubscriptionCtrlCmdExecPartitionsGetProfileCapacity_IMPL
         NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
             kmigmgrGetComputeProfileFromSize(pGpu, pKernelMIGManager, pParams->computeSize, &profile));
         NV_ASSERT_OK_OR_RETURN(
-            kgrmgrGetVeidStepSize(pGpu, pKernelGraphicsManager, &veidStepSize));
+            kgrmgrGetVeidSizePerSpan(pGpu, pKernelGraphicsManager, &veidSizePerSpan));
 
         // Create a mask for VEIDs associated with this GPU instance
         veidMask = DRF_SHIFTMASK64(profile.veidCount - 1:0);
@@ -1172,13 +1174,13 @@ gisubscriptionCtrlCmdExecPartitionsGetProfileCapacity_IMPL
         availableSpanCount = 0;
         totalSpanCount = 0;
         count = 0;
-        for (i = pKernelMIGGpuInstance->resourceAllocation.veidOffset; i < GPUInstanceVeidEnd; i += veidStepSize)
+        for (i = pKernelMIGGpuInstance->resourceAllocation.veidOffset; i < GPUInstanceVeidEnd; i += veidSizePerSpan)
         {
             // Determine max correctly sized VEID segments
             if (((GPUInstanceFreeVeidMask >> i) & veidMask) == veidMask)
             {
                 pParams->availableSpans[availableSpanCount].lo = count;
-                pParams->availableSpans[availableSpanCount].hi = count + (profile.veidCount / veidStepSize) - 1;
+                pParams->availableSpans[availableSpanCount].hi = count + (profile.veidCount / veidSizePerSpan) - 1;
                 availableSpanCount++;
             }
 
@@ -1186,7 +1188,7 @@ gisubscriptionCtrlCmdExecPartitionsGetProfileCapacity_IMPL
             if (((GPUInstanceVeidMask >> i) & veidMask) == veidMask)
             {
                 pParams->totalSpans[totalSpanCount].lo = count;
-                pParams->totalSpans[totalSpanCount].hi = count + (profile.veidCount / veidStepSize) - 1;
+                pParams->totalSpans[totalSpanCount].hi = count + (profile.veidCount / veidSizePerSpan) - 1;
                 totalSpanCount++;
             }
 
@@ -1218,7 +1220,7 @@ gisubscriptionCtrlCmdGetUuid_IMPL
     NV_CHECK_OR_RETURN(LEVEL_ERROR, !pGPUInstanceSubscription->bDeviceProfiling,
                        NV_ERR_NOT_SUPPORTED);
 
-    portMemCopy(pParams->uuid, sizeof(pParams->uuid), 
+    portMemCopy(pParams->uuid, sizeof(pParams->uuid),
                 pGPUInstanceSubscription->pKernelMIGGpuInstance->uuid.uuid,
                 sizeof(pGPUInstanceSubscription->pKernelMIGGpuInstance->uuid.uuid));
 
