@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2022-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2022-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -29,9 +29,10 @@
 #include "gpu/gpu.h"
 #include "gpu/mem_mgr/mem_mgr.h"
 #include "nverror.h"
-#include "objtmr.h"
+#include "gpu/timer/objtmr.h"
 #include "gpu_mgr/gpu_mgr.h"
 #include "gpu/gpu_fabric_probe.h"
+#include "platform/sli/sli.h"
 
 /*!
  * @brief Check if ALI is supported for the given device
@@ -110,6 +111,39 @@ knvlinkValidateFabricBaseAddress_GH100
     return NV_OK;
 }
 
+/*!
+ * @brief   Validates fabric EGM base address.
+ *
+ * @param[in]  pGpu              OBJGPU pointer
+ * @param[in]  pKernelNvlink     KernelNvlink pointer
+ * @param[in]  fabricEgmBaseAddr Address to be validated
+ *
+ * @returns On success, NV_OK.
+ *          On failure, returns NV_ERR_XXX.
+ */
+NV_STATUS
+knvlinkValidateFabricEgmBaseAddress_GH100
+(
+    OBJGPU       *pGpu,
+    KernelNvlink *pKernelNvlink,
+    NvU64         fabricEgmBaseAddr
+)
+{
+    //
+    // Hopper SKUs will be paired with NVSwitches supporting 2K
+    // mapslots that can cover 512GB each. Make sure that the EGM fabric base
+    // address being used is valid to cover whole frame buffer.
+    //
+
+    // Check if fabric EGM address is aligned to mapslot size.
+    if (fabricEgmBaseAddr & (NVBIT64(39) - 1))
+    {
+        return NV_ERR_INVALID_ARGUMENT;
+    }
+
+    return NV_OK;
+}
+
 /*!
  * @brief Do post setup on nvlink peers
  *
@@ -320,15 +354,23 @@ NV_STATUS
 knvlinkLogAliDebugMessages_GH100
 (
     OBJGPU       *pGpu,
-    KernelNvlink *pKernelNvlink
+    KernelNvlink *pKernelNvlink,
+    NvBool        bFinal
 )
 {
-    NV2080_CTRL_NVLINK_GET_ERR_INFO_PARAMS *nvlinkErrInfoParams = portMemAllocNonPaged(sizeof(NV2080_CTRL_NVLINK_GET_ERR_INFO_PARAMS));
-    portMemSet(nvlinkErrInfoParams, 0, sizeof(NV2080_CTRL_NVLINK_GET_ERR_INFO_PARAMS));
-    nvlinkErrInfoParams->ErrInfoFlags |= NV2080_CTRL_NVLINK_ERR_INFO_FLAGS_ALI_STATUS;
+    NV2080_CTRL_NVLINK_GET_ERR_INFO_PARAMS *nvlinkErrInfoParams;
     NvU32         i;
     // This is a Physical, Hopper specific HAL for debug purposes.
-    NV_STATUS status = knvlinkExecGspRmRpc(pGpu, pKernelNvlink,
+    NV_STATUS status;
+
+    if (bFinal)
+        return NV_OK;
+
+    nvlinkErrInfoParams = portMemAllocNonPaged(sizeof(NV2080_CTRL_NVLINK_GET_ERR_INFO_PARAMS));
+    portMemSet(nvlinkErrInfoParams, 0, sizeof(NV2080_CTRL_NVLINK_GET_ERR_INFO_PARAMS));
+    nvlinkErrInfoParams->ErrInfoFlags |= NV2080_CTRL_NVLINK_ERR_INFO_FLAGS_ALI_STATUS;
+
+    status = knvlinkExecGspRmRpc(pGpu, pKernelNvlink,
                         NV2080_CTRL_CMD_NVLINK_GET_ERR_INFO,
                         (void *)nvlinkErrInfoParams,
                         sizeof(NV2080_CTRL_NVLINK_GET_ERR_INFO_PARAMS));
@@ -645,6 +687,82 @@ knvlinkClearUniqueFabricBaseAddress_GH100
     pKernelNvlink->fabricBaseAddr = NVLINK_INVALID_FABRIC_ADDR;
 }
 
+/*!
+ * @brief   Set unique EGM fabric base address for NVSwitch enabled systems.
+ *
+ * @param[in] pGpu              OBJGPU pointer
+ * @param[in] pKernelNvlink     KernelNvlink pointer
+ * @param[in] fabricEgmBaseAddr EGM Fabric Address to set
+ *
+ * @returns On success, sets unique EGM fabric address and returns NV_OK.
+ *          On failure, returns NV_ERR_XXX.
+ */
+NV_STATUS
+knvlinkSetUniqueFabricEgmBaseAddress_GH100
+(
+    OBJGPU       *pGpu,
+    KernelNvlink *pKernelNvlink,
+    NvU64         fabricEgmBaseAddr
+)
+{
+    NV_STATUS status = NV_OK;
+
+    status = knvlinkValidateFabricEgmBaseAddress_HAL(pGpu, pKernelNvlink,
+                                                  fabricEgmBaseAddr);
+    if (status != NV_OK)
+    {
+        NV_PRINTF(LEVEL_ERROR, "EGM Fabric base addr validation failed for GPU %x\n",
+                  pGpu->gpuInstance);
+        return status;
+    }
+
+    if (IsSLIEnabled(pGpu))
+    {
+        NV_PRINTF(LEVEL_ERROR,
+                  "Operation is unsupported on SLI enabled GPU %x\n",
+                  pGpu->gpuInstance);
+        return NV_ERR_NOT_SUPPORTED;
+    }
+
+    if (pKernelNvlink->fabricEgmBaseAddr == fabricEgmBaseAddr)
+    {
+        NV_PRINTF(LEVEL_INFO,
+                  "The same EGM fabric base addr is being re-assigned to GPU %x\n",
+                  pGpu->gpuInstance);
+        return NV_OK;
+    }
+
+    if (pKernelNvlink->fabricEgmBaseAddr != NVLINK_INVALID_FABRIC_ADDR)
+    {
+        NV_PRINTF(LEVEL_ERROR, "EGM Fabric base addr is already assigned to GPU %x\n",
+                  pGpu->gpuInstance);
+        return NV_ERR_STATE_IN_USE;
+    }
+
+    pKernelNvlink->fabricEgmBaseAddr = fabricEgmBaseAddr;
+
+    NV_PRINTF(LEVEL_INFO, "EGM Fabric base addr %llx is assigned to GPU %x\n",
+              pKernelNvlink->fabricEgmBaseAddr, pGpu->gpuInstance);
+
+    return NV_OK;
+}
+
+/*!
+ * @brief   Clear unique EGM fabric base address for NVSwitch enabled systems.
+ *
+ * @param[in] pGpu           OBJGPU pointer
+ * @param[in] pKernelNvlink  KernelNvlink pointer
+ */
+void
+knvlinkClearUniqueFabricEgmBaseAddress_GH100
+(
+    OBJGPU       *pGpu,
+    KernelNvlink *pKernelNvlink
+)
+{
+    pKernelNvlink->fabricEgmBaseAddr = NVLINK_INVALID_FABRIC_ADDR;
+}
+
 /*!
  * @brief   Check if system has enough active NVLinks and
  *          enough NVLink bridges
