@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -50,89 +50,18 @@
 #include "kernel/gpu/rc/kernel_rc.h"
 #include "platform/nbsi/nbsi_read.h"
 
-//
-// Functions to fill function stubs
-//
-static void initOSFunctionPointers(OBJOS *);
-
-//
-// Helper functions to assist the above functions
-//
-static void initMiscOSFunctionPointers(OBJOS *);
-static void initCommonMiscOSFunctionPointers(OBJOS *);
-static void initStubMiscOSFunctionPointers(OBJOS *);
-static void initWinNTStubOSFunctionPointers(OBJOS *);
-static void initMacOSCoreOSFunctionPointers(OBJOS *);
-static void initAPIOSFunctionPointers(OBJOS *);
-
 // Bug check code string common to all OS
 const char *ppOsBugCheckBugcodeStr[] = OS_BUG_CHECK_BUGCODE_STR;
 
 NV_STATUS
 constructObjOS(OBJOS *pOS)
 {
-    // Stub out function pointers
-    initOSFunctionPointers(pOS);
-
     // Now call the OS specific initialization
     osInitObjOS(pOS);
 
     return NV_OK;
 }
 
-static void
-initOSFunctionPointers(OBJOS *pOS)
-{
-    initMiscOSFunctionPointers(pOS);
-    initWinNTStubOSFunctionPointers(pOS);
-    initMacOSCoreOSFunctionPointers(pOS);
-    initAPIOSFunctionPointers(pOS);
-}
-
-static void
-initMiscOSFunctionPointers(OBJOS *pOS)
-{
-    initCommonMiscOSFunctionPointers(pOS);
-    initStubMiscOSFunctionPointers(pOS);
-}
-
-static void
-initCommonMiscOSFunctionPointers(OBJOS *pOS)
-{
-    // Common OS function pointers.
-}
-
-static void
-initStubMiscOSFunctionPointers(OBJOS *pOS)
-{
-    // Stubbed OS function pointers.
-    pOS->osPageArrayGetPhysAddr         = stubOsPageArrayGetPhysAddr;
-
-    pOS->osInternalReserveAllocCallback = stubOsInternalReserveAllocCallback;
-    pOS->osInternalReserveFreeCallback  = stubOsInternalReserveFreeCallback;
-}
-
-static void
-initWinNTStubOSFunctionPointers(OBJOS *pOS)
-{
-}
-
-static void
-initMacOSCoreOSFunctionPointers(OBJOS *pOS)
-{
-    pOS->osNv_rdcr4                      = stubOsnv_rdcr4;
-    pOS->osNv_rdxcr0                     = stubOsnv_rdxcr0;
-    pOS->osNv_cpuid                      = stubOsnv_cpuid;
-    pOS->osNv_rdmsr                      = stubOsnv_rdmsr;
-    pOS->osNv_wrmsr                      = stubOsnv_wrmsr;
-}
-
-static void
-initAPIOSFunctionPointers(OBJOS *pOS)
-{
-    pOS->osRmInitRm                      = osRmInitRm;
-}
-
 //
 // Function to find the maximum number of cores in the system
 //
@@ -300,6 +229,11 @@ void vgpuDevWriteReg032(
     NV_ASSERT_OK(kbifGetPciConfigSpacePriMirror_HAL(pGpu, GPU_GET_KERNEL_BIF(pGpu),
                                                     &configSpaceMirrorBase, &configSpaceMirrorSize));
 
+    if (IS_VIRTUAL_WITH_SRIOV(pGpu))
+    {
+        configSpaceSize = configSpaceMirrorSize;
+    }
+    else
     {
         configSpaceSize = NV_CONFIG_PCI_NV_12;
     }
@@ -371,6 +305,11 @@ NvU32 vgpuDevReadReg032(
     NV_ASSERT_OK(kbifGetPciConfigSpacePriMirror_HAL(pGpu, GPU_GET_KERNEL_BIF(pGpu),
                                                     &configSpaceMirrorBase, &configSpaceMirrorSize));
 
+    if (IS_VIRTUAL_WITH_SRIOV(pGpu))
+    {
+        configSpaceSize = configSpaceMirrorSize;
+    }
+    else
     {
         configSpaceSize = NV_CONFIG_PCI_NV_12;
     }
