@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -28,11 +28,12 @@
 #include "kernel/gpu/bif/kernel_bif.h"
 #include "gpu/subdevice/subdevice.h"
 #include "gpu/gpu.h"
+#include "virtualization/hypervisor/hypervisor.h"
 #include "vgpu/rpc.h"
 #include "vgpu/vgpu_events.h"
 #include "platform/chipset/chipset.h"
 #include "platform/p2p/p2p_caps.h"
-#include "nvRmReg.h"
+#include "nvrm_registry.h"
 #include "nvlimits.h"
 #include "nvdevid.h"
 
@@ -96,7 +97,7 @@ p2pGetCaps
     // The classes like NV50_P2P, NV50_THIRD_PARTY_P2P depends on direct P2P
     // connectivity, hence the check.
     //
-    if (!((connectivity == P2P_CONNECTIVITY_PCIE) ||
+    if (!((connectivity == P2P_CONNECTIVITY_PCIE_PROPRIETARY) ||
           (connectivity == P2P_CONNECTIVITY_PCIE_BAR1) ||
           (connectivity == P2P_CONNECTIVITY_NVLINK) ||
           (connectivity == P2P_CONNECTIVITY_C2C)))
@@ -225,6 +226,7 @@ _gpumgrGetP2PCapsStatusOverNvLink
                 {
                     NV_PRINTF(LEVEL_ERROR,
                             "Links failed to train for the given gpu pairs!\n");
+                    knvlinkLogAliDebugMessages(pFirstGpu, pKernelNvlink, NV_TRUE);
                     return status;
                 }
             }
@@ -345,6 +347,7 @@ _kp2pCapsGetStatusOverPcie
     NvU8 gpuP2PWriteCapsStatus = NV0000_P2P_CAPS_STATUS_OK;
     NvU32 lockedGpuMask = 0;
     NV_STATUS status = NV_OK;
+    OBJHYPERVISOR *pHypervisor = SYS_GET_HYPERVISOR(pSys);
 
     // Check if any overrides are enabled.
     if (_kp2pCapsCheckStatusOverridesForPcie(gpuMask, pP2PWriteCapStatus,
@@ -362,6 +365,24 @@ _kp2pCapsGetStatusOverPcie
         }
     }
 
+    // Check for hypervisor oriented PCIe P2P overrides
+    if (pHypervisor &&
+        pHypervisor->bDetected &&
+        hypervisorPcieP2pDetection(pHypervisor, gpuMask))
+    {
+        *pP2PReadCapStatus = NV0000_P2P_CAPS_STATUS_OK;
+        *pP2PWriteCapStatus = NV0000_P2P_CAPS_STATUS_OK;
+        goto done;
+    }
+
+    // If the PCIE topology cannot be accessed do not allow P2P
+    if (!pCl->ChipsetInitialized)
+    {
+        *pP2PReadCapStatus = NV0000_P2P_CAPS_STATUS_NOT_SUPPORTED;
+        *pP2PWriteCapStatus= NV0000_P2P_CAPS_STATUS_NOT_SUPPORTED;
+        goto done;
+    }
+
     // PCI-E topology checks
     gpuInstance = 0;
     while ((pGpu = gpumgrGetNextGpu(gpuMask, &gpuInstance)) != NULL)
@@ -760,7 +781,7 @@ p2pGetCapsStatus
             NvU8 bar1P2PWriteCapStatus = *pP2PWriteCapStatus;
             NvU8 bar1P2PReadCapStatus = *pP2PReadCapStatus;
 
-            *pConnectivity = P2P_CONNECTIVITY_PCIE;
+            *pConnectivity = P2P_CONNECTIVITY_PCIE_PROPRIETARY;
 
             if (_kp2pCapsGetStatusOverPcieBar1(gpuMask, &bar1P2PWriteCapStatus,
                     &bar1P2PReadCapStatus, bCommonSwitchFound) == NV_OK)
@@ -777,132 +798,6 @@ p2pGetCapsStatus
     return NV_OK;
 }
 
-static NV_STATUS
-_removeP2PPeerGpuCapsByGpuId
-(
-    OBJGPU *pGpu,
-    NvU32 peerGpuId
-)
-{
-    GPU_P2P_PEER_GPU_CAPS *pLocalPeerCaps = NULL;
-
-    pLocalPeerCaps = gpuFindP2PPeerGpuCapsByGpuId(pGpu, peerGpuId);
-    NV_CHECK_OR_RETURN(LEVEL_WARNING, pLocalPeerCaps != NULL, NV_ERR_OBJECT_NOT_FOUND);
-
-    // Swap the target element with the last element and remove the last.
-    // pLocalPeerCaps points to an item in the pGpu->P2PPeerGpuCaps array.
-    NV_ASSERT(pGpu->P2PPeerGpuCount != 0);
-    pGpu->P2PPeerGpuCount--;
-
-    portMemMove(pLocalPeerCaps,
-                sizeof(GPU_P2P_PEER_GPU_CAPS),
-                &pGpu->P2PPeerGpuCaps[pGpu->P2PPeerGpuCount],
-                sizeof(GPU_P2P_PEER_GPU_CAPS));
-
-    return NV_OK;
-}
-
-NV_STATUS
-subdeviceCtrlCmdInternalSetP2pCaps_IMPL
-(
-    Subdevice *pSubdevice,
-    NV2080_CTRL_INTERNAL_SET_P2P_CAPS_PARAMS *pParams
-)
-{
-    NV_STATUS status = NV_OK;
-    NvU32 i;
-    NvU32 failingIndex;
-    OBJGPU *pGpu = gpumgrGetGpuFromSubDeviceInst(pSubdevice->deviceInst,
-                                                 pSubdevice->subDeviceInst);
-
-    // TODO: GPUSWSEC-1433 remove this check to enable this control for baremetal
-    if (!IS_VGPU_GSP_PLUGIN_OFFLOAD_ENABLED(pGpu))
-        return NV_ERR_NOT_SUPPORTED;
-
-    NV_CHECK_OR_RETURN(LEVEL_ERROR,
-                       pParams->peerGpuCount <= NV_ARRAY_ELEMENTS(pGpu->P2PPeerGpuCaps),
-                       NV_ERR_INVALID_ARGUMENT);
-
-    for (i = 0; i < pParams->peerGpuCount; i++)
-    {
-        NV2080_CTRL_INTERNAL_SET_P2P_CAPS_PEER_INFO *pParamsPeerInfo = &pParams->peerGpuInfos[i];
-        GPU_P2P_PEER_GPU_CAPS *pLocalPeerCaps = NULL;
-
-        // Try to find the existing entry
-        pLocalPeerCaps = gpuFindP2PPeerGpuCapsByGpuId(pGpu, pParamsPeerInfo->gpuId);
-
-        // If no entry has been found, add a new one instead
-        if (pLocalPeerCaps == NULL)
-        {
-            NV_CHECK_OR_ELSE(LEVEL_ERROR,
-                             pGpu->P2PPeerGpuCount < NV_ARRAY_ELEMENTS(pGpu->P2PPeerGpuCaps),
-                             status = NV_ERR_INSUFFICIENT_RESOURCES; goto fail);
-
-            pLocalPeerCaps = &pGpu->P2PPeerGpuCaps[pGpu->P2PPeerGpuCount];
-            pLocalPeerCaps->peerGpuId = pParamsPeerInfo->gpuId;
-
-            pGpu->P2PPeerGpuCount++;
-        }
-
-        pLocalPeerCaps->peerGpuInstance = pParamsPeerInfo->gpuInstance;
-        pLocalPeerCaps->p2pCaps = pParamsPeerInfo->p2pCaps;
-        pLocalPeerCaps->p2pOptimalReadCEs = pParamsPeerInfo->p2pOptimalReadCEs;
-        pLocalPeerCaps->p2pOptimalWriteCEs = pParamsPeerInfo->p2pOptimalWriteCEs;
-        pLocalPeerCaps->busPeerId = pParamsPeerInfo->busPeerId;
-
-        portMemCopy(pLocalPeerCaps->p2pCapsStatus,
-                    sizeof(pLocalPeerCaps->p2pCapsStatus),
-                    pParamsPeerInfo->p2pCapsStatus,
-                    sizeof(pParamsPeerInfo->p2pCapsStatus));
-    }
-
-    goto done;
-
-fail:
-    // Remove the successfully set caps
-    failingIndex = i;
-    for (i = 0; i < failingIndex; i++)
-    {
-        NV_STATUS ignoredStatus;
-        NV_CHECK_OK(ignoredStatus, LEVEL_ERROR,
-                    _removeP2PPeerGpuCapsByGpuId(pGpu, pParams->peerGpuInfos[i].gpuId));
-    }
-
-done:
-    return status;
-}
-
-NV_STATUS
-subdeviceCtrlCmdInternalRemoveP2pCaps_IMPL
-(
-    Subdevice *pSubdevice,
-    NV2080_CTRL_INTERNAL_REMOVE_P2P_CAPS_PARAMS *pParams
-)
-{
-    NV_STATUS status = NV_OK;
-    NvU32 i;
-    OBJGPU *pGpu = gpumgrGetGpuFromSubDeviceInst(pSubdevice->deviceInst,
-                                                 pSubdevice->subDeviceInst);
-
-    // TODO: GPUSWSEC-1433 remove this check to enable this control for baremetal
-    if (!IS_VGPU_GSP_PLUGIN_OFFLOAD_ENABLED(pGpu))
-        return NV_ERR_NOT_SUPPORTED;
-
-    NV_CHECK_OR_RETURN(LEVEL_ERROR,
-                       pParams->peerGpuIdCount <= NV_ARRAY_ELEMENTS(pGpu->P2PPeerGpuCaps),
-                       NV_ERR_INVALID_ARGUMENT);
-
-    for (i = 0; i < pParams->peerGpuIdCount; i++)
-    {
-        // Capture only the first error here, as trying to remove all caps
-        // is the best effort here.
-        NV_CHECK_OK_OR_CAPTURE_FIRST_ERROR(status, LEVEL_ERROR,
-                  _removeP2PPeerGpuCapsByGpuId(pGpu, pParams->peerGpuIds[i]));
-    }
-
-    return status;
-}
-
 static NV_STATUS
 subdeviceGetP2pCaps_VIRTUAL
 (
@@ -1053,6 +948,7 @@ subdeviceCtrlCmdGetP2pCaps_IMPL
             pParamsPeerInfo->p2pOptimalReadCEs = 0;
             pParamsPeerInfo->p2pOptimalWriteCEs = 0;
             pParamsPeerInfo->busPeerId = NV0000_CTRL_SYSTEM_GET_P2P_CAPS_INVALID_PEER;
+            pParamsPeerInfo->busEgmPeerId = NV0000_CTRL_SYSTEM_GET_P2P_CAPS_INVALID_PEER;
 
             portMemSet(pParamsPeerInfo->p2pCapsStatus,
                        NV0000_P2P_CAPS_STATUS_NOT_SUPPORTED,
@@ -1064,6 +960,7 @@ subdeviceCtrlCmdGetP2pCaps_IMPL
             pParamsPeerInfo->p2pOptimalReadCEs = pLocalPeerCaps->p2pOptimalReadCEs;
             pParamsPeerInfo->p2pOptimalWriteCEs = pLocalPeerCaps->p2pOptimalWriteCEs;
             pParamsPeerInfo->busPeerId = pLocalPeerCaps->busPeerId;
+            pParamsPeerInfo->busEgmPeerId = pLocalPeerCaps->busEgmPeerId;
 
             portMemCopy(pParamsPeerInfo->p2pCapsStatus,
                         sizeof(pParamsPeerInfo->p2pCapsStatus),
