@@ -23,7 +23,7 @@
 
 /* ------------------------ Includes --------------------------------------- */
 #include "nvRmReg.h"
-#include "objtmr.h"
+#include "gpu/timer/objtmr.h"
 #include "platform/platform_request_handler.h"
 #include "gpu_mgr/gpu_mgr.h"
 #include "core/locks.h"
