@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -44,7 +44,7 @@
 #include "class/cl83de.h" // GT200_DEBUGGER
 #include "gpu/gr/kernel_sm_debugger_session.h"
 #include "kernel/gpu/rc/kernel_rc.h"
-#include "tmr.h"
+#include "gpu/timer/tmr.h"
 
 //
 // RM Alloc & Free internal flags -- code should be migrated to use rsresdesc
@@ -293,36 +293,6 @@ serverTopLock_Epilogue
     }
 }
 
-static NvU32
-_resGetBackRefGpusMask(RsResourceRef *pResourceRef)
-{
-    NvU32 gpuMask = 0x0;
-    RS_INTER_MAPPING_BACK_REF *pBackRefItem;
-
-    if (pResourceRef == NULL)
-    {
-        return 0x0;
-    }
-
-    pBackRefItem = listHead(&pResourceRef->interBackRefs);
-    while (pBackRefItem != NULL)
-    {
-        RsInterMapping *pMapping = pBackRefItem->pMapping;
-        RsResourceRef *pDeviceRef = pMapping->pContextRef;
-        GpuResource *pGpuResource = dynamicCast(pDeviceRef->pResource, GpuResource);
-
-        if (pGpuResource != NULL)
-        {
-            OBJGPU *pGpu = GPU_RES_GET_GPU(pGpuResource);
-            gpuMask |= gpumgrGetGpuMask(pGpu);
-        }
-
-        pBackRefItem = listNext(&pResourceRef->interBackRefs, pBackRefItem);
-    }
-
-    return gpuMask;
-}
-
 NV_STATUS
 serverResLock_Prologue
 (
@@ -440,15 +410,8 @@ serverResLock_Prologue
         }
         else
         {
-            //
-            // Lock the parent GPU and if specified any GPUs that resource
-            // may backreference via mappings.
-            //
-            pLockInfo->gpuMask = gpumgrGetGpuMask(pParentGpu) |
-                                 _resGetBackRefGpusMask(pLockInfo->pResRefToBackRef);
-
-            status = rmGpuGroupLockAcquire(0,
-                                           GPU_LOCK_GRP_MASK,
+            status = rmGpuGroupLockAcquire(pParentGpu->gpuInstance,
+                                           GPU_LOCK_GRP_DEVICE,
                                            GPUS_LOCK_FLAGS_NONE,
                                            RM_LOCK_MODULES_CLIENT,
                                            &pLockInfo->gpuMask);
@@ -1153,7 +1116,7 @@ rmapiFreeResourcePrologue
         tmrapiDeregisterEvents(pTimerApi);
     }
 
-    CliDelObjectEvents(pRmFreeParams->hClient, pRmFreeParams->hResource);
+    CliDelObjectEvents(pResourceRef);
 
     return NV_OK;
 }
@@ -1245,11 +1208,19 @@ rmapiAllocWithSecInfo
     // RS-TODO: Fix calls that use RMAPI_GPU_LOCK_INTERNAL without holding the API lock
     if (pRmApi->bGpuLockInternal && !rmapiLockIsOwner())
     {
-        NV_PRINTF(LEVEL_ERROR, "RMAPI_GPU_LOCK_INTERNAL alloc requested without holding the RMAPI lock\n");
+        // CORERM-6052 targets fixing the API lockless path for RTD3.
+        if (!rmapiInRtd3PmPath())
+        {
+            NV_PRINTF(LEVEL_ERROR,
+                "NVRM: %s: RMAPI_GPU_LOCK_INTERNAL alloc requested without holding the RMAPI lock: client:0x%x parent:0x%x object:0x%x class:0x%x\n",
+                __FUNCTION__, hClient, hParent, *phObject, hClass);
+        }
+
         pLockInfo->flags |= RM_LOCK_FLAGS_NO_API_LOCK;
         pLockInfo->state &= ~RM_LOCK_STATES_API_LOCK_ACQUIRED;
     }
 
+
     // This flag applies to both VGPU and GSP cases
     if (flags & RMAPI_ALLOC_FLAGS_SKIP_RPC)
         allocInitStates |= RM_ALLOC_STATES_SKIP_RPC;
@@ -1372,7 +1343,7 @@ resservResourceFactory
 
     if (pResDesc->internalClassId == classId(Device))
     {
-        if (pParams == NULL || pParams->pAllocParams == NULL)
+        if (pParams->pAllocParams == NULL)
             return NV_ERR_INVALID_ARGUMENT;
 
         NV0080_ALLOC_PARAMETERS *pNv0080AllocParams = pParams->pAllocParams;
@@ -1409,7 +1380,7 @@ resservResourceFactory
 
         if (pGpu && !IS_MIG_IN_USE(pGpu))
         {
-            rmapiControlCacheSetGpuInstForObject(pParams->hClient, pParams->hResource, pGpu->gpuInstance);
+            rmapiControlCacheSetGpuAttrForObject(pParams->hClient, pParams->hResource, pGpu);
         }
     }
 
@@ -1496,7 +1467,12 @@ rmapiFreeWithSecInfo
     // RS-TODO: Fix calls that use RMAPI_GPU_LOCK_INTERNAL without holding the API lock
     if (pRmApi->bGpuLockInternal && !rmapiLockIsOwner())
     {
-        NV_PRINTF(LEVEL_ERROR, "RMAPI_GPU_LOCK_INTERNAL free requested without holding the RMAPI lock\n");
+        // CORERM-6052 targets fixing the API lockless path for RTD3.
+        if (!rmapiInRtd3PmPath())
+        {
+            NV_PRINTF(LEVEL_ERROR, "RMAPI_GPU_LOCK_INTERNAL free requested without holding the RMAPI lock\n");
+        }
+
         lockInfo.flags |= RM_LOCK_FLAGS_NO_API_LOCK;
         lockInfo.state &= ~RM_LOCK_STATES_API_LOCK_ACQUIRED;
     }
