@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -30,7 +30,6 @@
 #include "rmapi/resource_fwd_decls.h"
 #include "core/locks.h"
 #include "core/system.h"
-#include "gpu/device/device.h"
 #include "resource_desc.h"
 #include "gpu_mgr/gpu_mgr.h"
 #include "gpu/gpu.h"
@@ -66,8 +65,13 @@ rmclientConstruct_IMPL
     API_SECURITY_INFO *pSecInfo = pParams->pSecInfo;
     OBJGPU            *pGpu = NULL;
 
+    //
     // RM client objects can only be created/destroyed with the RW API lock.
-    LOCK_ASSERT_AND_RETURN(rmapiLockIsWriteOwner());
+    // Bug 4193761 - allow internal clients to be created with the GPU lock,
+    // GR-2409 will remove the possible race condition with the client list.
+    //
+    LOCK_ASSERT_AND_RETURN(rmapiLockIsWriteOwner() ||
+        (serverIsClientInternal(&g_resServ, pRsClient->hClient) && rmGpuLockIsOwner()));
 
     if (RMCFG_FEATURE_PLATFORM_GSP)
     {
@@ -84,6 +88,7 @@ rmclientConstruct_IMPL
     pClient->pUserInfo       = NULL;
     pClient->pSecurityToken  = NULL;
     pClient->pOSInfo         = pSecInfo->clientOSInfo;
+    pClient->imexChannel     = -1;
 
     pClient->cachedPrivilege = pSecInfo->privLevel;
 
@@ -213,6 +218,9 @@ rmclientConstruct_IMPL
     if (status == NV_OK && pParams->pAllocParams != NULL)
         *(NvHandle*)(pParams->pAllocParams) = pParams->hClient;
 
+    NV_PRINTF(LEVEL_INFO, "New RM Client: hClient=0x%08x (%c), ProcID=%u, name='%s'\n",
+        pRsClient->hClient, (pRsClient->type == CLIENT_TYPE_USER) ? 'U' : 'K', pClient->ProcID, pClient->name);
+
 out:
     if (status != NV_OK)
     {
@@ -232,8 +240,15 @@ rmclientDestruct_IMPL
     NV_STATUS           status = NV_OK;
     NvBool              bReleaseLock = NV_FALSE;
 
+    //
     // RM client objects can only be created/destroyed with the RW API lock.
-    NV_ASSERT(rmapiLockIsWriteOwner());
+    // Bug 4193761 - allow internal clients to be created with the GPU lock,
+    // GR-2409 will remove the possible race condition with the client list.
+    //
+    NV_ASSERT_OR_ELSE(rmapiLockIsWriteOwner() ||
+        (serverIsClientInternal(&g_resServ, staticCast(pClient, RsClient)->hClient) &&
+         rmGpuLockIsOwner()),
+        return);
 
     NV_PRINTF(LEVEL_INFO, "    type: client\n");
 
@@ -315,13 +330,12 @@ rmclientInterMap_IMPL
     mapToParams.hMemoryDevice    = pPrivate->hMemoryDevice;
     mapToParams.gpuMask          = pPrivate->gpuMask;
     mapToParams.bSubdeviceHandleProvided = pPrivate->bSubdeviceHandleProvided;
-    mapToParams.bDmaMapNeeded    = pPrivate->bDmaMapNeeded;
     mapToParams.bFlaMapping      = pPrivate->bFlaMapping;
 
     return resMapTo(pMapperRef->pResource, &mapToParams);
 }
 
-void
+NV_STATUS
 rmclientInterUnmap_IMPL
 (
     RmClient *pClient,
@@ -339,13 +353,14 @@ rmclientInterUnmap_IMPL
     unmapFromParams.hMemory   = pParams->hMappable;
     unmapFromParams.flags     = pParams->flags;
     unmapFromParams.dmaOffset = pParams->dmaOffset;
+    unmapFromParams.size      = pParams->size;
 
     unmapFromParams.pGpu             = pPrivate->pGpu;
     unmapFromParams.hBroadcastDevice = pPrivate->hBroadcastDevice;
     unmapFromParams.gpuMask          = pPrivate->gpuMask;
     unmapFromParams.bSubdeviceHandleProvided = pPrivate->bSubdeviceHandleProvided;
 
-    resUnmapFrom(pMapperRef->pResource, &unmapFromParams);
+    return resUnmapFrom(pMapperRef->pResource, &unmapFromParams);
 }
 
 RS_PRIV_LEVEL
@@ -560,10 +575,15 @@ NvBool rmclientIsAdminByHandle(NvHandle hClient, RS_PRIV_LEVEL privLevel)
     return pClient ? rmclientIsAdmin(pClient, privLevel) : NV_FALSE;
 }
 
+static inline NvBool rmclientIsKernelOnly(RmClient *pClient)
+{
+    return (pClient->pSecurityToken == NULL);
+}
+
 NvBool rmclientIsKernelOnlyByHandle(NvHandle hClient)
 {
     RmClient *pClient = serverutilGetClientUnderLock(hClient);
-    return pClient ? (pClient->pSecurityToken == NULL) : NV_FALSE;
+    return (pClient ? rmclientIsKernelOnly(pClient) : NV_FALSE);
 }
 
 NvBool rmclientSetClientFlagsByHandle(NvHandle hClient, NvU32 clientFlags)
@@ -749,6 +769,43 @@ rmclientValidate_IMPL
     return status;
 }
 
+NV_STATUS
+rmclientValidateLocks_IMPL
+(
+    RmClient *pClient,
+    RsServer *pServer,
+    const CLIENT_ENTRY *pClientEntry
+)
+{
+    // Possessing the client lock means it's always safe to use this client object
+    if (pClientEntry->lockOwnerTid == portThreadGetCurrentThreadId())
+        return NV_OK;
+
+    //
+    // Without the client lock, the API lock in write mode guarantees safety for the
+    // client object since nothing else can execute in parallel when holding it.
+    //
+    if (rmapiLockIsWriteOwner())
+        return NV_OK;
+
+    //
+    // Without the client lock, the API lock in read mode guarantees safety for the
+    // client object IF it's a client that cannot be used directly by user space (i.e.
+    // kernel privileged client and/or internal client).
+    //
+    if (rmapiLockIsOwner() &&
+        (rmclientIsKernelOnly(pClient) ||
+         serverIsClientInternal(pServer, pClientEntry->hClient)))
+    {
+        return NV_OK;
+    }
+
+
+    NV_ASSERT(0);
+    // Otherwise we don't have the required locks to use this RM client
+    return NV_ERR_INVALID_LOCK_STATE;
+}
+
 NV_STATUS
 rmclientFreeResource_IMPL
 (
@@ -761,6 +818,7 @@ rmclientFreeResource_IMPL
     OBJGPU *pGpu;
     NvBool bBcState;
     NvBool bRestoreBcState = NV_FALSE;
+    RsClient *pRsClient = staticCast(pClient, RsClient);
 
     if (gpuGetByRef(pRmFreeParams->pResourceRef, NULL, &pGpu) == NV_OK)
     {
@@ -770,7 +828,17 @@ rmclientFreeResource_IMPL
 
     rmapiFreeResourcePrologue(pRmFreeParams);
 
-    status = clientFreeResource_IMPL(staticCast(pClient, RsClient), pServer, pRmFreeParams);
+    //
+    // In the RTD3 case, the API lock isn't taken since it can be initiated
+    // from another thread that holds the API lock and because we now hold
+    // the GPU lock.
+    //
+    if (rmapiInRtd3PmPath())
+    {
+        pRmFreeParams->pLockInfo->flags |= RM_LOCK_FLAGS_NO_API_LOCK;
+    }
+
+    status = clientFreeResource_IMPL(pRsClient, pServer, pRmFreeParams);
 
     if (bRestoreBcState)
     {
@@ -800,6 +868,11 @@ static NvBool _rmclientIsCapable
             internalClassId = classId(FmSessionApi);
             break;
         }
+        case NV_RM_CAP_SYS_FABRIC_IMEX_MGMT:
+        {
+            internalClassId = classId(ImexSessionApi);
+            break;
+        }
         case NV_RM_CAP_SYS_SMC_MONITOR:
         {
             internalClassId = classId(MIGMonitorSession);
