@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -242,17 +242,27 @@ memMap_IMPL
         }
     }
 
-    if ((pGpu != NULL) && (pMemoryInfo->Flags & NVOS32_ALLOC_FLAGS_PROTECTED))
+    if ((pGpu != NULL) && gpuIsCCFeatureEnabled(pGpu) &&
+        (pMemoryInfo->Flags & NVOS32_ALLOC_FLAGS_PROTECTED))
     {
         ConfidentialCompute *pCC = GPU_GET_CONF_COMPUTE(pGpu);
         //
         // If neither BAR1 nor PCIE as a whole is trusted, fail the mapping
         // for allocations in CPR region. Mapping should still succeed for
         // allocations in non-CPR region
+        // Deny BAR1 access to CPU-RM by default irrespective of prod or devtools
+        // mode. Some mappings made by CPU-RM may be allowed to go thorough in
+        // devtools mode.
+        // However, allow the mapping to go through on platforms where GSP-DMA
+        // is not present e.g. MODS. User may have also set a regkey to force
+        // BAR accesses.
         //
-        if ((pCC != NULL) && !pCC->ccStaticInfo.bIsBar1Trusted &&
-            !pCC->ccStaticInfo.bIsPcieTrusted)
+        if (((pCC != NULL) && !pCC->ccStaticInfo.bIsBar1Trusted &&
+            !pCC->ccStaticInfo.bIsPcieTrusted) ||
+            (IS_GSP_CLIENT(pGpu) && pMapParams->bKernel && !pKernelBus->bForceBarAccessOnHcc &&
+             FLD_TEST_DRF(OS33, _FLAGS, _ALLOW_MAPPING_ON_HCC, _NO, pMapParams->flags)))
         {
+            NV_PRINTF(LEVEL_ERROR, "BAR1 mapping to CPR vidmem not supported\n");
             NV_ASSERT(0);
             return NV_ERR_NOT_SUPPORTED;
         }
@@ -300,8 +310,7 @@ memMap_IMPL
         effectiveAddrSpace = ADDR_SYSMEM;
     }
 
-    bIsSysmem = (effectiveAddrSpace == ADDR_SYSMEM);
-    bIsSysmem = bIsSysmem || (effectiveAddrSpace == ADDR_EGM);
+    bIsSysmem = (effectiveAddrSpace == ADDR_SYSMEM) || (effectiveAddrSpace == ADDR_EGM);
 
     if (dynamicCast(pMemoryInfo, FlaMemory) != NULL)
     {
@@ -323,35 +332,16 @@ memMap_IMPL
         {
             if (pMapParams->bKernel)
             {
-                if (pMemDesc->_flags & MEMDESC_FLAGS_PHYSICALLY_CONTIGUOUS)
-                {
-                    NvP64 tempCpuPtr = kbusMapCoherentCpuMapping_HAL(pGpu, pKernelBus, pMemDesc);
-                    if (tempCpuPtr == NULL)
-                    {
-                        rmStatus = NV_ERR_GENERIC;
-                    }
-                    else
-                    {
-                        rmStatus = NV_OK;
-                        tempCpuPtr = NvP64_PLUS_OFFSET(tempCpuPtr, pMapParams->offset);
-                    }
-                    *pMapParams->ppCpuVirtAddr = tempCpuPtr;
-
-                    if (rmStatus != NV_OK)
-                        return rmStatus;
-                }
-                else
-                {
-                    rmStatus = osMapSystemMemory(pMemDesc,
-                                                 pMapParams->offset,
-                                                 pMapParams->length,
-                                                 pMapParams->bKernel,
-                                                 pMapParams->protect,
-                                                 pMapParams->ppCpuVirtAddr,
-                                                 &priv);
-                    if (rmStatus != NV_OK)
-                        return rmStatus;
-                }
+                rmStatus = kbusMapCoherentCpuMapping_HAL(pGpu,
+                                                         pKernelBus,
+                                                         pMemDesc,
+                                                         pMapParams->offset,
+                                                         pMapParams->length,
+                                                         pMapParams->protect,
+                                                         pMapParams->ppCpuVirtAddr,
+                                                         &priv);
+                if (rmStatus != NV_OK)
+                    return rmStatus;
             }
             else
             {
@@ -482,7 +472,7 @@ memMap_IMPL
                 pMemDesc = memdescGetMemDescFromGpu(pMemDesc, pGpu);
 
                 // WAR for Bug 3564398, need to allocate doorbell for windows differently
-                if (RMCFG_FEATURE_PLATFORM_WINDOWS_LDDM &&
+                if (RMCFG_FEATURE_PLATFORM_WINDOWS &&
                     memdescGetFlag(pMemDesc, MEMDESC_FLAGS_MAP_SYSCOH_OVER_BAR1))
                 {
                     busMapFbFlags |= BUS_MAP_FB_FLAGS_MAP_DOWNWARDS;
@@ -491,10 +481,10 @@ memMap_IMPL
                 (void) deviceGetByHandle(staticCast(pClient, RsClient),
                                          pMapParams->hDevice, &pDevice);
 
-                rmStatus = kbusMapFbAperture_HAL(pGpu, pKernelBus,
-                                                 pMemDesc, pMapParams->offset,
-                                                 &gpuVirtAddr, &gpuMapLength,
-                                                 busMapFbFlags, pDevice);
+                rmStatus = kbusMapFbApertureSingle(pGpu, pKernelBus,
+                                                   pMemDesc, pMapParams->offset,
+                                                   &gpuVirtAddr, &gpuMapLength,
+                                                   busMapFbFlags, pDevice);
             }
 
             if (rmStatus != NV_OK)
@@ -566,12 +556,12 @@ memMap_IMPL
     _rmMapMemory_pciFail:
             if (!kbusIsBar1PhysicalModeEnabled(pKernelBus))
             {
-                kbusUnmapFbAperture_HAL(pGpu,
-                                        pKernelBus,
-                                        pMemDesc,
-                                        gpuVirtAddr,
-                                        gpuMapLength,
-                                        BUS_MAP_FB_FLAGS_MAP_UNICAST);
+                kbusUnmapFbApertureSingle(pGpu,
+                                          pKernelBus,
+                                          pMemDesc,
+                                          gpuVirtAddr,
+                                          gpuMapLength,
+                                          BUS_MAP_FB_FLAGS_MAP_UNICAST);
     _rmMapMemory_busFail:
                 gpumgrSetBcEnabledStatus(pGpu, bcState);
             }
@@ -707,19 +697,9 @@ memUnmap_IMPL
 
         if (pCpuMapping->pPrivate->bKernel)
         {
-            if(pMemDesc->_flags & MEMDESC_FLAGS_PHYSICALLY_CONTIGUOUS)
-            {
-                NV_ASSERT(pMemDesc->_flags & MEMDESC_FLAGS_PHYSICALLY_CONTIGUOUS);
-                kbusUnmapCoherentCpuMapping_HAL(pGpu, pKernelBus, pMemDesc);
-            }
-            else
-            {
-                osUnmapSystemMemory(pMemDesc,
-                                    pCpuMapping->pPrivate->bKernel,
-                                    pCpuMapping->processId,
-                                    pCpuMapping->pLinearAddress,
-                                    pCpuMapping->pPrivate->pPriv);
-            }
+            kbusUnmapCoherentCpuMapping_HAL(pGpu, pKernelBus, pMemDesc,
+                                            pCpuMapping->pLinearAddress,
+                                            pCpuMapping->pPrivate->pPriv);
         }
 
         NV_PRINTF(LEVEL_INFO,
@@ -733,7 +713,8 @@ memUnmap_IMPL
         //
     }
     // System Memory case
-    else if ((pGpu == NULL) || ((memdescGetAddressSpace(pMemDesc) == ADDR_SYSMEM) &&
+    else if ((pGpu == NULL) || (((memdescGetAddressSpace(pMemDesc) == ADDR_SYSMEM) ||
+                                  (memdescGetAddressSpace(pMemDesc) == ADDR_EGM)) &&
                                  FLD_TEST_DRF(OS33, _FLAGS, _MAPPING, _DIRECT, pCpuMapping->flags)))
     {
         if (FLD_TEST_DRF(OS33, _FLAGS, _MAPPING, _DIRECT, pCpuMapping->flags))
@@ -758,11 +739,11 @@ memUnmap_IMPL
         if (!kbusIsBar1PhysicalModeEnabled(pKernelBus))
         {
             {
-                kbusUnmapFbAperture_HAL(pGpu, pKernelBus,
-                                        pMemory->pMemDesc,
-                                        pCpuMapping->pPrivate->gpuAddress,
-                                        pCpuMapping->pPrivate->gpuMapLength,
-                                        BUS_MAP_FB_FLAGS_MAP_UNICAST);
+                kbusUnmapFbApertureSingle(pGpu, pKernelBus,
+                                          pMemory->pMemDesc,
+                                          pCpuMapping->pPrivate->gpuAddress,
+                                          pCpuMapping->pPrivate->gpuMapLength,
+                                          BUS_MAP_FB_FLAGS_MAP_UNICAST);
             }
         }
     }
@@ -1145,6 +1126,17 @@ rmapiMapToCpuWithSecInfoV2
         return status;
     }
 
+    //
+    // In the RTD3 case, the API lock isn't taken since it can be initiated
+    // from another thread that holds the API lock and because we now hold
+    // the GPU lock.
+    //
+    if (rmapiInRtd3PmPath())
+    {
+        lockInfo.flags |= RM_LOCK_FLAGS_NO_API_LOCK;
+        lockInfo.state &= ~RM_LOCK_STATES_API_LOCK_ACQUIRED;
+    }
+
     LOCK_METER_DATA(MAPMEM, flags, 0, 0);
 
     // clear params for good measure
